@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
+   Copyright (C) 2007-2010 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -71,6 +71,7 @@ extern int errno;
 #include <lib_functions.h>
 #include <module.h>
 #include <conf.h>
+#include <addrmatch.h>
 
 
 /*
@@ -411,12 +412,13 @@ const char *
 check_int (const char *s)
 {
 	char *ptr;
+	long int unused;
 
 #if HAVE_ASSERT_H
 	assert (s != NULL);
 #endif
 
-	strtol (s, &ptr, 0);
+	unused = strtol (s, &ptr, 0);
 	while (*ptr != 0 && isspace (*ptr) != 0) {
 		ptr++;
 	}
@@ -453,6 +455,48 @@ check_dirname (const char *s)
 	return NULL;
 }
 
+/*
+ * Check if the provided string contain an valid address family (any, inet
+ * or inet6)
+ *
+ * Return:
+ *   NULL if yes or an error message if no
+ */
+const char *
+check_addrfamily (const char *s)
+{
+#if HAVE_ASSERT_H
+	assert (s != NULL);
+#endif
+
+	if (	   schedwi_strcasecmp (s, "any") == 0
+		|| schedwi_strcasecmp (s, "inet") == 0
+		|| schedwi_strcasecmp (s, "inet6") == 0)
+	{
+		return NULL;
+	}
+	return _("the parameter value should be `any', `inet' (use IPv4 only), or `inet6' (use IPv6 only)");
+}
+
+/*
+ * Check if the provided network list is properly built.
+ *
+ * Return:
+ *   NULL if yes or an error message if no
+ */
+const char *
+check_netlist (const char *s)
+{
+#if HAVE_ASSERT_H
+	assert (s != NULL);
+#endif
+
+	if (addr_match_list (NULL, s) != 0) {
+		return _("Inconsistent mask length in one of the address");
+	}
+	return NULL;
+}
+
 
 /*
  * strtok (see strtok(3)).
@@ -517,7 +561,7 @@ split_module (	const char *name, size_t name_len,
 	const char *mod_s;
 	char *mod_k, *s, *k;
 	size_t l;
-	
+
 #if HAVE_ASSERT_H
 	assert (   name != NULL && name_len > 0
 		&& mod_name != NULL && mod_key != NULL);
@@ -671,7 +715,7 @@ conf_init (const char *file_name, conf_param_t *params_array)
 		if (name_len == 0) {
 			/* ie. '= value' (missing parameter name) */
 			nb_errors++;
-			fprintf (stderr, _("%s: syntax error on line %d: "), 
+			fprintf (stderr, _("%s: syntax error on line %d: "),
 					file_name, num_line);
 			fputs (_("missing parameter name:\n"), stderr);
 			fputs (save_line, stderr);
@@ -686,7 +730,7 @@ conf_init (const char *file_name, conf_param_t *params_array)
 		if (*line != '=') {
 			/* ie. 'name  value' (missing '=') */
 			nb_errors++;
-			fprintf (stderr, _("%s: syntax error on line %d: "), 
+			fprintf (stderr, _("%s: syntax error on line %d: "),
 					file_name, num_line);
 			fputs (_("missing `=' sign between name and value:\n"),
 					stderr);
@@ -704,7 +748,7 @@ conf_init (const char *file_name, conf_param_t *params_array)
 		if (*line == '\0' || *line == '#' || *line == ';') {
 			/* ie. 'name =' (missing value) */
 			nb_errors++;
-			fprintf (stderr, _("%s: syntax error on line %d: "), 
+			fprintf (stderr, _("%s: syntax error on line %d: "),
 					file_name, num_line);
 			fputs (_("missing value:\n"), stderr);
 			fputs (save_line, stderr);
@@ -739,7 +783,7 @@ conf_init (const char *file_name, conf_param_t *params_array)
 				case 1:
 					nb_errors++;
 					fprintf (stderr,
-				_("%s: unknown parameter name on line %d:\n"), 
+				_("%s: unknown parameter name on line %d:\n"),
 						file_name, num_line);
 					fputs (save_line, stderr);
 					fputc ('\n', stderr);
@@ -761,7 +805,7 @@ conf_init (const char *file_name, conf_param_t *params_array)
 			if (err_msg != 0) {
 				nb_errors++;
 				fprintf (stderr,
-			_("%s: erroneous value parameter on line %d: "), 
+			_("%s: erroneous value parameter on line %d: "),
 					file_name, num_line);
 				fputs (err_msg, stderr);
 				fputs (":\n", stderr);
