@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
+   Copyright (C) 2007-2011 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -34,19 +34,32 @@ static conf_param_t params_clnt[] = {
 	{ "SYSLOG", SYSLOG_FACILITY, {0}, check_syslog_facility, 0, 0, 0},
 	{ "DATE_FORMAT", STRING, {0}, 0, 0, 0, SCHEDWI_DEFAULT_DATEFORMAT},
 	{ "PID_FILE", STRING, {0}, 0, 0, 0, SCHEDWI_DEFAULT_PIDFILE},
+	{ "PID_FILE_REGISTRAR", STRING, {0}, 0, 0, 0, SCHEDWI_DEFAULT_PIDFILE_REG},
 	{ "WORKLOAD_PURGE_COMPLETED", INT, {0}, check_int, 0, 0, SCHEDWI_DEFAULT_WORKLOAD_PURGE_COMPLETED},
 	{ "WORKLOAD_PURGE_FAILED", INT, {0}, check_int, 0, 0, SCHEDWI_DEFAULT_WORKLOAD_PURGE_FAILED},
+	{ "ALLOW_FROM", STRING, {0}, check_netlist, 0, 0, ""},
 	{ "SERVER_PORT", STRING, {0}, 0, 0, 0, SCHEDWI_DEFAULT_SRVPORT},
+	{ "REGISTRAR_PORT", STRING, {0}, 0, 0, 0, SCHEDWI_DEFAULT_REGPORT},
+	{ "ADDRESS_FAMILY", STRING, {0}, check_addrfamily, 0, 0, SCHEDWI_DEFAULT_ADDRESS_FAMILY},
 	{ "IFACE_LISTEN", STRING, {0}, 0, 0, 0, 0},
-	{ "SSLServerCertificateFile", STRING, {0}, 0, 0, 0, SCHEDWI_DEFAULT_SSL_SERVER_CERTIFICATE_FILE},
-	{ "SSLServerCertificateKeyFile", STRING, {0}, 0, 0, 0, SCHEDWI_DEFAULT_SSL_SERVER_CERTIFICATE_KEY_FILE},
-	{ "SSLQuickRandom", BOOL, {0}, check_bool, 0, 0, 0},
-	{ "DB_HOST", STRING, {0}, 0, 0, 0, 0},
-	{ "DB_USER", STRING, {0}, 0, 0, 0, 0},
-	{ "DB_PASSWD", STRING, {0}, 0, 0, 0, 0},
-	{ "DB_DATABASE", STRING, {0}, 0, 0, 0, 0},
-	{ "DB_PORT", INT, {0}, check_int, 0, 0, 0},
-	{ "DB_UNIXSOCKET", STRING, {0}, 0, 0, 0, 0},
+	{ "SSLCertificateFile", STRING, {0}, 0, 0, 0, SCHEDWI_DEFAULT_SSL_CERTIFICATE_FILE},
+	{ "SSLCertificateKeyFile", STRING, {0}, 0, 0, 0, SCHEDWI_DEFAULT_SSL_CERTIFICATE_KEY_FILE},
+	{ "SSLCACertificateFile", STRING, {0}, 0, 0, 0, SCHEDWI_DEFAULT_SSL_CA_CERTIFICATE_FILE},
+	{ "SSLCACertificateKeyFile", STRING, {0}, 0, 0, 0, SCHEDWI_DEFAULT_SSL_CA_CERTIFICATE_KEY_FILE},
+	{ "SSLCACRLFile", STRING, {0}, 0, 0, 0, SCHEDWI_DEFAULT_SSL_CA_CRL_FILE},
+	{ "SSLAgentCertificateDir", STRING, {0}, 0, 0, 0, SCHEDWI_DEFAULT_SSL_CLNT_CERTIFICATE_DIR},
+	{ "SSLAgentRequestDir", STRING, {0}, 0, 0, 0, SCHEDWI_DEFAULT_SSL_CLNT_REQUEST_DIR},
+	{ "SSLAgentSaveRequestDir", STRING, {0}, 0, 0, 0, SCHEDWI_DEFAULT_SSL_CLNT_SAVE_REQUEST_DIR},
+	{ "SSLQuickRandom", BOOL, {0}, check_bool, 0, 0, "1"},
+	{ "DBI_DRIVERDIR", STRING, {0}, 0, 0, 0, 0},
+	{ "DBI_DRIVERNAME", STRING, {0}, 0, 0, 0, SCHEDWI_DEFAULT_DBI_DRIVERNAME},
+	{ "DBI_USER", STRING, {0}, 0, 0, 0, 0},
+	{ "DBI_PASSWORD", STRING, {0}, 0, 0, 0, 0},
+	{ "DBI_DBDIR", STRING, {0}, 0, 0, 0, SCHEDWI_DEFAULT_DBI_DBDIR},
+	{ "DBI_DBHOSTNAME", STRING, {0}, 0, 0, 0, "localhost"},
+	{ "DBI_DBNAME", STRING, {0}, 0, 0, 0, SCHEDWI_DEFAULT_DBI_DBNAME},
+	{ "DBI_FREETDS_VERSION", STRING, {0}, 0, 0, 0, SCHEDWI_DEFAULT_DBI_FREETDS_VERSION},
+	{ "DBI_ENCODING", STRING, {0}, 0, 0, 0, 0},
 	{ "MODULE_DIR", FUNCTION, {0}, 0, module_path, 0, 0},
 	{ "MODULE_LOAD", FUNCTION, {0}, 0, module_load, 0, 0},
 	{ 0, INT, {0}, 0, 0, 0, 0}
