@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
+   Copyright (C) 2007-2010 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -39,8 +39,7 @@
  * Error callback function for the sql_status_set() function
  */
 static void
-sql_status_set_error_logger (	void *data, const char *msg,
-				unsigned int err_code)
+sql_status_set_error_logger (void *data, const char *msg, int err_code)
 {
 	if (msg != NULL) {
 		lwc_writeLog (LOG_ERR, msg);
@@ -57,7 +56,7 @@ sql_status_set_error_logger (	void *data, const char *msg,
  * sql_stat_fail() functions
  */
 static void
-sql_stat_error (void *data, const char *msg, unsigned int err_code)
+sql_stat_error (void *data, const char *msg, int err_code)
 {
 	if (msg != NULL) {
 		lwc_writeLog (LOG_ERR, msg);
@@ -95,7 +94,7 @@ job_status_set (unsigned long long int id,
 {
 	int j_status;
 	char id_str[25], *wl_str;
-	unsigned int ret;
+	int ret;
 
 	/* Update the job_status database table */
 	j_status = job_status_state_status2int (status);
