@@ -24,14 +24,14 @@
  *    // Using the syslog mechanism
  *    int n = 0;
  *    lwc_newLog ("my_program", 1, LOG_LOCAL0, NULL, NULL);
- *    lwc_writeLog (LOG_INFO, "sample information message number %d", n++); 
- *    lwc_writeLog (LOG_INFO, "sample information message number %d", n++); 
+ *    lwc_writeLog (LOG_INFO, "sample information message number %d", n++);
+ *    lwc_writeLog (LOG_INFO, "sample information message number %d", n++);
  *    lwc_closeLog ();
  *
- *    // Using a log file 
+ *    // Using a log file
  *    lwc_newLog (NULL, 0, 0, "/tmp/my_program.log", "%m/%d/%Y");
- *    lwc_writeLog (LOG_INFO, "sample information message number %d", n++); 
- *    lwc_writeLog (LOG_INFO, "sample information message number %d", n++); 
+ *    lwc_writeLog (LOG_INFO, "sample information message number %d", n++);
+ *    lwc_writeLog (LOG_INFO, "sample information message number %d", n++);
  *    lwc_closeLog ();
  */
 
@@ -321,7 +321,7 @@ lwc_vwriteLog (int level, const char *format, va_list ap)
 	}
 	else {
 #if HAVE_SYSLOG
-		syslog (level, buffer);
+		syslog (level, buffer, 0);
 #endif
 	}
 	pthread_mutex_unlock (&lock);
