@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
+   Copyright (C) 2007-2011 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -36,6 +36,10 @@
 #include <time.h>
 #endif
 
+#if HAVE_ARPA_INET_H
+#include <arpa/inet.h>
+#endif
+
 #if HAVE_ERRNO_H
 #include <errno.h>
 #endif
@@ -55,21 +59,27 @@ extern int errno;
 #include <gnutls/x509.h>
 #include <gcrypt.h>
 #include <pthread.h>
+
+#if GNUTLS_VERSION_NUMBER <= 0x020b00
 GCRY_THREAD_OPTION_PTHREAD_IMPL;
+#endif
 
 #include <lib_functions.h>
 #include <lwc_log.h>
 #include <net_utils_sock.h>
 #include <net_utils_ssl.h>
+#include <JSON_parser.h>
+#include <cert_utils.h>
 
 
 #define BUFFER_LEN 1024
-#define ENF_OF_REQUEST "\nbYe"
 
 
 /* Our certificate and its associated key file */
 static char *schedwisrv_crt = NULL;
 static char *schedwisrv_key = NULL;
+static char *schedwisrv_ca = NULL;
+static char *schedwisrv_crl = NULL;
 
 /* Diffie Hellman parameters */
 static gnutls_dh_params_t dh_params;
@@ -79,6 +89,63 @@ static gnutls_rsa_params_t rsa_params;
 
 static char params_set = 0;
 
+/* Wether GnuTLS has been initialized (1) or not (0) */
+static char gnutls_initialized = 0;
+
+
+/*
+ * Check that the hostnames defined in the provided certificate signing
+ * request (the string `s') matche at least one hostname in names.
+ *
+ * Return:
+ *    0 --> No match
+ *   >0 --> a match (the index+1 of the hostname in names)
+ *   -1 --> Error (a message has been logged)
+ */
+int
+check_string_request_names (char *s, const char *const *names)
+{
+	gnutls_x509_crq_t crq;
+	gnutls_datum_t dat;
+	int ret;
+
+#if HAVE_ASSERT_H
+	assert (s != NULL && names != NULL);
+#endif
+
+	ret = gnutls_x509_crq_init (&crq);
+	if (ret < 0) {
+		lwc_writeLog (LOG_ERR,
+			_("SSL: Cannot initialize the certificate request: %s"),
+			gnutls_strerror (ret));
+		return -1;
+	}
+
+	dat.data = (unsigned char *) s;
+	dat.size = schedwi_strlen (s);
+
+	ret = gnutls_x509_crq_import (crq, &dat, GNUTLS_X509_FMT_PEM);
+	if (	   ret < 0
+		&& gnutls_x509_crq_import (crq, &dat, GNUTLS_X509_FMT_DER) < 0)
+	{
+		if (ret == GNUTLS_E_BASE64_UNEXPECTED_HEADER_ERROR) {
+			lwc_writeLog (LOG_ERR,
+				_("SSL: Cannot import the certificate request from agent: Could not find a valid PEM header"));
+			gnutls_x509_crq_deinit (crq);
+			return -1;
+		}
+		lwc_writeLog (LOG_ERR,
+		_("SSL: Cannot import the certificate request from agent: %s"),
+			gnutls_strerror (ret));
+		gnutls_x509_crq_deinit (crq);
+		return -1;
+	}
+
+	ret = my_gnutls_x509_crt_check_hostname (crq, REQ, names);
+	gnutls_x509_crq_deinit (crq);
+	return ret;
+}
+
 
 /*
  * Verify the remote certificate
@@ -89,16 +156,13 @@ static char params_set = 0;
  */
 static int
 net_utils_ssl_check_certificate_client (gnutls_session_t *session,
-					char **names)
+					const char *const *names)
 {
 	int ret;
-	unsigned int status, cert_list_size, i, j;
+	unsigned int status, cert_list_size;
 	gnutls_x509_crt_t cert;
 	const gnutls_datum_t *cert_list;
 	time_t now;
-	char dnsname[512];
-	size_t dnsnamesize;
-	char found_dnsname;
 
 #if HAVE_ASSERT_H
 	assert (names != NULL && session != NULL);
@@ -108,26 +172,26 @@ net_utils_ssl_check_certificate_client (gnutls_session_t *session,
 	ret = gnutls_certificate_verify_peers2 (*session, &status);
 	if (ret < 0) {
 		lwc_writeLog (	LOG_ERR,
-	_("SSL: Failed to verify the cerficate of the remote client: %s"),
+	_("SSL: Failed to verify the cerficate of the remote agent: %s"),
 				gnutls_strerror (ret));
 		return -1;
 	}
 
 	if (status & GNUTLS_CERT_INVALID) {
 		lwc_writeLog (	LOG_ERR,
-		_("SSL: The certificate of the remote client is invalid."));
+		_("SSL: The certificate of the remote agent is invalid."));
 		return -1;
 	}
 
 	if (status & GNUTLS_CERT_SIGNER_NOT_FOUND) {
 		lwc_writeLog (	LOG_ERR,
-_("SSL: The certificate of the remote client does not have a known issuer."));
+_("SSL: The certificate of the remote agent does not have a known issuer."));
 		return -1;
 	}
 
 	if (status & GNUTLS_CERT_REVOKED) {
 		lwc_writeLog (	LOG_ERR,
-	_("SSL: The certificate of the remote client has been revoked."));
+	_("SSL: The certificate of the remote agent has been revoked."));
 		return -1;
 	}
 
@@ -143,7 +207,7 @@ _("SSL: The certificate of the remote client does not have a known issuer."));
 	cert_list = gnutls_certificate_get_peers (*session, &cert_list_size);
 	if (cert_list == NULL) {
 		lwc_writeLog (LOG_ERR,
-			_("SSL: No certificate found for the remote client."));
+			_("SSL: No certificate found for the remote agent."));
 		gnutls_x509_crt_deinit (cert);
 		return -1;
 	}
@@ -151,12 +215,12 @@ _("SSL: The certificate of the remote client does not have a known issuer."));
 	/* Only check the first certificate */
 	ret = gnutls_x509_crt_import (	cert, &cert_list[0],
 					GNUTLS_X509_FMT_PEM);
-	if (	   ret != 0
+	if (	   ret < 0
 		&& gnutls_x509_crt_import (	cert, &cert_list[0],
-						GNUTLS_X509_FMT_DER) != 0)
+						GNUTLS_X509_FMT_DER) < 0)
 	{
 		lwc_writeLog (	LOG_ERR,
-	_("SSL: Cannot retrieve the certificate of the remote client: %s"),
+	_("SSL: Cannot retrieve the certificate of the remote agent: %s"),
 				gnutls_strerror (ret));
 		gnutls_x509_crt_deinit (cert);
 		return -1;
@@ -166,14 +230,14 @@ _("SSL: The certificate of the remote client does not have a known issuer."));
 	now = time (NULL);
 	if (gnutls_x509_crt_get_expiration_time (cert) < now) {
 		lwc_writeLog (LOG_ERR,
-		_("SSL: The certificate of the remote client has expired."));
+		_("SSL: The certificate of the remote agent has expired."));
 		gnutls_x509_crt_deinit (cert);
 		return -1;
 	}
 
 	if (gnutls_x509_crt_get_activation_time (cert) > now) {
 		lwc_writeLog (	LOG_ERR,
-	_("SSL: The certificate of the remote client is not yet activated."));
+	_("SSL: The certificate of the remote agent is not yet activated."));
 		gnutls_x509_crt_deinit (cert);
 		return -1;
 	}
@@ -182,58 +246,57 @@ _("SSL: The certificate of the remote client does not have a known issuer."));
 	 * Check the hostname.  One could use the provided
 	 * gnutls_x509_crt_check_hostname() function.  However, we will check
 	 * all the provided names.
-	 * The following lines are then inspired from this function
-	 * in GNUTLS (see rfc2818_hostname.c)
 	 */
-	ret = 0;
-	found_dnsname = 0;
-	for (i = 0; ret >= 0; i++) {
-		dnsnamesize = sizeof (dnsname);
-		ret = gnutls_x509_crt_get_subject_alt_name (cert, i,
-						dnsname, &dnsnamesize, NULL);
-		if (ret == GNUTLS_SAN_DNSNAME) {
-			found_dnsname = 1;
-			for (j = 0; names[j] != NULL; j++) {
-				if (schedwi_strcasecmp (names[j],
-							dnsname) == 0)
-				{
-					gnutls_x509_crt_deinit (cert);
-					return 0;
-				}
-			}
-		}
-	}
-
-	if (found_dnsname == 0) {
-		dnsnamesize = sizeof (dnsname);
-		ret = gnutls_x509_crt_get_dn_by_oid (cert,
-						GNUTLS_OID_X520_COMMON_NAME,
-						0, 0,
-						dnsname, &dnsnamesize);
-		if (ret < 0) {
-			lwc_writeLog (LOG_ERR,
-_("SSL: Cannot retrieve the certificate hostname for the remote client: %s"),
-				gnutls_strerror (ret));
-			gnutls_x509_crt_deinit (cert);
-			return -1;
-		}
-		for (j = 0; names[j] != NULL; j++) {
-			if (schedwi_strcasecmp (names[j], dnsname) == 0) {
-				gnutls_x509_crt_deinit (cert);
-				return 0;
-			}
-		}
-	}
-
+	ret = my_gnutls_x509_crt_check_hostname (cert, CERT, names);
 	gnutls_x509_crt_deinit (cert);
+	if (ret != 0) {
+		return 0;
+	}
 	lwc_writeLog (LOG_ERR,
-_("SSL: The certificate hostname does not match the known names of the remote client."));
+_("SSL: The certificate hostname does not match the known names of the remote agent."));
 	return -1;
 }
 
 
 /*
- * Create a SSL session connected to a client
+ * Load the CRL file if it exists
+ *
+ * Return:
+ *   0 --> no error (CRL file does not exist or is a valid CRL file)
+ *  -1 --> Error in the CRL file (an error has been logged)
+ */
+static int
+load_crl (gnutls_certificate_credentials_t x509_cred, const char *crl_file)
+{
+	int ret;
+
+#if HAVE_ASSERT_H
+	assert (x509_cred != NULL);
+#endif
+
+	if (	   crl_file == NULL || crl_file[0] == '\0'
+		|| access (crl_file, F_OK) != 0)
+	{
+		return 0;
+	}
+
+	ret = gnutls_certificate_set_x509_crl_file (	x509_cred, crl_file,
+							GNUTLS_X509_FMT_PEM);
+	if (	   ret < 0
+		&& gnutls_certificate_set_x509_crl_file (x509_cred, crl_file,
+						GNUTLS_X509_FMT_DER) < 0)
+	{
+		lwc_writeLog (LOG_ERR,
+			_("SSL: Cannot load the provided CRL `%s': %s"),
+			crl_file, gnutls_strerror (ret));
+		return -1;
+	}
+	return 0;
+}
+
+
+/*
+ * Create a SSL session connected to an agent
  *
  * Return:
  *   0 on success
@@ -242,40 +305,60 @@ _("SSL: The certificate hostname does not match the known names of the remote cl
 int
 net_server_ssl (gnutls_session_t *session, int fd,
 		gnutls_certificate_credentials_t *trusted_cert_ptr,
-		char **names,
+		const char *const *names,
 		char *remote_crt, unsigned long int remote_crt_len)
 {
 	int ret;
 	gnutls_datum_t crt;
-	const int cert_type_priority[2] = { GNUTLS_CRT_X509, 0 };
 
 #if HAVE_ASSERT_H
-	assert (   session != NULL && trusted_cert_ptr != NULL
-		&& names != NULL && remote_crt != NULL && remote_crt_len > 0);
+	assert (session != NULL && trusted_cert_ptr != NULL && names != NULL);
 #endif
 
-	/* Trusted certificate (the client one).  Try in PEM and DER format */
 	ret = gnutls_certificate_allocate_credentials (trusted_cert_ptr);
 	if (ret != 0) {
 		lwc_writeLog (LOG_ERR,
-	_("SSL: Cannot initialize the credentials of the remote client: %s"),
+	_("SSL: Cannot initialize the credentials of the remote agent: %s"),
 			gnutls_strerror (ret));
 		return -1;
 	}
 
-	crt.data = (unsigned char *)remote_crt;
-	crt.size = remote_crt_len;
-	ret = gnutls_certificate_set_x509_trust_mem (*trusted_cert_ptr, &crt,
-							GNUTLS_X509_FMT_PEM);
-	if (	   ret < 0
-		&& gnutls_certificate_set_x509_trust_mem (*trusted_cert_ptr,
-						&crt, GNUTLS_X509_FMT_DER) < 0)
+	if (remote_crt_len > 0 && remote_crt != NULL && remote_crt[0] != '\0')
 	{
-		lwc_writeLog (LOG_ERR,
-	_("SSL: Cannot load the certificate of the remote client: %s"),
+		crt.data = (unsigned char *)remote_crt;
+		crt.size = remote_crt_len;
+		ret = gnutls_certificate_set_x509_trust_mem (*trusted_cert_ptr,
+							&crt,
+							GNUTLS_X509_FMT_PEM);
+		if (	   ret < 0
+			&& gnutls_certificate_set_x509_trust_mem (
+						*trusted_cert_ptr,
+						&crt, GNUTLS_X509_FMT_DER) < 0)
+		{
+			lwc_writeLog (LOG_ERR,
+	_("SSL: Cannot load the certificate of the remote agent: %s"),
 				gnutls_strerror (ret));
-		gnutls_certificate_free_credentials (*trusted_cert_ptr);
-		return -1;
+			gnutls_certificate_free_credentials (*trusted_cert_ptr);
+			return -1;
+		}
+	}
+	else {
+		ret = gnutls_certificate_set_x509_trust_file (
+							*trusted_cert_ptr,
+							schedwisrv_ca,
+							GNUTLS_X509_FMT_PEM);
+		if (	   ret <= 0
+			&& gnutls_certificate_set_x509_trust_file (
+						*trusted_cert_ptr,
+						schedwisrv_ca,
+						GNUTLS_X509_FMT_DER) <= 0)
+		{
+			lwc_writeLog (LOG_ERR,
+		_("SSL: Cannot load the provided CA certificat `%s': %s"),
+				schedwisrv_ca, gnutls_strerror (ret));
+			gnutls_certificate_free_credentials (*trusted_cert_ptr);
+			return -1;
+		}
 	}
 
 	/*
@@ -300,6 +383,11 @@ net_server_ssl (gnutls_session_t *session, int fd,
 		return -1;
 	}
 
+	if (load_crl (*trusted_cert_ptr, schedwisrv_crl) != 0) {
+		gnutls_certificate_free_credentials (*trusted_cert_ptr);
+		return -1;
+	}
+
 	gnutls_certificate_set_dh_params (*trusted_cert_ptr, dh_params);
 	gnutls_certificate_set_rsa_export_params (	*trusted_cert_ptr,
 							rsa_params);
@@ -308,7 +396,7 @@ net_server_ssl (gnutls_session_t *session, int fd,
 	ret = gnutls_init (session, GNUTLS_SERVER);
 	if (ret != 0) {
 		lwc_writeLog (LOG_ERR,
-	_("SSL: Cannot initialize the session with the remote client: %s"),
+	_("SSL: Cannot initialize the session with the remote agent: %s"),
 			gnutls_strerror (ret));
 		gnutls_certificate_free_credentials (*trusted_cert_ptr);
 		return -1;
@@ -316,7 +404,7 @@ net_server_ssl (gnutls_session_t *session, int fd,
 
 	/* Use default priority */
 	gnutls_set_default_priority (*session);
-	gnutls_certificate_type_set_priority (*session, cert_type_priority);
+	gnutls_priority_set_direct (*session, "NORMAL", NULL);
 
 	/* Put the x509 credentials to the current session */
 	ret = gnutls_credentials_set (	*session, GNUTLS_CRD_CERTIFICATE,
@@ -330,7 +418,7 @@ net_server_ssl (gnutls_session_t *session, int fd,
 		return -1;
 	}
 
-	/* The remote client certificate is required */
+	/* The remote agent certificate is required */
 	gnutls_certificate_server_set_request (*session, GNUTLS_CERT_REQUIRE);
 	gnutls_dh_set_prime_bits (*session, 1024);
 
@@ -343,7 +431,7 @@ net_server_ssl (gnutls_session_t *session, int fd,
 	} while (ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED);
 	if (ret < 0) {
 		lwc_writeLog (	LOG_ERR,
-	_("SSL: Connection handshake failed with the remote client: %s"),
+	_("SSL: Connection handshake failed with the remote agent: %s"),
 				gnutls_strerror (ret));
 		gnutls_bye (*session, GNUTLS_SHUT_RDWR);
 		gnutls_deinit (*session);
@@ -351,7 +439,7 @@ net_server_ssl (gnutls_session_t *session, int fd,
 		return -1;
 	}
 
-	/* Check the client name against the received certificate */
+	/* Check the agent name against the received certificate */
 	if (net_utils_ssl_check_certificate_client (session, names) != 0) {
 		gnutls_bye (*session, GNUTLS_SHUT_RDWR);
 		gnutls_deinit (*session);
@@ -374,7 +462,10 @@ net_destroy_ssl ()
 	if (schedwisrv_crt != NULL) {
 		free (schedwisrv_crt);
 	}
-	schedwisrv_key = schedwisrv_crt = NULL;
+	if (schedwisrv_ca != NULL) {
+		free (schedwisrv_ca);
+	}
+	schedwisrv_key = schedwisrv_crt = schedwisrv_ca = NULL;
 
 	if (params_set != 0) {
 		gnutls_rsa_params_deinit (rsa_params);
@@ -440,31 +531,21 @@ _("SSL: Failed to generate the parameters for the RSA algorithms: %s"),
 
 
 /*
- * Initialize the SSL layer
+ * Initialize the GnuTLS layer
  *
  * Return:
- *   0 --> No error
- *  -1 --> Error (a message has been logged by lwc_writeLog())
+ *   0 --> OK
+ *  -1 --> Error (a message has been logged)
  */
 int
-net_init_ssl (	const char *schedwisrv_crt_file,
-		const char *schedwisrv_key_file,
-		char quick_random)
+net_init_gnutls (char quick_random)
 {
 	int ret;
-	gnutls_certificate_credentials_t x509_cred;
-
-#if HAVE_ASSERT_H
-	assert (schedwisrv_crt_file != NULL && schedwisrv_key_file != NULL);
-#endif
 
 	/* For the support of threads in libgcrypt */
+#if GNUTLS_VERSION_NUMBER <= 0x020b00
 	gcry_control (GCRYCTL_SET_THREAD_CBS, &gcry_threads_pthread);
-
-	/* Fast random mode (less secure) */
-	if (quick_random != 0) {
-		gcry_control (GCRYCTL_ENABLE_QUICK_RANDOM, 0);
-	}
+#endif
 
 	/* GNUTLS init */
 	ret = gnutls_global_init ();
@@ -475,8 +556,44 @@ net_init_ssl (	const char *schedwisrv_crt_file,
 		return -1;
 	}
 
+	/* Fast random mode (less secure) */
+	if (quick_random != 0) {
+		gcry_control (GCRYCTL_ENABLE_QUICK_RANDOM, 0);
+	}
+
+	gnutls_initialized = 1;
+	return 0;
+}
+
+
+/*
+ * Initialize the SSL layer
+ *
+ * Return:
+ *   0 --> No error
+ *  -1 --> Error (a message has been logged by lwc_writeLog())
+ */
+int
+net_init_ssl (	const char *schedwisrv_crt_file,
+		const char *schedwisrv_key_file,
+		const char *schedwisrv_ca_file,
+		const char *schedwisrv_crl_file,
+		char quick_random)
+{
+	int ret;
+	gnutls_certificate_credentials_t x509_cred;
+
+#if HAVE_ASSERT_H
+	assert (   schedwisrv_crt_file != NULL && schedwisrv_key_file != NULL
+		&& schedwisrv_ca_file != NULL && schedwisrv_crl_file != NULL);
+#endif
+
+	if (gnutls_initialized == 0 && net_init_gnutls (quick_random) != 0) {
+		return -1;
+	}
+
 	/*
-	 * Try the load the provided certificate and its key to be sure that
+	 * Try to load the provided certificate and its key to be sure that
 	 * everything is fine
 	 */
 
@@ -512,6 +629,29 @@ _("SSL: Cannot load the provided certificat `%s' with the key `%s': %s"),
 		gnutls_global_deinit ();
 		return -1;
 	}
+
+	ret = gnutls_certificate_set_x509_trust_file (	x509_cred,
+							schedwisrv_ca_file,
+							GNUTLS_X509_FMT_PEM);
+	if (	   ret <= 0
+		&& gnutls_certificate_set_x509_trust_file (x509_cred,
+						schedwisrv_ca_file,
+						GNUTLS_X509_FMT_DER) <= 0)
+	{
+		lwc_writeLog (LOG_ERR,
+		_("SSL: Cannot load the provided CA certificat `%s': %s"),
+				schedwisrv_ca_file, gnutls_strerror (ret));
+		gnutls_certificate_free_credentials (x509_cred);
+		gnutls_global_deinit ();
+		return -1;
+	}
+
+	if (load_crl (x509_cred, schedwisrv_crl_file) != 0) {
+		gnutls_certificate_free_credentials (x509_cred);
+		gnutls_global_deinit ();
+		return -1;
+	}
+
 	gnutls_certificate_free_credentials (x509_cred);
 
 	/* Save the path to the provided files */
@@ -535,6 +675,34 @@ _("SSL: Cannot load the provided certificat `%s' with the key `%s': %s"),
 	}
 	strcpy (schedwisrv_key, schedwisrv_key_file);
 
+	schedwisrv_ca = (char *) malloc (
+				schedwi_strlen (schedwisrv_ca_file) + 1);
+	if (schedwisrv_ca == NULL) {
+		free (schedwisrv_key);
+		free (schedwisrv_crt);
+		schedwisrv_key = NULL;
+		schedwisrv_crt = NULL;
+		gnutls_global_deinit ();
+		lwc_writeLog (LOG_CRIT, _("Memory allocation error"));
+		return -1;
+	}
+	strcpy (schedwisrv_ca, schedwisrv_ca_file);
+
+	schedwisrv_crl = (char *) malloc (
+				schedwi_strlen (schedwisrv_crl_file) + 1);
+	if (schedwisrv_crl == NULL) {
+		free (schedwisrv_ca);
+		free (schedwisrv_key);
+		free (schedwisrv_crt);
+		schedwisrv_ca = NULL;
+		schedwisrv_key = NULL;
+		schedwisrv_crt = NULL;
+		gnutls_global_deinit ();
+		lwc_writeLog (LOG_CRIT, _("Memory allocation error"));
+		return -1;
+	}
+	strcpy (schedwisrv_crl, schedwisrv_crl_file);
+
 	return 0;
 }
 
@@ -551,14 +719,12 @@ net_utils_ssl_check_certificate (	gnutls_session_t *session,
 					const char *port_number,
 					const char *hostname)
 {
-	int ret, i;
+	int ret;
 	unsigned int status, cert_list_size;
 	gnutls_x509_crt_t cert;
 	const gnutls_datum_t *cert_list;
 	time_t now;
-	char dnsname[512];
-	size_t dnsnamesize;
-	char found_dnsname;
+	const char *name[2];
 
 #if HAVE_ASSERT_H
 	assert (port_number != NULL && hostname != NULL && session != NULL);
@@ -644,7 +810,21 @@ _("SSL: The certificate for %s (port %s) does not have a known issuer."),
 		return -1;
 	}
 
-	/* Log a message if the certificate expires in less than a month */
+	/* Check the hostname */
+	name[0] = hostname;
+	name[1] = NULL;
+	if (my_gnutls_x509_crt_check_hostname (cert, CERT, name) == 0) {
+		lwc_writeLog (LOG_ERR,
+	_("SSL: The certificate hostname does not match %s (port %s)."),
+			hostname, port_number);
+		gnutls_x509_crt_deinit (cert);
+		return -1;
+	}
+
+	/*
+	 * Log an information message if the certificate expires in less
+	 * than a month
+	 */
 	if (gnutls_x509_crt_get_expiration_time (cert)
 			< now + 30 * 24 * 60 * 60)
 	{
@@ -653,52 +833,8 @@ _("SSL: The certificate for %s (port %s) is going to expire in less than a month
 				hostname, port_number);
 	}
 
-	/*
-	 * Check the hostname.  One could use the provided
-	 * gnutls_x509_crt_check_hostname() function.  However, we will check
-	 * all the known names of the host (from the DNS, /etc/hosts, ...)
-	 * The following lines are then inspired from this function
-	 * in GNUTLS (see rfc2818_hostname.c)
-	 */
-	found_dnsname = 0;
-	for (i = ret = 0; ret >= 0; i++) {
-		dnsnamesize = sizeof (dnsname);
-		ret = gnutls_x509_crt_get_subject_alt_name (cert, i,
-						dnsname, &dnsnamesize, NULL);
-		if (ret == GNUTLS_SAN_DNSNAME) {
-			found_dnsname = 1;
-			if (hostname_cmp (hostname, dnsname) == 0) {
-				gnutls_x509_crt_deinit (cert);
-				return 0;
-			}
-		}
-	}
-
-	if (found_dnsname == 0) {
-		dnsnamesize = sizeof (dnsname);
-		ret = gnutls_x509_crt_get_dn_by_oid (cert,
-						GNUTLS_OID_X520_COMMON_NAME,
-						0, 0,
-						dnsname, &dnsnamesize);
-		if (ret < 0) {
-			lwc_writeLog (LOG_ERR,
-_("SSL: Cannot retrieve the certificate hostname for %s (port %s): %s"),
-				hostname, port_number,
-				gnutls_strerror (ret));
-			gnutls_x509_crt_deinit (cert);
-			return -1;
-		}
-		if (hostname_cmp (hostname, dnsname) == 0) {
-			gnutls_x509_crt_deinit (cert);
-			return 0;
-		}
-	}
-
 	gnutls_x509_crt_deinit (cert);
-	lwc_writeLog (	LOG_ERR,
-	_("SSL: The certificate hostname does not match %s (port %s)."),
-			hostname, port_number);
-	return -1;
+	return 0;
 }
 
 
@@ -719,12 +855,10 @@ net_client_ssl (const char *port_number, const char *hostname,
 {
 	int ret;
 	gnutls_datum_t crt;
-	const int cert_type_priority[2] = { GNUTLS_CRT_X509, 0 };
 
 #if HAVE_ASSERT_H
 	assert (   port_number != NULL && hostname != NULL
-		&& session != NULL && fd != NULL && trusted_cert_ptr != NULL
-		&& remote_crt != NULL && remote_crt_len > 0);
+		&& session != NULL && fd != NULL && trusted_cert_ptr != NULL);
 #endif
 
 	/* Trusted certificate (the server one).  Try in PEM and DER format */
@@ -737,20 +871,43 @@ net_client_ssl (const char *port_number, const char *hostname,
 		return -1;
 	}
 
-	crt.data = (unsigned char *)remote_crt;
-	crt.size = remote_crt_len;
-	ret = gnutls_certificate_set_x509_trust_mem (*trusted_cert_ptr, &crt,
-							GNUTLS_X509_FMT_PEM);
-	if (	   ret < 0
-		&& gnutls_certificate_set_x509_trust_mem (*trusted_cert_ptr,
-						&crt, GNUTLS_X509_FMT_DER) < 0)
+	if (remote_crt_len > 0 && remote_crt != NULL && remote_crt[0] != '\0')
 	{
-		lwc_writeLog (LOG_ERR,
+		crt.data = (unsigned char *)remote_crt;
+		crt.size = remote_crt_len;
+		ret = gnutls_certificate_set_x509_trust_mem (*trusted_cert_ptr,
+							&crt,
+							GNUTLS_X509_FMT_PEM);
+		if (	   ret < 0
+			&& gnutls_certificate_set_x509_trust_mem (
+						*trusted_cert_ptr,
+						&crt, GNUTLS_X509_FMT_DER) < 0)
+		{
+			lwc_writeLog (LOG_ERR,
 	_("SSL: Cannot load the certificate for %s (port %s): %s"),
 				hostname, port_number,
 				gnutls_strerror (ret));
-		gnutls_certificate_free_credentials ( *trusted_cert_ptr);
-		return -1;
+			gnutls_certificate_free_credentials (*trusted_cert_ptr);
+			return -1;
+		}
+	}
+	else {
+		ret = gnutls_certificate_set_x509_trust_file (
+							*trusted_cert_ptr,
+							schedwisrv_ca,
+							GNUTLS_X509_FMT_PEM);
+		if (	   ret <= 0
+			&& gnutls_certificate_set_x509_trust_file (
+						*trusted_cert_ptr,
+						schedwisrv_ca,
+						GNUTLS_X509_FMT_DER) <= 0)
+		{
+			lwc_writeLog (LOG_ERR,
+		_("SSL: Cannot load the provided CA certificat `%s': %s"),
+				schedwisrv_ca, gnutls_strerror (ret));
+			gnutls_certificate_free_credentials (*trusted_cert_ptr);
+			return -1;
+		}
 	}
 
 	/*
@@ -775,6 +932,12 @@ net_client_ssl (const char *port_number, const char *hostname,
 		return -1;
 	}
 
+	if (load_crl (*trusted_cert_ptr, schedwisrv_crl) != 0) {
+		gnutls_certificate_free_credentials (*trusted_cert_ptr);
+		return -1;
+	}
+
+
 	/* Initialize the session */
 	ret = gnutls_init (session, GNUTLS_CLIENT);
 	if (ret != 0) {
@@ -788,7 +951,7 @@ net_client_ssl (const char *port_number, const char *hostname,
 
 	/* Use default priority */
 	gnutls_set_default_priority (*session);
-	gnutls_certificate_type_set_priority (*session, cert_type_priority);
+	gnutls_priority_set_direct (*session, "NORMAL", NULL);
 
 	/* Put the x509 credentials to the current session */
 	ret = gnutls_credentials_set (	*session, GNUTLS_CRD_CERTIFICATE,
@@ -844,6 +1007,33 @@ _("SSL: Cannot associate certificate with the session for %s (port %s): %s"),
 }
 
 
+/*
+ * JSON callback function
+ */
+static int
+json_char_callback (void* ctx, int type, const JSON_value* value)
+{
+	int *nb_open_brackets = (int *)ctx;
+
+	switch (type) {
+		case JSON_T_ARRAY_BEGIN:
+		case JSON_T_OBJECT_BEGIN:
+			if (*nb_open_brackets < 0) {
+				*nb_open_brackets = 1;
+			}
+			else {
+				(*nb_open_brackets)++;
+			}
+			break;
+		case JSON_T_ARRAY_END:
+		case JSON_T_OBJECT_END:
+			(*nb_open_brackets)--;
+			break;
+	}
+	return 1;
+}
+
+
 /*
  * Read data from the provided SSL session handler.
  * The data are store in *buff (of size *len). If buffer is NULL or not big
@@ -856,11 +1046,15 @@ _("SSL: Cannot associate certificate with the session for %s (port %s): %s"),
  *       lwc_writeLog())
  */
 ssize_t
-net_read_ssl (gnutls_session_t session, char **buff, unsigned int *len)
+net_read_ssl (gnutls_session_t session, char **buff, size_t *len)
 {
 	ssize_t nb_read, total;
 	size_t to_read;
 	char *b, *tmp;
+	int nb_open_brackets, i;
+	JSON_config config;
+	struct JSON_parser_struct *jc;
+
 
 #if HAVE_ASSERT_H
 	assert (buff != NULL && len != NULL);
@@ -876,12 +1070,25 @@ net_read_ssl (gnutls_session_t session, char **buff, unsigned int *len)
 		*len = BUFFER_LEN;
 	}
 
+	/* JSON parser initialization */
+	init_JSON_config (&config);
+
+	nb_open_brackets              = -1;
+	config.depth                  = 20;
+	config.callback               = &json_char_callback;
+	config.allow_comments         = 0;
+	config.handle_floats_manually = 0;
+	config.callback_ctx           = &nb_open_brackets;
+
+	jc = new_JSON_parser (&config);
+
 	total = 0;
 	to_read = *len;
 	b = *buff;
 	do {
 		nb_read = gnutls_record_recv (session, b, to_read);
 		if (nb_read == 0) {
+			delete_JSON_parser (jc);
 			return total;
 		}
 
@@ -892,6 +1099,7 @@ net_read_ssl (gnutls_session_t session, char **buff, unsigned int *len)
 				lwc_writeLog (LOG_ERR,
 					_("Error while reading data: %s"),
 					gnutls_strerror (nb_read));
+				delete_JSON_parser (jc);
 				return nb_read;
 			}
 			continue;
@@ -899,10 +1107,22 @@ net_read_ssl (gnutls_session_t session, char **buff, unsigned int *len)
 
 		total += nb_read;
 
-		/* Find the EOF tag */
-		tmp = schedwi_strnstr (*buff, ENF_OF_REQUEST, total);
-		if (tmp != NULL) {
-			return (ssize_t)(tmp - *buff);
+		/*
+		 * Count the numbers of open/close brackets.  If this number
+		 * drops to zero it means that the full JSON request has
+		 * been receved.
+		 */
+		for (i = 0; i < nb_read; i++) {
+			if (! JSON_parser_char (jc, b[i])) {
+				lwc_writeLog (  LOG_ERR,
+					_("Syntax error in received data"));
+				delete_JSON_parser (jc);
+				return -1;
+			}
+			if (nb_open_brackets == 0) {
+				delete_JSON_parser (jc);
+				return total;
+			}
 		}
 
 		if (nb_read < to_read) {
@@ -914,6 +1134,7 @@ net_read_ssl (gnutls_session_t session, char **buff, unsigned int *len)
 				/* Too many data */
 				lwc_writeLog (LOG_NOTICE,
 		_("Failed to read the network request: buffer overflow"));
+				delete_JSON_parser (jc);
 				return -2;
 			}
 
@@ -921,6 +1142,7 @@ net_read_ssl (gnutls_session_t session, char **buff, unsigned int *len)
 			if (tmp == NULL) {
 				lwc_writeLog (	LOG_CRIT,
 						_("Memory allocation error"));
+				delete_JSON_parser (jc);
 				return -1;
 			}
 			*buff = tmp;
@@ -930,6 +1152,8 @@ net_read_ssl (gnutls_session_t session, char **buff, unsigned int *len)
 		}
 	} while (1);
 
+	delete_JSON_parser (jc);
+
 	return total;
 }
 
