@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
+   Copyright (C) 2007-2010 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -17,7 +17,7 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-/* request.h -- Common functions to build a request to a client */
+/* request.h -- Common functions to build a request to a agent */
 
 #ifndef _REQUEST_H
 #define _REQUEST_H
@@ -34,12 +34,12 @@ extern "C" {
 
 extern int request_job_init OF((int workload_date,
 				unsigned long long int job_id,
-				char **host_id_out,
+				unsigned long long int *host_id_out,
 				startjob_request_ptr *request_out,
 				lwc_LL **hierarchy_list_out));
 extern int request_testfile_init OF((int workload_date,
 				unsigned long long int job_id,
-				const char *host_id_str,
+				unsigned long long int host_id,
 				const char *filename,
 				startjob_request_ptr *request_out));
 
