@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
+   Copyright (C) 2007-2010 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -67,7 +67,7 @@ extern int errno;
  * sql_job_get_retries_interval() and sql_status_get () functions
  */
 static void
-sql_get_error_logger (void *data, const char *msg, unsigned int err_code)
+sql_get_error_logger (void *data, const char *msg, int err_code)
 {
 	if (msg != NULL) {
 		lwc_writeLog (LOG_ERR, msg);
@@ -79,6 +79,142 @@ sql_get_error_logger (void *data, const char *msg, unsigned int err_code)
 }
 
 
+/*
+ * Retrieve the parameters of a job from the database
+ *
+ * Return:
+ *   0 --> No error
+ *  -1 --> Error (an error message has already been logged by this function)
+ */
+static int
+set_schedwi_jobtree_node (	schedwi_jobtree_node_ptr ptr,
+				int workload_date,
+				schedwi_date job_date,
+				int job_time,
+				lwc_LL *hierarchy_list)
+{
+    short int time_limit;
+    int current_status;
+    long int start_time;
+    lwc_LL *rows;
+    row_item_t *row;
+    int n, i, j;
+    char *filename;
+    schedwi_jobtree_file_ptr constraint_files;
+
+#if HAVE_ASSERT_H
+    assert (ptr != NULL && hierarchy_list != NULL);
+#endif
+
+    /* Retrieve parameters from the database */
+    if (sql_job_get_start_limit (workload_date, hierarchy_list, &time_limit,
+				sql_get_error_logger, NULL) != 0)
+    {
+	return -1;
+    }
+    ptr->start_limit = time_limit * 60; /* The limit in DB is in min */
+
+    if (sql_job_get_max_duration (workload_date, hierarchy_list, &time_limit,
+				sql_get_error_logger, NULL) != 0)
+    {
+	return -1;
+    }
+    ptr->max_duration = time_limit * 60; /* The limit in DB is in min */
+
+    if (sql_job_get_retries (workload_date, hierarchy_list, &(ptr->retries),
+                             sql_get_error_logger, NULL) != 0)
+    {
+        return -1;
+    }
+
+    if (sql_job_get_retries_interval (workload_date, hierarchy_list,
+                                      &time_limit,
+                                      sql_get_error_logger, NULL) != 0)
+    {
+        return -1;
+    }
+    ptr->retries_interval = time_limit * 60; /* The value in DB is in min */
+
+    if (sql_status_get (workload_date, ptr->id, &current_status, &start_time,
+                        &(ptr->retry_num), NULL,
+                        sql_get_error_logger, NULL) != 0)
+    {
+        return -1;
+    }
+
+    ptr->status = (current_status == 0)
+                ? JOB_STATUS_STATE_WAITING
+                : job_status_state_int2status (current_status);
+    ptr->start_time = (ptr->status == JOB_STATUS_STATE_RUNNING)
+                                               ? (schedwi_time) start_time : 0;
+    ptr->run_time   = schedwi_time_convert (job_date, job_time);
+
+
+    /* Retrieve contraint files*/
+    constraint_files = NULL;
+    j = 0;
+    if (sql_constraint_file_get (workload_date, &rows, ptr->id,
+                                 sql_get_error_logger, NULL) != 0)
+    {
+        return -1;
+    }
+    n = lwc_getNumNode (rows);
+    if (n != 0) {
+        constraint_files = (schedwi_jobtree_file_ptr) malloc (
+                                            sizeof (schedwi_jobtree_file) * n);
+        if (constraint_files == NULL) {
+            lwc_delLL (rows, (void (*)(const void *)) sql_free_row);
+            lwc_writeLog (LOG_CRIT, _("Memory allocation error"));
+            return -1;
+        }
+        for (i = 0;
+             i < n && (row = (row_item_t *) lwc_delStartLL (rows)) != NULL;
+             i++)
+        {
+            /*
+             * row[0] --> Job ID
+             * row[1] --> Host ID
+             * row[2] --> File name
+             * row[3] --> Test on file exists (1) or not (0)
+             */
+            filename = row[2].value_string;
+            if (filename != NULL && filename[0] != '\0') {
+                constraint_files[j].file = (char *)malloc(row[2].len +1);
+                if (constraint_files[j].file == NULL) {
+                    while (--j >= 0) {
+                        free (constraint_files[j].file);
+                    }
+                    free (constraint_files);
+                    sql_free_row (row);
+                    lwc_delLL (rows, (void (*)(const void *)) sql_free_row);
+                    lwc_writeLog (LOG_CRIT, _("Memory allocation error"));
+                    return -1;
+                }
+                strcpy (constraint_files[j].file, filename);
+                constraint_files[j].host_id =
+                           (unsigned long long int)sql_row_item2ll (&(row[1]));
+                constraint_files[j].exist =
+                           (char)sql_row_item2ll (&(row[3]));
+                j++;
+            }
+            sql_free_row (row);
+        }
+    }
+	if (ptr->constraint_files != NULL) {
+		i = ptr->num_constraint_files;
+		while (--i >= 0) {
+			free ((ptr->constraint_files)[i].file);
+		}
+		free (ptr->constraint_files);
+	}
+    ptr->constraint_files = constraint_files;
+    ptr->num_constraint_files = j;
+    lwc_delLL (rows, (void (*)(const void *)) sql_free_row);
+
+    return 0;
+}
+
+
 /*
  * Create a new node
  *
@@ -88,25 +224,18 @@ sql_get_error_logger (void *data, const char *msg, unsigned int err_code)
  *	  using lwc_writeLog())
  */
 static schedwi_jobtree_node_ptr
-new_schedwi_jobtree_node (	const char *id,
+new_schedwi_jobtree_node (	unsigned long long int id,
 				char *path,
 				int workload_date,
-				const char *node_type,
+				char node_type,
 				schedwi_date job_date,
-				const char *job_time,
+				int job_time,
 				lwc_LL *hierarchy_list)
 {
 	schedwi_jobtree_node_ptr ptr;
-	short int time_limit;
-	int current_status;
-	long int start_time;
-	lwc_LL *rows;
-	char **row;
-	int n, i, j;
 
 #if HAVE_ASSERT_H
-	assert (   id != NULL && path != NULL && node_type != NULL
-		&& job_time != NULL && hierarchy_list != NULL);
+	assert (path != NULL && hierarchy_list != NULL);
 #endif
 
 	/* Allocate memory for the new node */
@@ -115,136 +244,25 @@ new_schedwi_jobtree_node (	const char *id,
 		lwc_writeLog (LOG_CRIT, _("Memory allocation error"));
 		return NULL;
 	}
-
-	/* Retrieve parameters from the database */
-	if (sql_job_get_start_limit (	workload_date, hierarchy_list,
-					&time_limit,
-					sql_get_error_logger, NULL) != 0)
-	{
-		free (ptr);
-		return NULL;
-	}
-	ptr->start_limit = time_limit * 60; /* The limit in DB is in min */
-
-	if (sql_job_get_max_duration (	workload_date, hierarchy_list,
-					&time_limit,
-					sql_get_error_logger, NULL) != 0)
-	{
-		free (ptr);
-		return NULL;
-	}
-	ptr->max_duration = time_limit * 60; /* The limit in DB is in min */
-
-	if (sql_job_get_retries (	workload_date, hierarchy_list,
-					&(ptr->retries),
-					sql_get_error_logger, NULL) != 0)
-	{
-		free (ptr);
-		return NULL;
-	}
-
-	if (sql_job_get_retries_interval (workload_date, hierarchy_list,
-					&time_limit,
-					sql_get_error_logger, NULL) != 0)
-	{
-		free (ptr);
-		return NULL;
-	}
-	ptr->retries_interval = time_limit * 60; /* The value in DB is in min*/
-
- 	if (sql_status_get (	workload_date,
-				id, &current_status, &start_time,
-				&(ptr->retry_num),
-				NULL,
-				sql_get_error_logger, NULL) != 0)
-	{
-		free (ptr);
-		return NULL;
-	}
-
-	if (sql_constraint_file_get (	workload_date, &rows, id,
-					sql_get_error_logger, NULL) != 0)
-	{
-		free (ptr);
-		return NULL;
-	}
-
-	n = lwc_getNumNode (rows);
-	if (n != 0) {
-		ptr->constraint_files = (schedwi_jobtree_file_ptr) malloc (
-				sizeof (schedwi_jobtree_file) * n);
-		if (ptr->constraint_files == NULL) {
-			lwc_delLL (	rows,
-					(void (*)(const void *)) sql_free_row);
-			free (ptr);
-			lwc_writeLog (LOG_CRIT, _("Memory allocation error"));
-			return NULL;
-		}
-		for (	i = j = 0;
-			   i < n
-			&& (row = (char **) lwc_delStartLL (rows)) != NULL;
-			i++)
-		{
-			/*
-			 * row[0] --> Job ID
-			 * row[1] --> Host ID
-			 * row[2] --> File name
-			 * row[3] --> Test on file exists (1) or not (0)
-			 */
-			if (row[2] != '\0') {
-				(ptr->constraint_files)[j].file =  (char *)
-					malloc (schedwi_strlen (row[2]) + 1);
-				if ((ptr->constraint_files)[j].file == NULL) {
-					while (--j >= 0) {
-						free (
-					(ptr->constraint_files)[j].file);
-					}
-					free (ptr->constraint_files);
-					free (ptr);
-					sql_free_row (row);
-					lwc_delLL (rows,
-					(void (*)(const void *)) sql_free_row);
-					lwc_writeLog (LOG_CRIT,
-						_("Memory allocation error"));
-					return NULL;
-				}
-				strcpy ((ptr->constraint_files)[j].file,
-					row[2]);
-				(ptr->constraint_files)[j].host_id =
-						strtoull (row[1], NULL, 0);
-				(ptr->constraint_files)[j].exist =
-						(row[3][0] == '0') ? 0 : 1;
-				j++;
-			}
-			sql_free_row (row);
-		}
-		ptr->num_constraint_files = j;
-	}
-	else {
-		ptr->constraint_files = NULL;
-		ptr->num_constraint_files = 0;
-	}
-	lwc_delLL (rows, (void (*)(const void *)) sql_free_row);
-
-
-
-	/* Fill the structure */
-	ptr->status = (current_status == 0)
-				? JOB_STATUS_STATE_WAITING
-				: job_status_state_int2status (current_status);
-	ptr->path       = path;
 	ptr->parent     = NULL;
 	ptr->children   = NULL;
 	ptr->next       = NULL;
 	ptr->prev       = NULL;
+	ptr->id         = id;
+	ptr->path       = path;
+	ptr->start_day  = job_date;
+	ptr->node_type  = node_type;
 	ptr->links      = NULL;
 	ptr->links_in   = NULL;
-	ptr->start_time = (ptr->status == JOB_STATUS_STATE_RUNNING)
-				? (schedwi_time) start_time
-				: 0;
-	ptr->node_type  = (*node_type == '0') ? 0 : 1;
-	ptr->id	        = strtoull (id, NULL, 0);
-	ptr->run_time  = schedwi_time_convert_from_string (job_date, job_time);
+	ptr->constraint_files = NULL;
+	ptr->num_constraint_files = 0;
+
+	if (set_schedwi_jobtree_node (	ptr, workload_date, job_date, job_time,
+					hierarchy_list) != 0)
+	{
+		free (ptr);
+		return NULL;
+	}
 
 	return ptr;
 }
@@ -306,6 +324,64 @@ free_jobtree_all_nodes (schedwi_jobtree_node_ptr ptr)
 }
 
 
+/*
+ * Update a node
+ * Return:
+ *
+ *   0 --> No error
+ *  -1 --> Error (an error message has already been logged by this function)
+ */
+static int
+update_jobtree_node (schedwi_jobtree_node_ptr ptr, int workload_date)
+{
+	lwc_LL *hierarchy_list;
+	schedwi_jobtree_node_ptr p;
+	int job_time;
+
+#if HAVE_ASSERT_H
+	assert (ptr != NULL);
+#endif
+
+	/*
+	 * Initialize the hierarchy working list.
+	 * This list will be used to retrieve the job parameters from the
+	 * database.
+	 */
+	hierarchy_list = lwc_newLL ();
+	if (hierarchy_list == NULL) {
+		lwc_writeLog (LOG_CRIT, _("Memory allocation error"));
+		return -1;
+	}
+
+	for (p = ptr; p != NULL; p = p->parent) {
+		if (lwc_addEndLL (hierarchy_list, (&ptr->id)) != 0) {
+			lwc_delLL (hierarchy_list, NULL);
+			lwc_writeLog (LOG_CRIT, _("Memory allocation error"));
+			return -1;
+		}
+	}
+
+	/* Retrieve the run time from the database */
+	if (sql_job_get_start_time (	workload_date, ptr->id, &job_time,
+					sql_get_error_logger, NULL) != 0)
+	{
+		lwc_delLL (hierarchy_list, NULL);
+		return -1;
+	}
+
+	/* Update the job parameters */
+	if (set_schedwi_jobtree_node (	ptr, workload_date, ptr->start_day,
+					job_time, hierarchy_list) != 0)
+	{
+		lwc_delLL (hierarchy_list, NULL);
+		return -1;
+	}
+
+	lwc_delLL (hierarchy_list, NULL);
+	return 0;
+}
+
+
 /*
  * Comparison function used by qsort() to sort the job/jobset list by ID
  */
@@ -424,14 +500,14 @@ build_index ( schedwi_jobtree_node_ptr top, unsigned int nb,
  * functions
  */
 static void
-sql_list_error_logger (void *data, const char *msg, unsigned int err_code)
+sql_list_error_logger (void *data, const char *msg, int err_code)
 {
 	if (msg != NULL) {
 		lwc_writeLog (LOG_ERR, msg);
 	}
 	else {
 		lwc_writeLog (LOG_ERR,
-	    _("Database error while trying to retrieve the job list"));
+		_("Database error while trying to retrieve the job list"));
 	}
 }
 
@@ -440,7 +516,7 @@ sql_list_error_logger (void *data, const char *msg, unsigned int err_code)
  * Recursively build the job/jobset tree
  *
  * Return:
- *   0 --> No error.  children contains the tree and must be freed by 
+ *   0 --> No error.  children contains the tree and must be freed by
  *         free_jobtree_all_nodes(). nb_jobs contains the number of
  *         jobs/jobsets in the returned tree.
  *  -1 --> Memory allocation error.  An error message has been logged by
@@ -450,262 +526,314 @@ sql_list_error_logger (void *data, const char *msg, unsigned int err_code)
  */
 static int
 build_jobtree_recur (	schedwi_jobtree_node_ptr jobset,
-			const char *jobset_id,
+			unsigned long long int jobset_id,
 			const char *jobset_path,
 			unsigned int jobset_path_length,
 			schedwi_date jobset_date,
 			int workload_date,
-			const char *jobset_time,
+			int jobset_time,
 			calendar_list_t_ptr calendars,
 			lwc_LL *hierarchy_list,
 			schedwi_jobtree_node_ptr *children,
 			unsigned int *nb_jobs)
 {
-	lwc_LL *job_list;
-	char **row;
-	unsigned int sql_ret, length;
-	int ret;
-	char *calendars_for_today, *calendars_for_tomorrow, *path;
-	schedwi_date date_tomorrow;
-	schedwi_jobtree_node_ptr ptr, nxt;
+    lwc_LL *job_list;
+    row_item_t *row;
+    unsigned int length;
+    unsigned long long int *id, i;
+    char jtype;
+    int stime, ret;
+    char *calendars_for_today, *calendars_for_tomorrow, *path;
+    schedwi_date date_tomorrow;
+    schedwi_jobtree_node_ptr ptr, nxt;
 
 #if HAVE_ASSERT_H
-	assert (   jobset_id != NULL && jobset_path != NULL
-		&& calendars != NULL && hierarchy_list != NULL
-		&& jobset_time != NULL && children != NULL
-		&& nb_jobs != NULL);
+    assert (   jobset_path != NULL && calendars != NULL
+            && hierarchy_list != NULL && children != NULL && nb_jobs != NULL);
 #endif
 
-	ptr = nxt = NULL;
+    ptr = nxt = NULL;
 
-	/*
-	 * Build the tree with the jobs and jobsets that must run today
-	 */
+    /*
+     * Build the tree with the jobs and jobsets that must run today
+     */
 
-	/* Get the calendar ID list for today */
-	if (get_calendar_list_for_the_day (	calendars, jobset_date,
-						&calendars_for_today) != 0)
-	{
-		return -1;
-	}
+    /* Get the calendar ID list for today */
+    if (get_calendar_list_for_the_day (calendars, jobset_date,
+                                       &calendars_for_today) != 0)
+    {
+        return -1;
+    }
 
-	/* Get the job/jobset list */
-	sql_ret = sql_job_today (	workload_date,
-					&job_list,
-					jobset_id, calendars_for_today,
-					jobset_time, sql_list_error_logger,
-					NULL);
-	free (calendars_for_today);
-	if (sql_ret != 0) {
-		return (sql_ret == 1) ? -1 : -2;
-	}
+    /* Get the job/jobset list */
+    ret = sql_job_today (workload_date,
+                         &job_list,
+                         jobset_id,
+                         calendars_for_today,
+                         jobset_time,
+                         sql_list_error_logger,
+                         NULL);
+    free (calendars_for_today);
+    if (ret != 0) {
+        return ret;
+    }
 
-	/* Build the tree from the list */
-	while ((row = (char **) lwc_delStartLL (job_list)) != NULL) {
-		/*
-		 * row[0] --> ID
-		 * row[1] --> Type (0: jobset and 1: job)
-		 * row[2] --> Start time
-		 * row[3] --> Job/Jobset name
-		 */
+    /* Build the tree from the list */
+    while ((row = (row_item_t *) lwc_delStartLL (job_list)) != NULL) {
+        /*
+         * row[0] --> ID
+         * row[1] --> Type (0: jobset and 1: job)
+         * row[2] --> Start time
+         * row[3] --> Job/Jobset name
+         */
 
-		/* Build the full path name of the job/jobset */
-		length = jobset_path_length + schedwi_strlen (row[3]) + 1; 
-		path = (char *) malloc (length + 1);
-		if (path == NULL) {
-			sql_free_row (row);
-			lwc_delLL (	job_list,
-					(void (*)(const void *)) sql_free_row);
-			free_jobtree_all_nodes (nxt);
-			lwc_writeLog (LOG_CRIT, _("Memory allocation error"));
-			return -1;
-		}
-		strcpy (path, jobset_path);
-		path[jobset_path_length] = '/';
-		strcpy (path + jobset_path_length + 1, row[3]);
+        /* Get the job/jobset id */
+        i = (unsigned long long int) sql_row_item2ll (&(row[0]));
 
-		/* Add the new job/jobset ID to the hierarchy list */
-		if (lwc_addStartLL (hierarchy_list, row[0]) != 0) {
-			free (path);
-			sql_free_row (row);
-			lwc_delLL (	job_list,
-					(void (*)(const void *)) sql_free_row);
-			free_jobtree_all_nodes (nxt);
-			lwc_writeLog (LOG_CRIT, _("Memory allocation error"));
-			return -1;
-		}
+        /* Build the full path name of the job/jobset */
+        length = jobset_path_length + row[3].len + 1;
+        path = (char *) malloc (length + 1);
+        if (path == NULL) {
+            sql_free_row (row);
+            lwc_delLL (job_list, (void (*)(const void *)) sql_free_row);
+            free_jobtree_all_nodes (nxt);
+            lwc_writeLog (LOG_CRIT, _("Memory allocation error"));
+            return -1;
+        }
+        if (i == 1) { /* Root jobset */
+                path[0] = '/';
+                path[1] = '\0';
+                length = 1;
+        }
+        else {
+                if (jobset_id == 1) {
+                        path[0] = '/';
+                        strcpy (path + 1, row[3].value_string);
+                        length = row[3].len + 1;
+                }
+                else {
+                        strcpy (path, jobset_path);
+                        path[jobset_path_length] = '/';
+                        strcpy (path + jobset_path_length + 1,
+                                row[3].value_string);
+                }
+        }
 
-		/* Create the new job object */
-		ptr = new_schedwi_jobtree_node (	row[0], path,
-							workload_date,
-							row[1],
-							jobset_date, row[2],
-							hierarchy_list);
-		if (ptr == NULL) {
-			lwc_delStartLL (hierarchy_list);
-			free (path);
-			sql_free_row (row);
-			lwc_delLL (	job_list,
-					(void (*)(const void *)) sql_free_row);
-			free_jobtree_all_nodes (nxt);
-			return -1;
-		}
+        /* Add the new job/jobset ID to the hierarchy list */
+        id = (unsigned long long int *) malloc (sizeof(unsigned long long int));
+        if (id == NULL || lwc_addStartLL (hierarchy_list, id) != 0) {
+            if (id != NULL) {
+                free (id);
+            }
+            free (path);
+            sql_free_row (row);
+            lwc_delLL (job_list, (void (*)(const void *)) sql_free_row);
+            free_jobtree_all_nodes (nxt);
+            lwc_writeLog (LOG_CRIT, _("Memory allocation error"));
+            return -1;
+        }
+        *id = i;
+        jtype = (char) sql_row_item2ll (&(row[1]));
+        stime = (int) sql_row_item2ll (&(row[2]));
+        sql_free_row (row);
 
-		/* Link the new job/jobset */
-		(*nb_jobs)++;
-		ptr->parent = jobset;
-		ptr->next = nxt;
-		if (nxt != NULL) {
-			nxt->prev = ptr;
-		}
-		nxt = ptr;
+        /* Create the new job object */
+        ptr = new_schedwi_jobtree_node (*id,
+                                        path,
+                                        workload_date,
+                                        jtype,
+                                        jobset_date,
+                                        stime,
+                                        hierarchy_list);
+        if (ptr == NULL) {
+            free (id);
+            lwc_delStartLL (hierarchy_list);
+            free (path);
+            lwc_delLL (job_list, (void (*)(const void *)) sql_free_row);
+            free_jobtree_all_nodes (nxt);
+            return -1;
+        }
 
-		/* If it's a jobset, recursively build the job tree */
-		if (ptr->node_type == 0) {
-			ret = build_jobtree_recur (	ptr,
-							row[0],
-							path, length,
-							jobset_date,
-							workload_date,
-							row[2],
-							calendars,
-							hierarchy_list,
-							&(ptr->children),
-							nb_jobs);
-			if (ret != 0) {
-				lwc_delStartLL (hierarchy_list);
-				sql_free_row (row);
-				lwc_delLL (	job_list,
-					(void (*)(const void *)) sql_free_row);
-				free_jobtree_all_nodes (nxt);
-				return ret;
-			}
-		}
+        /* Link the new job/jobset */
+        (*nb_jobs)++;
+        ptr->parent = jobset;
+        ptr->next = nxt;
+        if (nxt != NULL) {
+            nxt->prev = ptr;
+        }
+        nxt = ptr;
 
-		/* Remove the job/jobset ID from the hierarchy list */
-		lwc_delStartLL (hierarchy_list);
+        /* If it's a jobset, recursively build the job tree */
+        if (ptr->node_type == 0) {
+            ret = build_jobtree_recur (ptr,
+                                       *id,
+                                       path,
+                                       length,
+                                       jobset_date,
+                                       workload_date,
+                                       stime,
+                                       calendars,
+                                       hierarchy_list,
+                                       &(ptr->children),
+                                       nb_jobs);
+            if (ret != 0) {
+                free (id);
+                lwc_delStartLL (hierarchy_list);
+                lwc_delLL (job_list, (void (*)(const void *)) sql_free_row);
+                free_jobtree_all_nodes (nxt);
+                return ret;
+            }
+        }
 
-		sql_free_row (row);
-	}
-	lwc_delLL (job_list, NULL);
+        /* Remove the job/jobset ID from the hierarchy list */
+        free (id);
+        lwc_delStartLL (hierarchy_list);
+    }
+    lwc_delLL (job_list, NULL);
 
 
-	/*
-	 * Complete the tree with the jobs and jobsets that must run tomorrow 
-	 */
+    /*
+     * Complete the tree with the jobs and jobsets that must run tomorrow
+     */
 
-	/* Get the calendar ID list for tomorrow */
-	schedwi_date_add_days (jobset_date, &date_tomorrow, 1);
-	if (get_calendar_list_for_the_day (	calendars, date_tomorrow,
-						&calendars_for_tomorrow) != 0)
-	{
-		free_jobtree_all_nodes (nxt);
-		return -1;
-	}
+    /* Get the calendar ID list for tomorrow */
+    schedwi_date_add_days (jobset_date, &date_tomorrow, 1);
+    if (get_calendar_list_for_the_day (calendars, date_tomorrow,
+                                       &calendars_for_tomorrow) != 0)
+    {
+        free_jobtree_all_nodes (nxt);
+        return -1;
+    }
 
-	/* Get the job/jobset list */
-	sql_ret = sql_job_tomorrow (	workload_date,
-					&job_list,
-					jobset_id, calendars_for_tomorrow,
-					jobset_time, sql_list_error_logger,
-					NULL);
-	free (calendars_for_tomorrow);
-	if (sql_ret != 0) {
-		free_jobtree_all_nodes (nxt);
-		return (sql_ret == 1) ? -1 : -2;
-	}
+    /* Get the job/jobset list */
+    ret = sql_job_tomorrow (workload_date,
+                            &job_list,
+                            jobset_id,
+                            calendars_for_tomorrow,
+                            jobset_time,
+                            sql_list_error_logger,
+                            NULL);
+    free (calendars_for_tomorrow);
+    if (ret != 0) {
+        free_jobtree_all_nodes (nxt);
+        return ret;
+    }
 
-	/* Build the tree from the list */
-	while ((row = (char **) lwc_delStartLL (job_list)) != NULL) {
-		/*
-		 * row[0] --> ID
-		 * row[1] --> Type (0: jobset and 1: job)
-		 * row[2] --> Start time
-		 * row[3] --> Job/Jobset name
-		 */
+    /* Build the tree from the list */
+    while ((row = (row_item_t *) lwc_delStartLL (job_list)) != NULL) {
+        /*
+         * row[0] --> ID
+         * row[1] --> Type (0: jobset and 1: job)
+         * row[2] --> Start time
+         * row[3] --> Job/Jobset name
+         */
 
-		/* Build the full path name of the job/jobset */
-		length = jobset_path_length + schedwi_strlen (row[3]) + 1; 
-		path = (char *) malloc (length + 1);
-		if (path == NULL) {
-			sql_free_row (row);
-			lwc_delLL (	job_list,
-					(void (*)(const void *)) sql_free_row);
-			free_jobtree_all_nodes (nxt);
-			lwc_writeLog (LOG_CRIT, _("Memory allocation error"));
-			return -1;
-		}
-		strcpy (path, jobset_path);
-		path[jobset_path_length] = '/';
-		strcpy (path + jobset_path_length + 1, row[3]);
+        /* Get the job/jobset id */
+        i = (unsigned long long int) sql_row_item2ll (&(row[0]));
 
-		/* Add the new job/jobset ID to the hierarchy list */
-		if (lwc_addStartLL (hierarchy_list, row[0]) != 0) {
-			free (path);
-			sql_free_row (row);
-			lwc_delLL (	job_list,
-					(void (*)(const void *)) sql_free_row);
-			free_jobtree_all_nodes (nxt);
-			lwc_writeLog (LOG_CRIT, _("Memory allocation error"));
-			return -1;
-		}
+        /* Build the full path name of the job/jobset */
+        length = jobset_path_length + row[3].len + 1;
+        path = (char *) malloc (length + 1);
+        if (path == NULL) {
+            sql_free_row (row);
+            lwc_delLL (job_list, (void (*)(const void *)) sql_free_row);
+            free_jobtree_all_nodes (nxt);
+            lwc_writeLog (LOG_CRIT, _("Memory allocation error"));
+            return -1;
+        }
+        if (i == 1) { /* Root jobset */
+                path[0] = '/';
+                path[1] = '\0';
+                length = 1;
+        }
+        else {
+                if (jobset_id == 1) {
+                        path[0] = '/';
+                        strcpy (path + 1, row[3].value_string);
+                        length = row[3].len + 1;
+                }
+                else {
+                        strcpy (path, jobset_path);
+                        path[jobset_path_length] = '/';
+                        strcpy (path + jobset_path_length + 1,
+                                row[3].value_string);
+                }
+        }
 
-		/* Create the new job object */
-		ptr = new_schedwi_jobtree_node (	row[0], path,
-							workload_date,
-							row[1],
-							date_tomorrow, row[2],
-							hierarchy_list);
-		if (ptr == NULL) {
-			lwc_delStartLL (hierarchy_list);
-			free (path);
-			sql_free_row (row);
-			lwc_delLL (	job_list,
-					(void (*)(const void *)) sql_free_row);
-			free_jobtree_all_nodes (nxt);
-			return -1;
-		}
+        /* Add the new job/jobset ID to the hierarchy list */
+        id = (unsigned long long int *) malloc (sizeof(unsigned long long int));
+        if (id == NULL || lwc_addStartLL (hierarchy_list, id) != 0) {
+            if (id != NULL) {
+                free (id);
+            }
+            free (path);
+            sql_free_row (row);
+            lwc_delLL (job_list, (void (*)(const void *)) sql_free_row);
+            free_jobtree_all_nodes (nxt);
+            lwc_writeLog (LOG_CRIT, _("Memory allocation error"));
+            return -1;
+        }
+        *id = i;
+        jtype = (char) sql_row_item2ll (&(row[1]));
+        stime = (int) sql_row_item2ll (&(row[2]));
+        sql_free_row (row);
 
-		/* Link the new job/jobset */
-		(*nb_jobs)++;
-		ptr->parent = jobset;
-		ptr->next = nxt;
-		if (nxt != NULL) {
-			nxt->prev = ptr;
-		}
-		nxt = ptr;
+        /* Create the new job object */
+        ptr = new_schedwi_jobtree_node (*id,
+                                        path,
+                                        workload_date,
+                                        jtype,
+                                        date_tomorrow,
+                                        stime,
+                                        hierarchy_list);
+        if (ptr == NULL) {
+            free (id);
+            lwc_delStartLL (hierarchy_list);
+            free (path);
+            lwc_delLL (job_list, (void (*)(const void *)) sql_free_row);
+            free_jobtree_all_nodes (nxt);
+            return -1;
+        }
 
-		/* If it's a jobset, recursively build the job tree */
-		if (ptr->node_type == 0) {
-			ret = build_jobtree_recur (	ptr,
-							row[0],
-							path, length,
-							date_tomorrow,
-							workload_date,
-							row[2],
-							calendars,
-							hierarchy_list,
-							&(ptr->children),
-							nb_jobs);
-			if (ret != 0) {
-				lwc_delStartLL (hierarchy_list);
-				sql_free_row (row);
-				lwc_delLL (job_list,
-					(void (*)(const void *)) sql_free_row);
-				free_jobtree_all_nodes (nxt);
-				return ret;
-			}
-		}
+        /* Link the new job/jobset */
+        (*nb_jobs)++;
+        ptr->parent = jobset;
+        ptr->next = nxt;
+        if (nxt != NULL) {
+            nxt->prev = ptr;
+        }
+        nxt = ptr;
 
-		/* Remove the job/jobset ID from the hierarchy list */
-		lwc_delStartLL (hierarchy_list);
+        /* If it's a jobset, recursively build the job tree */
+        if (ptr->node_type == 0) {
+            ret = build_jobtree_recur (ptr,
+                                       *id,
+                                       path,
+                                       length,
+                                       date_tomorrow,
+                                       workload_date,
+                                       stime,
+                                       calendars,
+                                       hierarchy_list,
+                                       &(ptr->children),
+                                       nb_jobs);
+            if (ret != 0) {
+                free (id);
+                lwc_delStartLL (hierarchy_list);
+                lwc_delLL (job_list, (void (*)(const void *)) sql_free_row);
+                free_jobtree_all_nodes (nxt);
+                return ret;
+            }
+        }
 
-		sql_free_row (row);
-	}
-	lwc_delLL (job_list, NULL);
+        /* Remove the job/jobset ID from the hierarchy list */
+        free (id);
+        lwc_delStartLL (hierarchy_list);
+    }
+    lwc_delLL (job_list, NULL);
 
-	*children = ptr;
-	return 0;
+    *children = ptr;
+    return 0;
 }
 
 
@@ -713,7 +841,7 @@ build_jobtree_recur (	schedwi_jobtree_node_ptr jobset,
  * Build the job/jobset tree
  *
  * Return:
- *   0 --> No error.  top contains the tree and must be freed by 
+ *   0 --> No error.  top contains the tree and must be freed by
  *         free_jobtree_all_nodes(). nb_jobs contains the number of
  *         jobs/jobsets in the returned tree.
  *  -1 --> Memory allocation error.  An error message has been logged by
@@ -728,20 +856,20 @@ build_jobtree (	schedwi_date jobset_date,
 		unsigned int *nb_jobs)
 {
 	lwc_LL *hierarchy_list;
-	char *id;
+	unsigned long long int *id;
 	int ret, workload_date;
 
 	/*
 	 * Initialize the hierarchy working list with the root job (ID=1).
 	 * This list will be used to retrieve the job parameters from the
 	 * database.
-	 */ 
+	 */
 	hierarchy_list = lwc_newLL ();
 	if (hierarchy_list == NULL) {
 		lwc_writeLog (LOG_CRIT, _("Memory allocation error"));
 		return -1;
 	}
-	id = (char *) malloc (2);
+	id = (unsigned long long int *) malloc (sizeof(unsigned long long int));
 	if (id == NULL || lwc_addEndLL (hierarchy_list, id) != 0) {
 		if (id != NULL) {
 			free (id);
@@ -750,14 +878,13 @@ build_jobtree (	schedwi_date jobset_date,
 		lwc_writeLog (LOG_CRIT, _("Memory allocation error"));
 		return -1;
 	}
-	id[0] = '1';
-	id[1] = '\0';
+	*id = 1;
 
 	/* Build the job/jobset tree */
 	*nb_jobs = 0;
 	workload_date = schedwi_date_to_int (jobset_date);
-	ret = build_jobtree_recur (	NULL, "0", "", 0, jobset_date,
-					workload_date, "0", calendars,
+	ret = build_jobtree_recur (	NULL, 0, "", 0, jobset_date,
+					workload_date, 0, calendars,
 					hierarchy_list,
 					top, nb_jobs);
 	lwc_delLL (hierarchy_list, (void (*)(const void *))free);
@@ -769,14 +896,14 @@ build_jobtree (	schedwi_date jobset_date,
  * Error callback function for the sql_link_get_list() function
  */
 static void
-sql_link_get_list_logger (void *data, const char *msg, unsigned int err_code)
+sql_link_get_list_logger (void *data, const char *msg, int err_code)
 {
 	if (msg != NULL) {
 		lwc_writeLog (LOG_ERR, msg);
 	}
 	else {
 		lwc_writeLog (LOG_ERR,
-	    _("Database error while trying to retrieve the job links"));
+		_("Database error while trying to retrieve the job links"));
 	}
 }
 
@@ -796,9 +923,10 @@ build_links (jobtree_ptr ptr)
 {
 	unsigned long long int id;
 	unsigned int i, n, idx;
+	int ret;
 	char *job_id_list;
 	lwc_LL *rows;
-	char **row;
+	row_item_t *row;
 	schedwi_jobtree_node_ptr node_src, node_dst;
 	schedwi_jobtree_link_ptr new_link_out, new_link_in;
 
@@ -829,17 +957,17 @@ build_links (jobtree_ptr ptr)
 	}
 
 	/* Retrieve the links from the database */
-	n = sql_link_get_list (	&rows,
+	ret = sql_link_get_list (&rows,
 				schedwi_date_to_int (ptr->date),
 				job_id_list,
 				sql_link_get_list_logger, NULL);
 	free (job_id_list);
-	if (n != 0) {
-		return (n == 1)? -1: -2;
+	if (ret != 0) {
+		return ret;
 	}
 
 	node_src = NULL;
-	while ((row = (char **) lwc_delStartLL (rows)) != NULL) {
+	while ((row = (row_item_t *) lwc_delStartLL (rows)) != NULL) {
 		/*
 		 * row[0] --> Source job ID
 		 * row[1] --> Destination job ID
@@ -847,7 +975,7 @@ build_links (jobtree_ptr ptr)
 		 */
 
 		/* Find the source job/jobset in the tree */
-		id = strtoull (row[0], NULL, 0);
+		id = (unsigned long long int) sql_row_item2ll (&(row[0]));
 		if (node_src == NULL || node_src->id != id) {
 			node_src = jobtree_find (ptr, id);
 			if (node_src == NULL) {
@@ -857,8 +985,8 @@ build_links (jobtree_ptr ptr)
 			}
 		}
 
-		/* Find the destination job/jobset in the tree */ 
-		id = strtoull (row[1], NULL, 0);
+		/* Find the destination job/jobset in the tree */
+		id = (unsigned long long int) sql_row_item2ll (&(row[1]));
 		node_dst = jobtree_find (ptr, id);
 		if (node_dst == NULL) {
 			/*
@@ -895,8 +1023,8 @@ build_links (jobtree_ptr ptr)
 		new_link_out->next = node_src->links;
 		node_src->links = new_link_out;
 		new_link_out->linked_node = node_dst;
-		new_link_out->required_status = job_status_state_str2status (
-								row[2]);
+		new_link_out->required_status = job_status_state_int2status (
+				(int)sql_row_item2ll (&(row[2])));
 
 		new_link_in->next = node_dst->links_in;
 		node_dst->links_in = new_link_in;
@@ -1101,7 +1229,7 @@ do_transition (	schedwi_date workload_date,
 			return 0;
 
 		default:
-			/*TODO : add transition actions for other status */ 
+			/*TODO : add transition actions for other status */
 			return 0;
 	}
 }
@@ -1111,7 +1239,7 @@ do_transition (	schedwi_date workload_date,
  * Change the status of a job/jobset. The fast parameter specifies if the
  * status must be propagated to the other jobs (0) or not (1).
  * This flag is used to avoid dead locks with clients and is used when a
- * client has sent the result status of a job.
+ * agent has sent the result status of a job.
  *
  * Return:
  *   0 --> No error
@@ -1284,7 +1412,7 @@ schedwi_jobtree_node_set_status (	schedwi_date workload_date,
 
 
 /*
- * Check if the provided job/jobset must be started 
+ * Check if the provided job/jobset must be started
  *
  * Return:
  *      0 --> No error
@@ -1305,6 +1433,7 @@ schedwi_jobtree_node_check (	schedwi_date workload_date,
 	}
 
 	d = schedwi_date_to_int (workload_date);
+	update_jobtree_node (ptr, d);
 
 	switch (ptr->status) {
 
@@ -1546,7 +1675,7 @@ schedwi_jobtree_job_finished (	jobtree_ptr ptr,
 
 	/*
 	 * Change the status of the job (and do not check the
-	 * other jobs for now as the client network connection is waiting)
+	 * other jobs for now as the agent network connection is waiting)
 	 */
 	ret = schedwi_jobtree_node_set_status (	ptr->date,
 						node_ptr,
@@ -1680,36 +1809,40 @@ schedwi_jobtree_adjust (jobtree_ptr ptr, int drift, schedwi_time current_time)
 {
 	unsigned int i;
 	schedwi_jobtree_node_ptr node;
+	int workload_date;
 
 	if (ptr == NULL || drift == 0) {
 		return;
 	}
 
+	workload_date = schedwi_date_to_int (ptr->date);
+	lwc_writeLog (	LOG_INFO,
+	_("Workload %d: clock drift (%+d seconds): countermeasures engaged"),
+			workload_date, drift);
 	pthread_mutex_lock (&(ptr->mutex));
-	if (drift > 0) {
-		for (i = 0; i < ptr->nb_jobs; i++) {
-			node = (ptr->jobs)[i].node;
-			if (	   node->status == JOB_STATUS_STATE_RUNNING
+	for (i = 0; i < ptr->nb_jobs; i++) {
+		node = (ptr->jobs)[i].node;
+		if (	   node->status == JOB_STATUS_STATE_RUNNING
 				&& node->max_duration > 0)
-			{
-				node->max_duration += drift;
-			}
-			if (	   node->status == JOB_STATUS_STATE_WAITING
-				&& node->start_limit > 0
-				&& node->run_time < current_time)
-			{
-				node->start_limit += drift;
-			}
+		{
+			node->start_time += drift;
+			/* Update the database */
+			sql_status_adjust_start_time (	workload_date,
+							node->id,
+							node->start_time,
+							NULL, NULL);
 		}
-	}
-	else {
-		for (i = 0; i < ptr->nb_jobs; i++) {
-			node = (ptr->jobs)[i].node;
-			if (	   node->status == JOB_STATUS_STATE_RUNNING
-				&& node->max_duration > 0)
-			{
-				node->start_time += drift;
-			}
+		else if (   drift > 0
+			 && node->status == JOB_STATUS_STATE_WAITING
+			 && node->start_limit > 0
+			 && node->run_time < current_time)
+		{
+			node->start_limit += drift;
+			/* Update the database */
+			sql_job_set_start_limit (	workload_date,
+							node->id,
+							node->start_limit,
+							NULL, NULL);
 		}
 	}
 	pthread_mutex_unlock (&(ptr->mutex));
