@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
+   Copyright (C) 2007-2010 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -77,8 +77,8 @@ static levels verbose_level = NORMAL;
 static const char *configuration_file = NULL;
 static char autofix = 0;
 static char *lost_found_name;
-static unsigned long int lost_found_jobset_id = 0;
-static unsigned long int lost_found_calendar_folder_id = 0;
+static unsigned long long int lost_found_jobset_id = 0;
+static unsigned long long int lost_found_calendar_folder_id = 0;
 static int x, y;
 
 
@@ -122,7 +122,7 @@ NULL
 #endif
 
 	fprintf (stderr, _("Usage: %s [OPTION]...\n"), prog_name);
-						
+
 	for (i = 0; msg[i] != NULL; i++) {
 		if (msg[i][0] == '~') {
 			fputs (	_("The default configuration file is "),
@@ -146,7 +146,7 @@ NULL
  * Error callback function for the sql functions
  */
 static void
-sql_error_logger (void *data, const char *msg, unsigned int err_code)
+sql_error_logger (void *data, const char *msg, int err_code)
 {
 	if (msg != NULL) {
 		fputs (msg, stderr);
@@ -163,17 +163,18 @@ sql_error_logger (void *data, const char *msg, unsigned int err_code)
  * is detected
  */
 static int
-check_job_main_cb_parent (void *data, const char *id, const char *name,
-		char job_type, const char *description)
+check_job_main_cb_parent (	void *data, unsigned long long int id,
+				const char *name,
+				char job_type, const char *description)
 {
 	switch (verbose_level) {
 		case NORMAL:
-			printf (_("%s: %s (%s): error: orphan\n"),
+			printf (_("%s: %s (%lld): error: orphan\n"),
 				(job_type == 0) ? _("Jobset") : _("Job"),
 				name, id);
 			break;
 		case VERBOSE:
-			printf (_("%s: %s (%s): error: orphan\n"),
+			printf (_("%s: %s (%lld): error: orphan\n"),
 				(job_type == 0) ? _("Jobset") : _("Job"),
 				name, id);
 			if (description != NULL && description[0] != '\0') {
@@ -217,17 +218,18 @@ check_job_main_cb_parent (void *data, const char *id, const char *name,
  * is detected
  */
 static int
-check_job_main_cb_calendar (void *data, const char *id, const char *name,
-		char job_type, const char *description)
+check_job_main_cb_calendar (	void *data, unsigned long long int id,
+				const char *name,
+				char job_type, const char *description)
 {
 	switch (verbose_level) {
 		case NORMAL:
-			printf (_("%s: %s (%s): error: unknown calendar\n"),
+			printf (_("%s: %s (%lld): error: unknown calendar\n"),
 				(job_type == 0) ? _("Jobset") : _("Job"),
 				name, id);
 			break;
 		case VERBOSE:
-			printf (_("%s: %s (%s): error: unknown calendar\n"),
+			printf (_("%s: %s (%lld): error: unknown calendar\n"),
 				(job_type == 0) ? _("Jobset") : _("Job"),
 				name, id);
 			if (description != NULL && description[0] != '\0') {
@@ -260,20 +262,20 @@ check_job_main_cb_calendar (void *data, const char *id, const char *name,
  * is detected
  */
 static int
-check_job_main_cb_time (void *data, const char *id, const char *name,
-		char job_type, const char *description,
-		const char *start_time)
+check_job_main_cb_time (void *data, unsigned long long int id,
+			const char *name, char job_type,
+			const char *description, int start_time)
 {
 	switch (verbose_level) {
 		case NORMAL:
 			printf (
-_("%s: %s (%s): error: wrong start time `%s' (must be between -1 and 2359)\n"),
+_("%s: %s (%lld): error: wrong start time `%d' (must be between -1 and 2359)\n"),
 				(job_type == 0) ? _("Jobset") : _("Job"),
 				name, id, start_time);
 			break;
 		case VERBOSE:
 			printf (
-_("%s: %s (%s): error: wrong start time `%s' (must be between -1 and 2359)\n"),
+_("%s: %s (%lld): error: wrong start time `%d' (must be between -1 and 2359)\n"),
 				(job_type == 0) ? _("Jobset") : _("Job"),
 				name, id, start_time);
 			if (description != NULL && description[0] != '\0') {
@@ -384,27 +386,52 @@ static char *job_tables[][3] = {
  * Callback for the sql_check_id() function when an error is detected
  */
 static int
-check_job_parameters_cb (void *data, const char *id, const char *value)
+check_job_parameters_cb (void *data, row_item_t *id, row_item_t *value)
 {
 	char *table_name = (char *)data;
+	unsigned long long int jobid;
+	long long int v;
 
+	jobid = (unsigned long long int) sql_row_item2ll (id);
 	switch (verbose_level) {
 		case NORMAL:
 		case VERBOSE:
-			if (	   value != NULL
-				&& value[0] != '\0'
-				&& value[0] != '0')
+			if (value->type == RES_INT || value->type == RES_UINT)
 			{
-				printf (
-		_("table %s: error: unknown job Id `%s' for value `%s'\n"),
-					table_name, id, value);
+				v = sql_row_item2ll (value);
+				if (v != 0) {
+					printf (
+		_("table %s: error: unknown job Id `%lld' for value `%lld'\n"),
+						table_name, jobid, v);
+				}
+				else {
+					printf (
+				_("table %s: error: unknown job Id `%lld'\n"),
+						table_name, jobid);
+				}
+			}
+			else
+			if (value->type == RES_STRING) {
+				if (	   value->value_string != NULL
+					&& (value->value_string)[0] != '\0')
+				{
+					printf (
+		_("table %s: error: unknown job Id `%lld' for value `%s'\n"),
+					table_name, jobid, value->value_string);
+				}
+				else {
+					printf (
+				_("table %s: error: unknown job Id `%lld'\n"),
+						table_name, jobid);
+				}
 			}
 			else {
 				printf (
-				_("table %s: error: unknown job Id `%s'\n"),
-					table_name, id);
+				_("table %s: error: unknown job Id `%lld'\n"),
+					table_name, jobid);
 			}
 			break;
+
 		default:
 			break;
 	}
@@ -418,7 +445,7 @@ check_job_parameters_cb (void *data, const char *id, const char *value)
 			default:
 				break;
 		}
-		sql_check_delete1 (	table_name, "job_id", id,
+		sql_check_delete1 (	table_name, "job_id", jobid,
 					sql_error_logger, NULL);
 	}
 	return 0;
@@ -458,13 +485,13 @@ check_job_parameters ()
 	return global_ret;
 }
 
-
+#ifdef NO_NEED_TO_CHECK
 /*
  * Callback for the sql_hosts_check_cert() function when an error
  * is detected
  */
 static int
-check_hosts_cb_cert (	void *data, const char *id, const char *name,
+check_hosts_cb_cert (	void *data, unsigned long long int id, const char *name,
 			const char *tcp_port, const char *description)
 {
 	switch (verbose_level) {
@@ -528,6 +555,7 @@ check_hosts ()
 
 	return ret;
 }
+#endif
 
 
 /*
@@ -535,18 +563,19 @@ check_hosts ()
  * is detected
  */
 static int
-check_calendars_cb_parent (	void *data, const char *id, const char *name,
+check_calendars_cb_parent (	void *data, unsigned long long int id,
+				const char *name,
 				char cal_type, const char *description)
 {
 	switch (verbose_level) {
 		case NORMAL:
-			printf (_("%s: %s (%s): error: orphan\n"),
+			printf (_("%s: %s (%lld): error: orphan\n"),
 				(cal_type == 0) ?	_("Calendar") :
 							_("Calendar folder"),
 				name, id);
 			break;
 		case VERBOSE:
-			printf (_("%s: %s (%s): error: orphan\n"),
+			printf (_("%s: %s (%lld): error: orphan\n"),
 				(cal_type == 0) ?	_("Calendar") :
 							_("Calendar folder"),
 				name, id);
@@ -590,14 +619,14 @@ check_calendars_cb_parent (	void *data, const char *id, const char *name,
  * Comment out (add a `#') all the lines of the provided string.
  * Return:
  *     A new allocated string (to be freed by the caller by free()) which
- *     contains the commented out input string 
- */ 
+ *     contains the commented out input string
+ */
 static char *
 comment_lines (const char *src)
 {
 	char *dst;
 	const char *s;
-	char *d, *found; 
+	char *d, *found;
 	unsigned int len;
 
 	dst = (char *) malloc (schedwi_strlen (src) * 2 + 2);
@@ -628,11 +657,12 @@ comment_lines (const char *src)
 
 
 /*
- * Callback for the sql_calendars_check_parent() function when an error
+ * Callback for the sql_calendars_check_formula() function when an error
  * is detected
  */
 static int
-check_calendars_cb_formula (	void *data, const char *id, const char *name,
+check_calendars_cb_formula (	void *data, unsigned long long int id,
+				const char *name,
 				const char *description, const char *formula)
 {
 	char *new_formula;
@@ -640,12 +670,12 @@ check_calendars_cb_formula (	void *data, const char *id, const char *name,
 	switch (verbose_level) {
 		case NORMAL:
 			printf (
-		_("Calendar: %s (%s): error: formula systax error `%s'\n"),
+		_("Calendar: %s (%lld): error: formula systax error `%s'\n"),
 				name, id, formula);
 			break;
 		case VERBOSE:
 			printf (
-		_("Calendar: %s (%s): error: formula systax error `%s'\n"),
+		_("Calendar: %s (%lld): error: formula systax error `%s'\n"),
 				name, id, formula);
 			if (description != NULL && description[0] != '\0') {
 				fputc ('\t', stdout);
@@ -720,20 +750,25 @@ check_calendars ()
  * Callback functions for sql_check_id() when an error is detected
  */
 static int
-check_constraint_files_cb1 (void *data, const char *id, const char *filename) 
+check_constraint_files_cb1 (void *data, row_item_t *id, row_item_t *filename)
 {
+	unsigned long long int jobid;
+
+	jobid = (unsigned long long int) sql_row_item2ll (id);
 	switch (verbose_level) {
 		case NORMAL:
 		case VERBOSE:
-			if (filename != NULL && filename[0] != '\0') {
+			if (	   filename->value_string != NULL
+				&& (filename->value_string)[0] != '\0')
+			{
 				printf (
-_("Constraint file: error: unknown job/jobset Id `%s' for file name `%s'\n"),
-					id, filename);
+_("Constraint file: error: unknown job/jobset Id `%lld' for file name `%s'\n"),
+					jobid, filename->value_string);
 			}
 			else {
 				printf (
-		_("Constraint file: error: unknown job/jobset Id `%s'\n"),
-					id);
+		_("Constraint file: error: unknown job/jobset Id `%lld'\n"),
+					jobid);
 			}
 			break;
 		default:
@@ -748,27 +783,32 @@ _("Constraint file: error: unknown job/jobset Id `%s' for file name `%s'\n"),
 			default:
 				break;
 		}
-		sql_check_delete1 (	"constraint_file", "job_id", id,
+		sql_check_delete1 (	"constraint_file", "job_id", jobid,
 					sql_error_logger, NULL);
 	}
 	return 0;
 }
 
 static int
-check_constraint_files_cb2 (void *data, const char *id, const char *filename) 
+check_constraint_files_cb2 (void *data, row_item_t *id, row_item_t *filename)
 {
+	unsigned long long int hostid;
+
+	hostid = (unsigned long long int) sql_row_item2ll (id);
 	switch (verbose_level) {
 		case NORMAL:
 		case VERBOSE:
-			if (filename != NULL && filename[0] != '\0') {
+			if (	   filename->value_string != NULL
+				&& (filename->value_string)[0] != '\0')
+			{
 				printf (
-	_("Constraint file: error: unknown host Id `%s' for file name `%s'\n"),
-					id, filename);
+_("Constraint file: error: unknown host Id `%lld' for file name `%s'\n"),
+					hostid, filename->value_string);
 			}
 			else {
 				printf (
-			_("Constraint file: error: unknown host Id `%s'\n"),
-					id);
+			_("Constraint file: error: unknown host Id `%lld'\n"),
+					hostid);
 			}
 			break;
 		default:
@@ -783,7 +823,7 @@ check_constraint_files_cb2 (void *data, const char *id, const char *filename)
 			default:
 				break;
 		}
-		sql_check_delete1 (	"constraint_file", "host_id", id,
+		sql_check_delete1 (	"constraint_file", "host_id", hostid,
 					sql_error_logger, NULL);
 	}
 	return 0;
@@ -835,14 +875,18 @@ check_constraint_files ()
  * Callback functions for sql_check_id() when an error is detected
  */
 static int
-check_links_cb1 (void *data, const char *id, const char *job_id_destination) 
+check_links_cb1 (void *data, row_item_t *id, row_item_t *job_id_dest)
 {
+	unsigned long long int jobid, jobid_d;
+
+	jobid = (unsigned long long int) sql_row_item2ll (id);
+	jobid_d = (unsigned long long int) sql_row_item2ll (job_id_dest);
 	switch (verbose_level) {
 		case NORMAL:
 		case VERBOSE:
 			printf (
-_("Link: error: unknown source job/jobset Id `%s' for the link between Ids `%s' and `%s'\n"),
-				id, id, job_id_destination);
+_("Link: error: unknown source job/jobset Id `%lld' for the link between Ids `%lld' and `%lld'\n"),
+				jobid, jobid, jobid_d);
 			break;
 		default:
 			break;
@@ -856,21 +900,25 @@ _("Link: error: unknown source job/jobset Id `%s' for the link between Ids `%s'
 			default:
 				break;
 		}
-		sql_check_delete1 (	"link", "job_id_source", id,
+		sql_check_delete1 (	"link", "job_id_source", jobid,
 					sql_error_logger, NULL);
 	}
 	return 0;
 }
 
 static int
-check_links_cb2 (void *data, const char *id, const char *job_id_source) 
+check_links_cb2 (void *data, row_item_t *id, row_item_t *job_id_source)
 {
+	unsigned long long int jobid, jobid_s;
+
+	jobid = (unsigned long long int) sql_row_item2ll (id);
+	jobid_s = (unsigned long long int) sql_row_item2ll (job_id_source);
 	switch (verbose_level) {
 		case NORMAL:
 		case VERBOSE:
 			printf (
-_("Link: error: unknown destination job/jobset Id `%s' for the link between Ids `%s' and `%s'\n"),
-				id, job_id_source, id);
+_("Link: error: unknown destination job/jobset Id `%lld' for the link between Ids `%lld' and `%lld'\n"),
+				jobid, jobid_s, jobid);
 			break;
 		default:
 			break;
@@ -884,7 +932,7 @@ _("Link: error: unknown destination job/jobset Id `%s' for the link between Ids
 			default:
 				break;
 		}
-		sql_check_delete1 (	"link", "job_id_destination", id,
+		sql_check_delete1 (	"link", "job_id_destination", jobid,
 					sql_error_logger, NULL);
 	}
 	return 0;
@@ -936,14 +984,17 @@ check_links ()
  * Callback functions for sql_check_id() when an error is detected
  */
 static int
-check_job_host_cb1 (void *data, const char *id, const char *host_id) 
+check_job_host_cb1 (void *data, row_item_t *id, row_item_t *host_id)
 {
+	unsigned long long int jobid;
+
+	jobid = (unsigned long long int) sql_row_item2ll (id);
 	switch (verbose_level) {
 		case NORMAL:
 		case VERBOSE:
 			printf (
-			_("job/host: error: unknown job/jobset Id `%s'\n"),
-				id);
+			_("job/host: error: unknown job/jobset Id `%lld'\n"),
+				jobid);
 			break;
 		default:
 			break;
@@ -957,20 +1008,23 @@ check_job_host_cb1 (void *data, const char *id, const char *host_id)
 			default:
 				break;
 		}
-		sql_check_delete1 (	"job_host", "job_id", id,
+		sql_check_delete1 (	"job_host", "job_id", jobid,
 					sql_error_logger, NULL);
 	}
 	return 0;
 }
 
 static int
-check_job_host_cb2 (void *data, const char *id, const char *job_id) 
+check_job_host_cb2 (void *data, row_item_t *id, row_item_t *job_id)
 {
+	unsigned long long int hostid;
+
+	hostid = (unsigned long long int) sql_row_item2ll (id);
 	switch (verbose_level) {
 		case NORMAL:
 		case VERBOSE:
-			printf (_("job/host: error: unknown host Id `%s'\n"),
-				id);
+			printf (_("job/host: error: unknown host Id `%lld'\n"),
+				hostid);
 			break;
 		default:
 			break;
@@ -984,7 +1038,7 @@ check_job_host_cb2 (void *data, const char *id, const char *job_id)
 			default:
 				break;
 		}
-		sql_check_delete1 (	"job_host", "host_id", id,
+		sql_check_delete1 (	"job_host", "host_id", hostid,
 					sql_error_logger, NULL);
 	}
 	return 0;
@@ -1037,14 +1091,17 @@ check_job_host ()
  * Callback functions for sql_check_id() when an error is detected
  */
 static int
-check_job_environment_cb1 (void *data, const char *id, const char *env_id) 
+check_job_environment_cb1 (void *data, row_item_t *id, row_item_t *env_id)
 {
+	unsigned long long int jobid;
+
+	jobid = (unsigned long long int) sql_row_item2ll (id);
 	switch (verbose_level) {
 		case NORMAL:
 		case VERBOSE:
 			printf (
-		_("job/environment: error: unknown job/jobset Id `%s'\n"),
-				id);
+		_("job/environment: error: unknown job/jobset Id `%lld'\n"),
+				jobid);
 			break;
 		default:
 			break;
@@ -1058,20 +1115,23 @@ check_job_environment_cb1 (void *data, const char *id, const char *env_id)
 			default:
 				break;
 		}
-		sql_check_delete1 (	"job_environment", "job_id", id,
+		sql_check_delete1 (	"job_environment", "job_id", jobid,
 					sql_error_logger, NULL);
 	}
 	return 0;
 }
 
 static int
-check_job_environment_cb2 (void *data, const char *id, const char *job_id) 
+check_job_environment_cb2 (void *data, row_item_t *id, row_item_t *job_id)
 {
+	unsigned long long int envid;
+
+	envid = (unsigned long long int) sql_row_item2ll (id);
 	/*
-	 * Ignore the error if the environment ID is 0.  It simply means that 
+	 * Ignore the error if the environment ID is 0.  It simply means that
 	 * the job does not inherit its parent environment
 	 */
-	if (id[0] == '0' && id[1] == '\0') {
+	if (envid == 0) {
 		return 0;
 	}
 
@@ -1079,8 +1139,8 @@ check_job_environment_cb2 (void *data, const char *id, const char *job_id)
 		case NORMAL:
 		case VERBOSE:
 			printf (
-		_("job/environment: error: unknown environment Id `%s'\n"),
-				id);
+		_("job/environment: error: unknown environment Id `%lld'\n"),
+				envid);
 			break;
 		default:
 			break;
@@ -1094,7 +1154,7 @@ check_job_environment_cb2 (void *data, const char *id, const char *job_id)
 			default:
 				break;
 		}
-		sql_check_delete1 (	"job_environment", "env_id", id,
+		sql_check_delete1 (	"job_environment", "env_id", envid,
 					sql_error_logger, NULL);
 	}
 	return 0;
@@ -1148,14 +1208,17 @@ check_job_environment ()
  * Callback functions for sql_check_id() when an error is detected
  */
 static int
-check_host_environment_cb1 (void *data, const char *id, const char *env_id) 
+check_host_environment_cb1 (void *data, row_item_t *id, row_item_t *env_id)
 {
+	unsigned long long int hostid;
+
+	hostid = (unsigned long long int) sql_row_item2ll (id);
 	switch (verbose_level) {
 		case NORMAL:
 		case VERBOSE:
 			printf (
-			_("host/environment: error: unknown host Id `%s'\n"),
-				id);
+			_("host/environment: error: unknown host Id `%lld'\n"),
+				hostid);
 			break;
 		default:
 			break;
@@ -1169,21 +1232,24 @@ check_host_environment_cb1 (void *data, const char *id, const char *env_id)
 			default:
 				break;
 		}
-		sql_check_delete1 (	"host_environment", "host_id", id,
+		sql_check_delete1 (	"host_environment", "host_id", hostid,
 					sql_error_logger, NULL);
 	}
 	return 0;
 }
 
 static int
-check_host_environment_cb2 (void *data, const char *id, const char *host_id) 
+check_host_environment_cb2 (void *data, row_item_t *id, row_item_t *host_id)
 {
+	unsigned long long int envid;
+
+	envid = (unsigned long long int) sql_row_item2ll (id);
 	switch (verbose_level) {
 		case NORMAL:
 		case VERBOSE:
 			printf (
-		_("host/environment: error: unknown environment Id `%s'\n"),
-				id);
+		_("host/environment: error: unknown environment Id `%lld'\n"),
+				envid);
 			break;
 		default:
 			break;
@@ -1197,7 +1263,7 @@ check_host_environment_cb2 (void *data, const char *id, const char *host_id)
 			default:
 				break;
 		}
-		sql_check_delete1 (	"host_environment", "env_id", id,
+		sql_check_delete1 (	"host_environment", "env_id", envid,
 					sql_error_logger, NULL);
 	}
 	return 0;
@@ -1251,14 +1317,17 @@ check_host_environment ()
  * Callback function for sql_check_id() when an error is detected
  */
 static int
-check_environment_var_cb_env (void *data, const char *id, const char *key) 
+check_environment_var_cb_env (void *data, row_item_t *id, row_item_t *key)
 {
+	unsigned long long int envid;
+
+	envid = (unsigned long long int) sql_row_item2ll (id);
 	switch (verbose_level) {
 		case NORMAL:
 		case VERBOSE:
 			printf (
-_("Environment variable: error: unknown environment Id `%s' for variable `%s'\n"),
-				id, key);
+_("Environment variable: error: unknown environment Id `%lld' for variable `%s'\n"),
+				envid, key->value_string);
 			break;
 		default:
 			break;
@@ -1272,7 +1341,7 @@ _("Environment variable: error: unknown environment Id `%s' for variable `%s'\n"
 			default:
 				break;
 		}
-		sql_check_delete1 (	"environment_var", "env_id", id,
+		sql_check_delete1 (	"environment_var", "env_id", envid,
 					sql_error_logger, NULL);
 	}
 	return 0;
@@ -1284,14 +1353,14 @@ _("Environment variable: error: unknown environment Id `%s' for variable `%s'\n"
  * is detected
  */
 static int
-check_environment_var_cb_value (void *data, const char *id, const char *name,
-				const char *value) 
+check_environment_var_cb_value (void *data, unsigned long long int id,
+				const char *name, const char *value)
 {
 	switch (verbose_level) {
 		case NORMAL:
 		case VERBOSE:
 			printf (
-_("Environment variable: error: environment Id `%s': syntax error in value `%s' for variable `%s'\n"),
+_("Environment variable: error: environment Id `%lld': syntax error in value `%s' for variable `%s'\n"),
 				id, value, name);
 			break;
 		default:
@@ -1362,13 +1431,13 @@ check_environment_var ()
  */
 static int
 check_workload_retention_cb (	void *data, const char *table_name,
-				const char *workload)
+				int workload)
 {
 	switch (verbose_level) {
 		case NORMAL:
 		case VERBOSE:
 			printf (
-		_("Workload: error: table `%s_s': obsolete workload `%s'\n"),
+		_("Workload: error: table `%s_s': obsolete workload `%d'\n"),
 				table_name, workload);
 			break;
 		default:
@@ -1383,7 +1452,7 @@ check_workload_retention_cb (	void *data, const char *table_name,
 			default:
 				break;
 		}
-		sql_status_delete (atoi (workload),
+		sql_status_delete (workload,
 				sql_error_logger, NULL);
 	}
 	return 0;
@@ -1436,7 +1505,7 @@ main (int argc, char **argv)
 {
 	const char *prog_name;
 	int ret, global_ret;
-	MYSQL *sql_handler;
+	dbi_conn sql_handler;
 	char *err_msg;
 	time_t t;
 	struct tm stm;
@@ -1586,7 +1655,7 @@ main (int argc, char **argv)
 	 * Database connection
 	 */
 	err_msg = NULL;
-	sql_handler = begin_mysql (&err_msg);
+	sql_handler = begin_sql (&err_msg);
 	if (sql_handler == NULL) {
 		if (err_msg != NULL) {
 			fprintf (stderr,
@@ -1611,7 +1680,7 @@ main (int argc, char **argv)
 
 	ret = check_job_main ();
 	if (ret < 0) {
-		end_mysql ();
+		end_sql ();
 		conf_destroy_srv ();
 		module_exit ();
 		free (lost_found_name);
@@ -1623,7 +1692,7 @@ main (int argc, char **argv)
 
 	ret = check_job_parameters ();
 	if (ret < 0) {
-		end_mysql ();
+		end_sql ();
 		conf_destroy_srv ();
 		module_exit ();
 		free (lost_found_name);
@@ -1633,9 +1702,10 @@ main (int argc, char **argv)
 		global_ret = 2;
 	}
 
+/* Hosts are allowed to have SSL enable without a cert in the DB
 	ret = check_hosts ();
 	if (ret < 0) {
-		end_mysql ();
+		end_sql ();
 		conf_destroy_srv ();
 		module_exit ();
 		free (lost_found_name);
@@ -1644,10 +1714,11 @@ main (int argc, char **argv)
 	if (ret != 0) {
 		global_ret = 2;
 	}
+*/
 
 	ret = check_job_host ();
 	if (ret < 0) {
-		end_mysql ();
+		end_sql ();
 		conf_destroy_srv ();
 		module_exit ();
 		free (lost_found_name);
@@ -1659,7 +1730,7 @@ main (int argc, char **argv)
 
 	ret = check_calendars ();
 	if (ret < 0) {
-		end_mysql ();
+		end_sql ();
 		conf_destroy_srv ();
 		module_exit ();
 		free (lost_found_name);
@@ -1671,7 +1742,7 @@ main (int argc, char **argv)
 
 	ret = check_constraint_files ();
 	if (ret < 0) {
-		end_mysql ();
+		end_sql ();
 		conf_destroy_srv ();
 		module_exit ();
 		free (lost_found_name);
@@ -1683,7 +1754,7 @@ main (int argc, char **argv)
 
 	ret = check_links ();
 	if (ret < 0) {
-		end_mysql ();
+		end_sql ();
 		conf_destroy_srv ();
 		module_exit ();
 		free (lost_found_name);
@@ -1695,7 +1766,7 @@ main (int argc, char **argv)
 
 	ret = check_environment_var ();
 	if (ret < 0) {
-		end_mysql ();
+		end_sql ();
 		conf_destroy_srv ();
 		module_exit ();
 		free (lost_found_name);
@@ -1707,7 +1778,7 @@ main (int argc, char **argv)
 
 	ret = check_job_environment ();
 	if (ret < 0) {
-		end_mysql ();
+		end_sql ();
 		conf_destroy_srv ();
 		module_exit ();
 		free (lost_found_name);
@@ -1719,7 +1790,7 @@ main (int argc, char **argv)
 
 	ret = check_host_environment ();
 	if (ret < 0) {
-		end_mysql ();
+		end_sql ();
 		conf_destroy_srv ();
 		module_exit ();
 		free (lost_found_name);
@@ -1731,9 +1802,9 @@ main (int argc, char **argv)
 
 	ret = check_workload_retention (
 		(retention_completed > retention_failed)
-			? retention_completed :retention_failed); 
+			? retention_completed :retention_failed);
 	if (ret < 0) {
-		end_mysql ();
+		end_sql ();
 		conf_destroy_srv ();
 		module_exit ();
 		free (lost_found_name);
@@ -1744,7 +1815,7 @@ main (int argc, char **argv)
 	}
 
 	/* Destroy the database handler */
-	end_mysql ();
+	end_sql ();
 
 
 	/* Destroy the configuration file object */
