@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
+   Copyright (C) 2007-2010 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -63,10 +63,11 @@
 #include <conf.h>
 #include <conf_srv.h>
 #include <module.h>
-#include <net_module_name.h>
 #include <net_utils.h>
 #include <net_utils_sock.h>
+#include <net_utils_ssl.h>
 #include <sql_hosts.h>
+#include <lib_functions.h>
 
 static const char *configuration_file = NULL;
 
@@ -103,8 +104,8 @@ NULL
 	assert (prog_name != NULL && configuration_file != NULL);
 #endif
 
-	fprintf (stderr, _("Usage: %s [OPTION]... HOST\n"), prog_name);
-						
+	fprintf (stderr, _("Usage: %s [OPTION]... HOST[:PORT]\n"), prog_name);
+
 	for (i = 0; msg[i] != NULL; i++) {
 		if (msg[i][0] == '~') {
 			fputs (	_("The default configuration file is "),
@@ -128,7 +129,7 @@ NULL
  * Error callback function for the sql_host_get_main_row_names() function
  */
 static void
-sql_error_logger (void *data, const char *msg, unsigned int err_code)
+sql_error_logger (void *data, const char *msg, int err_code)
 {
 	if (msg != NULL) {
 		fputs (msg, stderr);
@@ -136,7 +137,7 @@ sql_error_logger (void *data, const char *msg, unsigned int err_code)
 	}
 	else {
 		fputs (
-		_("Database error while retrieving the client host details\n"),
+		_("Database error while retrieving the agent host details\n"),
 			stderr);
 	}
 }
@@ -150,18 +151,17 @@ sql_error_logger (void *data, const char *msg, unsigned int err_code)
 int
 main (int argc, char **argv)
 {
-	const char *server_key, *server_crt;
+	const char *server_key, *server_crt, *ca_crt, *ca_crl;
 	const char *prog_name;
 	char quick_random;
-	char *agent_name, *module, *result_msg;
-	unsigned int result_msg_len;
+	char *agent_name, *result_msg, *port;
 	char **names;
-	char **host_details;
-	unsigned long int *host_details_len;
-	int ret, module_len, ret_read;
-	MYSQL *sql_handler;
+	row_item_t *host_details;
+	int ret, ret_read;
+	dbi_conn sql_handler;
 	char *err_msg;
 	schedwi_BIO *b;
+	char *s = "[ \"version\" ]";
 
 #if HAVE_GETOPT_LONG
 	int option_index;
@@ -261,18 +261,31 @@ main (int argc, char **argv)
 		return 1;
 	}
 
-	/*
-	 * Initialize the network
-	 */
-	ret = conf_get_param_string ("SSLServerCertificateFile", &server_crt);
-	ret += conf_get_param_string (	"SSLServerCertificateKeyFile",
-					&server_key);
+	ret =  conf_get_param_string ("SSLCACertificateFile", &ca_crt);
+	ret += conf_get_param_string ("SSLCertificateFile", &server_crt);
+	ret += conf_get_param_string ("SSLCertificateKeyFile", &server_key);
+	ret += conf_get_param_string ("SSLCACRLFile", &ca_crl);
 	ret += conf_get_param_bool ("SSLQuickRandom", &quick_random);
 #if HAVE_ASSERT_H
 	assert (ret == 0);
 #endif
 
-	if (net_init (server_crt, server_key, quick_random) != 0) {
+	/*
+	 * Initialize the GnuTLS layer
+	 */
+	if (net_init_gnutls (quick_random) != 0) {
+		conf_destroy_srv ();
+		module_exit ();
+		return 1;
+	}
+
+	/*
+	 * Initialize the network
+	 */
+	if (net_init (	server_crt, server_key, ca_crt, ca_crl,
+			 quick_random) != 0)
+	{
+		net_destroy ();
 		conf_destroy_srv ();
 		module_exit ();
 		return 1;
@@ -282,12 +295,12 @@ main (int argc, char **argv)
 	 * Database connection
 	 */
 	err_msg = NULL;
-	sql_handler = begin_mysql (&err_msg);
+	sql_handler = begin_sql (&err_msg);
 	if (sql_handler == NULL) {
 		if (err_msg != NULL) {
-			fprintf (	stderr,
-					_("Failed to connect to database: %s\n"),
-					err_msg);
+			fprintf (stderr,
+				_("Failed to connect to database: %s\n"),
+				err_msg);
 			free (err_msg);
 		}
 		else {
@@ -300,8 +313,8 @@ main (int argc, char **argv)
 	}
 
 	/* Retrieve all the names of the agent from the DNS */
-	if (build_client_names_from_name (agent_name, &names) != 0) {
-		end_mysql ();
+	if (build_client_names_from_name (agent_name, &names, &port) != 0) {
+		end_sql ();
 		conf_destroy_srv ();
 		module_exit ();
 		net_destroy ();
@@ -309,65 +322,46 @@ main (int argc, char **argv)
 	}
 
 	/* Retrieve the agent details from the database */
-	if (sql_host_get_main_row_names (names, 
-					&host_details, &host_details_len,
+	if (sql_host_get_main_row_names (names, port, &host_details,
 					sql_error_logger, NULL) != 0)
 	{
+		free (port);
 		destroy_client_names (names);
-		end_mysql ();
+		end_sql ();
 		conf_destroy_srv ();
 		module_exit ();
 		net_destroy ();
 		return 1;
 	}
+	free (port);
 	destroy_client_names (names);
-	end_mysql ();
+	end_sql ();
 
-	/* Compose the module name */
-	module = NULL;
-	module_len = build_module_name ("version", &module);
-	if (module_len < 0) {
-		sql_free_row (host_details);
-		free (host_details_len);
-		conf_destroy_srv ();
-		module_exit ();
-		net_destroy ();
-		fputs (_("Memory allocation error\n"), stderr);
-		return 1;
-	}
-
-	/* Establish the connection to the client */
-	b = net_client (host_details[2],  /* TCP port */
-			host_details[1],  /* Client host name */
-			(host_details[3][0] == '0') ? 0: 1, /* SSL */
-			host_details[4],  /* Certificate */
-			(unsigned int)(host_details_len[4]));
-	free (host_details_len);
+	/* Establish the connection to the agent */
+	b = net_client (host_details[2].value_string,  /* TCP port */
+			host_details[1].value_string,  /* Agent host name */
+			(char)sql_row_item2ll (&(host_details[3])),
+			host_details[4].value_string,  /* Certificate */
+			(unsigned int)(host_details[4].len));
 	sql_free_row (host_details);
 	if (b == NULL) {
-		free (module);
 		conf_destroy_srv ();
 		module_exit ();
 		net_destroy ();
 		return 1;
 	}
 
-	if (	   net_write (b, module, (int)module_len) != 0
-		|| net_write_eof (b) != 0)
-	{
+	if (net_write (b, s, schedwi_strlen (s)) != 0) {
 		net_close (b);
-		free (module);
 		conf_destroy_srv ();
 		module_exit ();
 		net_destroy ();
 		return 1;
 	}
-	free (module);
 
 	/* Read the result */
 	result_msg = NULL;
-	result_msg_len = 0;
-	ret_read = net_read (b, &result_msg, &result_msg_len);
+	ret_read = net_read_result (b, &result_msg);
 	net_close (b);
 
 	/* System error */
@@ -383,7 +377,6 @@ main (int argc, char **argv)
 
 	/* Success */
 	if (result_msg != NULL) {
-		result_msg[ret_read] = '\0';
 		printf (_("The agent v%s is running.\n"), result_msg);
 		free (result_msg);
 	}
