@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
+   Copyright (C) 2007-2010 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -17,7 +17,7 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-/* sql_common.c -- Useful MYSQL functions */
+/* sql_common.c -- Useful SQL functions */
 
 #include <schedwi.h>
 
@@ -37,139 +37,91 @@
 #include <stdio.h>
 #endif
 
+#if HAVE_TIME_H
+#include <time.h>
+#endif
+
 #if HAVE_ASSERT_H
 #include <assert.h>
 #endif
 
 #include <pthread.h>
 
-#include <mysql/mysql.h>
 #include <lib_functions.h>
 #include <conf.h>
 #include <sql_common.h>
 
-
-static MYSQL *sql = NULL;
+static dbi_conn sql = NULL;
 static int nb_handlers = 0;
 
 
 /*
- * Mutex to protect the SQL request (around mysql_query()
- * and mysql_store_result()
+ * Mutex to protect libdbi requests (libdbi is not yet thread-safe)
  */
 static pthread_mutex_t lock = PTHREAD_MUTEX_INITIALIZER;
 
 
 /*
- * Establish a database connection.
- *
- * Return:
- *   0 --> No error.  The static sql handler is opened
- *  -1 --> Error.  If err_msg is not NULL, it is set with the error message
- *         (it must be freed by the caller)
- */
-static int
-init_db_connection (char **err_msg)
-{
-	const char *host, *user, *passwd, *db, *unix_socket;
-	char *msg;
-	long int port;
-	my_bool reconnect = 1;
-
-	/* Get the connection parameter from the configuration file */
-	host = user = passwd = db = unix_socket = NULL;
-	port = 0;
-	conf_get_param_string ("DB_HOST", &host);
-	conf_get_param_string ("DB_USER", &user);
-	conf_get_param_string ("DB_PASSWD", &passwd);
-	conf_get_param_string ("DB_DATABASE", &db);
-	conf_get_param_int ("DB_PORT", &port);
-	conf_get_param_string ("DB_UNIXSOCKET", &unix_socket);
-
-	sql = mysql_init (NULL);
-	if (sql == NULL) {
-		if (err_msg != NULL) {
-			msg = _("Memory allocation error");
-			*err_msg = (char *) malloc (schedwi_strlen (msg) + 1);
-			if (*err_msg != NULL) {
-				strcpy (*err_msg, msg);
-			}
-		}
-		return -1;
-	}
-
-	mysql_options (sql, MYSQL_READ_DEFAULT_GROUP, PACKAGE);
-	mysql_options (sql, MYSQL_OPT_RECONNECT, &reconnect);
-
-	if (mysql_real_connect (sql, host, user, passwd, db,
-				(unsigned int)port, unix_socket, 0) == NULL)
-	{
-		compose_error_message (	_("Failed to connect to database"),
-					err_msg);
-		mysql_close (sql);
-		sql = NULL;
-		return -1;
-	}
-
-	/* For MySQL prior to 5.0.19 */
-	mysql_options (sql, MYSQL_OPT_RECONNECT, &reconnect);
-	return 0;
-}
-
-
-/*
- * Free the database connection
- */
-void
-destroy_db_connection ()
-{
-	if (sql != NULL) {
-		mysql_close (sql);
-		sql = NULL;
-		nb_handlers = 0;
-	}
-}
-
-
-/*
- * Get a MYSQL handler
+ * Get a libdbi SQL handler
  *
  * Return:
  *    The handler or
  *    NULL in case of error.  If err_msg is not NULL, it is set with the error
  *    message (it must be freed by the caller)
  */
-MYSQL *
-begin_mysql (char **err_msg)
+dbi_conn
+begin_sql (char **err_msg)
 {
-	if (sql == NULL || nb_handlers <= 0) {
-		if (init_db_connection (err_msg) != 0) {
-			return NULL;
+	pthread_mutex_lock (&lock);
+	if (sql != NULL && nb_handlers > 0) {
+		/*
+		 * As there is a memory leak bug in the SQLite/SQLite3 drivers
+		 * in the dbi_conn_ping() function (in dbi_conn_ping()),  it
+		 * cannot be used and we suppose that the DB is still alive
+		 */
+		if (is_sqlite () || dbi_conn_ping (sql) != 0) {
+			/* Connection still alive */
+			nb_handlers++;
+			pthread_mutex_unlock (&lock);
+			return sql;
 		}
+	}
+	sql = schedwi_dbi_init (err_msg);
+	if (sql == NULL) {
+		pthread_mutex_unlock (&lock);
+		return NULL;
+	}
+	if (nb_handlers < 0) {
 		nb_handlers = 1;
 	}
 	else {
 		nb_handlers++;
 	}
+	pthread_mutex_unlock (&lock);
 	return sql;
 }
 
 
 /*
- * Free a MYSQL handler
+ * Free a libdbi SQL handler
  */
 void
-end_mysql ()
+end_sql ()
 {
 	nb_handlers--;
 	if (sql != NULL && nb_handlers <= 0) {
-		destroy_db_connection ();
+		pthread_mutex_lock (&lock);
+		schedwi_dbi_close (sql);
+		sql = NULL;
+		nb_handlers = 0;
+		pthread_mutex_unlock (&lock);
 	}
 }
 
 
 /*
  * Built an SQL error message
+ * Warning - a mutex must be set before calling this function.
  *
  * Return:
  *   0 --> No error (if not NULL, msg_out contains the error message
@@ -187,7 +139,7 @@ compose_error_message (const char *msg, char **msg_out)
 		return 0;
 	}
 
-	msg_sql = mysql_error (sql);
+	dbi_conn_error(sql, &msg_sql);
 
 	if (msg == NULL) {
 		msg_len = 0;
@@ -220,34 +172,25 @@ compose_error_message (const char *msg, char **msg_out)
 /*
  * Escape the provided string (if the string is NULL, an empty string is
  * returned - ie. the first char is 0 and the string still need to be
- * freed by the caller)
+ * freed by the caller). The returned string is itself in quotes.
+ * Warning - a mutex must be set before calling this function.
  *
  * Return:
  *   The escaped string (to be freed by the caller) or
  *   NULL in case of error
  */
 char *
-sql_escape (MYSQL *sql, const char *s)
+sql_escape (dbi_conn sql, const char *s)
 {
 	char *res;
-	unsigned int len;
 
 #if HAVE_ASSERT_H
 	assert (sql != NULL);
 #endif
 
-	len = (s != NULL) ? schedwi_strlen (s) : 0;
-
-	res = (char *) malloc (len * 2 + 1);
-	if (res == NULL) {
+	if (dbi_conn_quote_string_copy (sql, (s == NULL)?"":s, &res) == 0) {
 		return NULL;
 	}
-	if (s == NULL) {
-		res[0] = '\0';
-	}
-	else {
-		mysql_real_escape_string(sql, res, s, len);
-	}
 	return res;
 }
 
@@ -255,31 +198,29 @@ sql_escape (MYSQL *sql, const char *s)
 /*
  * Escape the provided binary string (if the string is NULL, an empty string is
  * returned - ie. the first char is 0 and the string still need to be
- * freed by the caller)
+ * freed by the caller). The returned string is itself in quotes.
+ * Warning - a mutex must be set before calling this function.
  *
  * Return:
  *   The escaped string (to be freed by the caller) or
  *   NULL in case of error
  */
-static char *
-sql_escape_bin (MYSQL *sql, const char *s, unsigned long int len)
+static unsigned char *
+sql_escape_bin (dbi_conn sql, const unsigned char *s, size_t len)
 {
-	char *res;
+	unsigned char *res;
 
 #if HAVE_ASSERT_H
 	assert (sql != NULL);
 #endif
 
-	res = (char *) malloc (len * 2 + 1);
-	if (res == NULL) {
+	if (dbi_conn_quote_binary_copy (sql,
+					(s == NULL) ? (unsigned char*)"" : s,
+					len,
+					&res) == 0)
+	{
 		return NULL;
 	}
-	if (s == NULL) {
-		res[0] = '\0';
-	}
-	else {
-		mysql_real_escape_string(sql, res, s, len);
-	}
 	return res;
 }
 
@@ -318,6 +259,8 @@ free_args (sql_args_t *args, int len)
 /*
  * Run a database request
  *
+ * The result is stored in the provided result parameter and must be freed
+ * by the caller by dbi_result_free().
  * buf (and buf_len) is used as a working buffer.  If buf is NULL, an internal
  * buffer is used. If *buf is NULL, sql_request() will allocate a buffer for
  * internal use (*buf must be freed by the caller) and *buf (and *buf_len) will
@@ -325,53 +268,58 @@ free_args (sql_args_t *args, int len)
  * can contain a pointer to a malloc()-allocated buffer *buf_len bytes in size.
  * If the buffer is not large enough, it will be resized, updating *buf and
  * *buf_len to reflect the buffer address and size.
+ * Warning - a mutex must be set before calling this function.
  *
  * The variable argument list should contain a sql_type followed by the value.
  * For the sql_type SQL_BIN, the value must be followed by the lenght of the
  * string value.
  * The list is terminated by SQL_END.  For examples:
  *
- *     sql_request (	sql, NULL, NULL, &err,
- *   			"DELETE FROM hosts WHERE id=%ld AND hostname=\"%s\"",
+ *     dbi_result result;
+ *     sql_request (	sql, &result, NULL, NULL, &err,
+ *   			"DELETE FROM hosts WHERE id=%ld AND hostname=%s",
  *   			SQL_INT, 12,
  *   			SQL_STRING, "foo.company.org",
  *   			SQL_END);
+ *     dbi_result_free(result);
  *
  *     img_data = (char *)gdk_pixdata_serialize (pixdata, &len_img_data);
- *     sql_request (	sql, NULL, NULL, &err,
- *			"INSERT INTO imgs (id,data) VALUES (%ld,\"%s\"),
+ *     sql_request (	sql, &result, NULL, NULL, &err,
+ *			"INSERT INTO imgs (id,data) VALUES (%ld,%s)",
  *			SQL_INT, 144,
  *			SQL_BIN, img_data, len_img_data,
  *                      SQL_END);
+ *     dbi_result_free(result);
  *     g_free (img_data);
  *
  * Return:
  *     0 --> No error
- *     1 --> Memory allocation error (if err_msg is not NULL it contains an
+ *    -1 --> Memory allocation error (if err_msg is not NULL it contains an
  *           error message which must be freed by the caller.  *err_msg may be
  *           NULL if there is not enough memory to store the error message)
- * other --> Database error (if err_msg is not NULL it contains an error
+ *    -2 --> Database error (if err_msg is not NULL it contains an error
  *           message which must be freed by the caller.  *err_msg may be NULL
  *           if there is not enough memory to store the error message).
- *           The returned value is the one returned by mysql_errno()
  */
-static unsigned int
-sql_request (	MYSQL *sql, char **buf, unsigned int *buf_len, char **err_msg,
+static int
+sql_request (dbi_conn sql, dbi_result *result,
+		char **buf, unsigned int *buf_len,
+		char **err_msg,
 		const char *fmt, va_list ap)
 {
 	sql_args_t args[MAX_NUM_ARGS];
 	sql_type t;
 	int i, ret;
-	unsigned int sql_ret;
 	char c;
 	char *s;
+	unsigned char *us;
 	unsigned int req_len;
 	char *req;
 	char req_to_free;
-	unsigned long int bin_len;
+	size_t bin_len;
 
 #if HAVE_ASSERT_H
-	assert (sql != NULL && fmt != NULL);
+	assert (sql != NULL && result != NULL && fmt != NULL);
 #endif
 
 	req_len = 0;
@@ -388,7 +336,7 @@ sql_request (	MYSQL *sql, char **buf, unsigned int *buf_len, char **err_msg,
 
 			case SQL_BOOL:
 				c = va_arg (ap, int);
-				args[i].value_bool = (c == 0 || c == '0') ?0:1; 
+				args[i].value_bool = (c == 0 || c == '0') ?0:1;
 				req_len += 1;
 				break;
 
@@ -400,12 +348,12 @@ sql_request (	MYSQL *sql, char **buf, unsigned int *buf_len, char **err_msg,
 					if (err_msg != NULL) {
 						s =_("Memory allocation error");
 						*err_msg = (char *) malloc (
-							schedwi_strlen (s) + 1);
+							schedwi_strlen(s) + 1);
 						if (*err_msg != NULL) {
 							strcpy (*err_msg, s);
 						}
 					}
-					return 1;
+					return -1;
 				}
 				args[i].value_string = s;
 				req_len += schedwi_strlen (s);
@@ -419,10 +367,10 @@ sql_request (	MYSQL *sql, char **buf, unsigned int *buf_len, char **err_msg,
 
 			case SQL_BIN:
 				/* Escape the binary string */
-				s = va_arg (ap, char *);
-				bin_len = va_arg (ap, unsigned long int);
-				s = sql_escape_bin (sql, s, bin_len);
-				if (s == NULL) {
+				us = va_arg (ap, unsigned char *);
+				bin_len = va_arg (ap, size_t);
+				us = sql_escape_bin (sql, us, bin_len);
+				if (us == NULL) {
 					free_args (args, i);
 					if (err_msg != NULL) {
 						s =_("Memory allocation error");
@@ -432,10 +380,10 @@ sql_request (	MYSQL *sql, char **buf, unsigned int *buf_len, char **err_msg,
 							strcpy (*err_msg, s);
 						}
 					}
-					return 1;
+					return -1;
 				}
-				args[i].value_string = s;
-				req_len += schedwi_strlen (s);
+				args[i].value_string = (char *)us;
+				req_len += schedwi_strlen ((char *)us);
 				break;
 
 			default:
@@ -465,7 +413,7 @@ sql_request (	MYSQL *sql, char **buf, unsigned int *buf_len, char **err_msg,
 					strcpy (*err_msg, s);
 				}
 			}
-			return 1;
+			return -1;
 		}
 		req_to_free = 1;
 	}
@@ -487,7 +435,7 @@ sql_request (	MYSQL *sql, char **buf, unsigned int *buf_len, char **err_msg,
 						strcpy (*err_msg, s);
 					}
 				}
-				return 1;
+				return -1;
 			}
 			*buf_len = req_len;
 		}
@@ -651,18 +599,14 @@ sql_request (	MYSQL *sql, char **buf, unsigned int *buf_len, char **err_msg,
 	free_args (args, MAX_NUM_ARGS);
 
 	/* Run the request */
-	if (mysql_query (sql, req) != 0) {
-		if (req_to_free != 0) {
-			free (req);
-		}
-		sql_ret = mysql_errno (sql);
-		compose_error_message (_("Database error"), err_msg);
-		return sql_ret;
-	}
-
+	*result = dbi_conn_query (sql, req);
 	if (req_to_free != 0) {
 		free (req);
 	}
+	if (*result == NULL && dbi_conn_error (sql, NULL)) {
+		compose_error_message (_("Database error"), err_msg);
+		return -2;
+	}
 	return 0;
 }
 
@@ -677,12 +621,18 @@ sql_request (	MYSQL *sql, char **buf, unsigned int *buf_len, char **err_msg,
  * *buf can contain a pointer to a malloc()-allocated buffer *buf_len bytes in
  * size.  If the buffer is not large enough, it will be resized, updating *buf
  * and *buf_len to reflect the buffer address and size.
+ *              id (out) --> ID of the last inserted item
+ *   affected_rows (out) --> Number of affected rows
+ *        tablename (in) --> Name of updated table. This is used only if `id'
+ *                           is not NULL and it is required with some databases
+ *                           (postgresql) to get the last inserted item from
+ *                           a sequence.
  *
  * The variable argument list should contain a sql_type followed by the value.
  * The list is terminated by SQL_END.  For example:
  *
- *     sql_non_select (	NULL, NULL, &err, NULL, NULL,
- *   			"DELETE FROM hosts WHERE id=%ld AND hostname=\"%s\"",
+ *     sql_non_select (	NULL, NULL, &err, NULL, NULL, "hosts",
+ *   			"DELETE FROM hosts WHERE id=%ld AND hostname=%s",
  *   			SQL_INT, 12,
  *   			SQL_STRING, "foo.company.org",
  *   			SQL_END);
@@ -691,62 +641,79 @@ sql_request (	MYSQL *sql, char **buf, unsigned int *buf_len, char **err_msg,
  *     0 --> No error (if id is not NULL it contains the value of the last ID
  *           inserted or updated in the database.  This is only relevant for
  *           INSERT and UPDATE requests)
- *     1 --> Memory allocation error (if err_msg is not NULL it contains an
+ *    -1 --> Memory allocation error (if err_msg is not NULL it contains an
  *           error message which must be freed by the caller.  *err_msg may be
  *           NULL if there is not enough memory to store the error message)
- *     2 --> Database connection error (if err_msg is not NULL it contains an
- *           error message which must be freed by the caller.  *err_msg may be
- *           NULL if there is not enough memory to store the error message)
- * other --> Database error (if err_msg is not NULL it contains an error
+ *    -2 --> Database error (if err_msg is not NULL it contains an error
  *           message which must be freed by the caller.  *err_msg may be NULL
  *           if there is not enough memory to store the error message).
- *           The returned value is the one returned by mysql_errno()
  */
-unsigned int
+int
 sql_non_select (	char **buf, unsigned int *buf_len, char **err_msg,
-			unsigned long int *id,
+			unsigned long long int *id,
 			unsigned long long int *affected_rows,
+			const char *tablename,
 			const char *fmt, ...)
 {
 	va_list ap;
-	MYSQL *sql;
-	unsigned int ret;
+	int ret;
+	dbi_conn sql;
+	dbi_result result;
+	char *sequence, *s;
 
 #if HAVE_ASSERT_H
 	assert (fmt != NULL);
 #endif
 
 	/* Connect to the database */
-	sql = begin_mysql (err_msg);
+	sql = begin_sql (err_msg);
 	if (sql == NULL) {
-		return 2;
+		return -2;
 	}
 
 	/* Run the request */
 	pthread_mutex_lock (&lock);
 	va_start (ap, fmt);
-	ret = sql_request (sql, buf, buf_len, err_msg, fmt, ap);
+	ret = sql_request (sql, &result, buf, buf_len, err_msg, fmt, ap);
 	va_end (ap);
 
 	if (ret != 0) {
 		pthread_mutex_unlock (&lock);
-		end_mysql ();
+		end_sql ();
 		return ret;
 	}
 
 	/* Get the number of affected rows */
 	if (affected_rows != NULL) {
-		*affected_rows = (unsigned long long int)
-					mysql_affected_rows (sql);
+		*affected_rows = dbi_result_get_numrows_affected (result);
 	}
 
 	/* Get the last id inserted in the database */
 	if (id != NULL) {
-		*id = (unsigned long int) mysql_insert_id (sql);
+		*id = dbi_conn_sequence_last (sql, NULL);
+		/*
+		 * If the dbi_conn_sequence_last() failed, try to use the
+		 * sequence which is called <tablename>_<column>_seq
+		 */
+		if (*id == 0 && tablename != NULL && tablename[0] != '\0') {
+			s = "_id_seq";
+			sequence = (char *) malloc (schedwi_strlen (tablename)
+						+ schedwi_strlen (s) + 1);
+			if (sequence != NULL) {
+				strcpy (sequence, tablename);
+				strcat (sequence, s);
+				*id = dbi_conn_sequence_last (sql, sequence);
+				free (sequence);
+			}
+		}
+	}
+
+	if (result != NULL) {
+		dbi_result_free (result);
 	}
 	pthread_mutex_unlock (&lock);
-	
-	end_mysql ();
+	end_sql ();
+
 	return 0;
 }
 
@@ -755,19 +722,70 @@ sql_non_select (	char **buf, unsigned int *buf_len, char **err_msg,
  * Free a result row
  */
 void
-sql_free_row (char **row)
+sql_free_row (row_item_t *row)
 {
 	int i;
 
 	if (row != NULL) {
-		for (i = 0; row[i] != NULL; i++) {
-			free (row[i]);
+		for (i = 0; row[i].type != RES_END; i++) {
+			if (	   row[i].type == RES_STRING
+				|| row[i].type == RES_BIN)
+			{
+				free (row[i].value_string);
+			}
 		}
 		free (row);
 	}
 }
 
 
+/*
+ * Return the int value in the provided row item (whatever the type is - signed
+ * or unsigned)
+ */
+long long int
+sql_row_item2ll (const row_item_t *item)
+{
+	if (item != NULL && (item->type == RES_INT || item->type == RES_UINT))
+	{
+		return (long long int)((item->type == RES_INT)
+							? item->value_int
+							: item->value_uint);
+	}
+	return 0;
+}
+
+
+/*
+ * Compare two row_item_t objects
+ *
+ * Return:
+ *   0 --> the two object have the same value
+ *  -1 --> they differ
+ */
+char
+sql_row_item_cmp (const row_item_t *a, const row_item_t *b)
+{
+	if (a == NULL) {
+		return (b == NULL) ? 0 : -1;
+	}
+	if (b == NULL || a->type != b->type) {
+		return -1;
+	}
+	if (	   a->type == RES_END
+		|| (a->type == RES_TIME && a->value_time == b->value_time)
+		|| (	   (a->type == RES_INT || a->type == RES_UINT)
+			&& sql_row_item2ll (a) == sql_row_item2ll (b))
+		|| (	   (a->type == RES_STRING || a->type == RES_BIN)
+			&& a->len == b->len
+			&& ! memcmp (a->value_string, b->value_string, b->len)))
+	{
+		return 0;
+	}
+	return -1;
+}
+
+
 /*
  * Run a select database request
  *
@@ -778,14 +796,17 @@ sql_free_row (char **row)
  * can contain a pointer to a malloc()-allocated buffer *buf_len bytes in size.
  * If the buffer is not large enough, it will be resized, updating *buf and
  * *buf_len to reflect the buffer address and size.
+ * The returned `row' list is a list of the rows. Each row is an array of
+ * row_item_t. The last item of the array has the `type' field set
+ * to `RES_END'.
  *
  * The variable argument list should contain a sql_type followed by the value.
  * For the sql_type SQL_BIN, the value must be followed by the lenght of the
  * string value.
  * The list is terminated by SQL_END.  For example:
  *
- *         sql_select (	NULL, NULL, &err, NULL, &list_row, NULL,
- *   			"SELECT FROM hosts WHERE id=%ld AND hostname=\"%s\"",
+ *         sql_select (	NULL, NULL, &err, NULL, &list_row,
+ *   			"SELECT FROM hosts WHERE id=%ld AND hostname=%s",
  *   			SQL_INT, 12,
  *   			SQL_STRING, "foo.company.org",
  *   			SQL_END);
@@ -794,250 +815,303 @@ sql_free_row (char **row)
  *     0 --> No error.  rows contains the list of rows and must be freed by the
  *           caller by
  *              lwc_delLL (rows, (void (*)(const void *)) sql_free_row);
- *           If not NULL, fields_len contains the list of fields len and must
- *           be freed by the caller by
- *              lwc_delLL (fields_len, (void (*)(const void *)) free);
- *     1 --> Memory allocation error (if err_msg is not NULL it contains an
+ *           If not NULL, num_columns contains the number of columns.
+ *    -1 --> Memory allocation error (if err_msg is not NULL it contains an
  *           error message which must be freed by the caller.  *err_msg may be
  *           NULL if there is not enough memory to store the error message)
- *     2 --> Database connection error (if err_msg is not NULL it contains an
- *           error message which must be freed by the caller.  *err_msg may be
- *           NULL if there is not enough memory to store the error message)
- * other --> Database error (if err_msg is not NULL it contains an error
+ *    -2 --> Database error (if err_msg is not NULL it contains an error
  *           message which must be freed by the caller.  *err_msg may be NULL
  *           if there is not enough memory to store the error message).
- *           The returned value is the one returned by mysql_errno()
  */
-unsigned int
+int
 sql_select (	char **buf, unsigned int *buf_len, char **err_msg,
-		unsigned int *num_columns, lwc_LL **rows, lwc_LL **fields_len,
+		unsigned int *num_columns, lwc_LL **rows,
 		const char *fmt, ...)
 {
-	va_list ap;
-	MYSQL *sql;
-	lwc_LL *list, *list_len;
-	char *s;
-	MYSQL_RES *result;
-        MYSQL_ROW row;
-	unsigned int cols, i, ret;
-	char **row_array;
-	unsigned long int *lengths, *len_array;
+    va_list ap;
+    dbi_conn sql;
+    dbi_result result;
+    lwc_LL *list;
+    char *s;
+    unsigned int cols, i, attrs;
+    int ret;
+    row_item_t *row_array;
 
 #if HAVE_ASSERT_H
-	assert (rows != NULL && fmt != NULL);
+    assert (rows != NULL && fmt != NULL);
 #endif
 
-	/* Allocate memory for the linked list */
-	list = lwc_newLL ();
-	if (list == NULL) {
-		if (err_msg != NULL) {
-			s = _("Memory allocation error");
-			*err_msg = (char *) malloc (schedwi_strlen (s) + 1);
-			if (*err_msg != NULL) {
-				strcpy (*err_msg, s);
-			}
-		}
-		return 1;
-	}
+    /* Allocate memory for the linked list */
+    list = lwc_newLL ();
+    if (list == NULL) {
+        if (err_msg != NULL) {
+            s = _("Memory allocation error");
+            *err_msg = (char *) malloc (schedwi_strlen (s) + 1);
+            if (*err_msg != NULL) {
+                strcpy (*err_msg, s);
+            }
+        }
+        return -1;
+    }
 
-	/* Allocate memory for the field lenghts linked list */
-	if (fields_len != NULL) {
-		list_len = lwc_newLL ();
-		if (list_len == NULL) {
-			lwc_delLL (list, NULL);
-			if (err_msg != NULL) {
-				s = _("Memory allocation error");
-				*err_msg = (char *) malloc (schedwi_strlen (s)
-								+ 1);
-				if (*err_msg != NULL) {
-					strcpy (*err_msg, s);
-				}
-			}
-			return 1;
-		}
-	}
-	else {
-		list_len = NULL;
-	}
-		
+    /* Connect to the database */
+    sql = begin_sql (err_msg);
+    if (sql == NULL) {
+        lwc_delLL (list, NULL);
+        return -2;
+    }
 
-	/* Connect to the database */
-	sql = begin_mysql (err_msg);
-	if (sql == NULL) {
-		lwc_delLL (list, NULL);
-		lwc_delLL (list_len, NULL);
-		return 2;
-	}
+    /* Run the request */
+    pthread_mutex_lock (&lock);
+    va_start (ap, fmt);
+    ret = sql_request (sql, &result, buf, buf_len, err_msg, fmt, ap);
+    va_end (ap);
 
-	/* Run the request */
-	pthread_mutex_lock (&lock);
-	va_start (ap, fmt);
-	ret = sql_request (sql, buf, buf_len, err_msg, fmt, ap);
-	va_end (ap);
+    if (ret != 0) {
+        pthread_mutex_unlock (&lock);
+        end_sql ();
+        lwc_delLL (list, NULL);
+        return ret;
+    }
 
-	if (ret != 0) {
-		pthread_mutex_unlock (&lock);
-		end_mysql ();
-		lwc_delLL (list, NULL);
-		lwc_delLL (list_len, NULL);
-		return ret;
-	}
+    /* Number of columns */
+    cols = dbi_result_get_numfields (result);
+    if (cols == DBI_FIELD_ERROR) {
+        compose_error_message (_("Database error"), err_msg);
+        dbi_result_free (result);
+        pthread_mutex_unlock (&lock);
+        end_sql ();
+        lwc_delLL (list, NULL);
+        return -2;
+    }
 
-	/* Get the result */
-	result = mysql_store_result (sql);
-	pthread_mutex_unlock (&lock);
-	if (result == NULL) {
-		ret = mysql_errno(sql);
-		compose_error_message (_("SELECT"), err_msg);
-		end_mysql ();
-		lwc_delLL (list, NULL);
-		lwc_delLL (list_len, NULL);
-		return ret;
-	}
+    /* Copy each returned row */
+    while (dbi_result_next_row (result)) {
 
-	/* Copy each returned row */
-	cols = mysql_num_fields (result);
-	while ((row = mysql_fetch_row (result)) != NULL) {
+        /* Allocate a new row array */
+        row_array = (row_item_t *) malloc (sizeof (row_item_t) * (cols + 1));
+        if (row_array == NULL) {
+            dbi_result_free (result);
+            pthread_mutex_unlock (&lock);
+            end_sql ();
+            lwc_delLL (list, (void (*)(const void *))sql_free_row);
+            if (err_msg != NULL) {
+                s = _("Memory allocation error");
+                *err_msg = (char *) malloc (schedwi_strlen (s) + 1);
+                if (*err_msg != NULL) {
+                    strcpy (*err_msg, s);
+                }
+            }
+            return -1;
+        }
+        row_array[cols].type = RES_END;
 
-		/* Allocate a new row array */
-		row_array = (char **) malloc (sizeof (char *) * (cols + 1));
-		if (row_array == NULL) {
-			mysql_free_result(result);
-			end_mysql ();
-			lwc_delLL (list, (void (*)(const void *))sql_free_row);
-			lwc_delLL (list_len, (void (*)(const void *))free);
-			if (err_msg != NULL) {
-				s = _("Memory allocation error");
-				*err_msg = (char *) malloc (schedwi_strlen (s)
-								+ 1);
-				if (*err_msg != NULL) {
-					strcpy (*err_msg, s);
-				}
-			}
-			return 1;
-		}
-		row_array[cols] = NULL;
+        /* Copy the returned row */
+        for (i = 0; i < cols; i++) {
 
-		/* Retrieve the length of the fields in the row */
-		lengths = mysql_fetch_lengths (result);
+            attrs = dbi_result_get_field_attribs_idx (result, i + 1);
+            switch (dbi_result_get_field_type_idx (result, i + 1)) {
 
-		/* Allocate a new row length array */
-		if (fields_len != NULL) {
-			len_array = (unsigned long int *) malloc (
-					sizeof (unsigned long int) * cols);
-			if (len_array == NULL) {
-				mysql_free_result(result);
-				end_mysql ();
-				lwc_delLL (list, (void (*)(const void *))
-								sql_free_row);
-				lwc_delLL (list_len, (void (*)(const void *))
-								free);
-				if (err_msg != NULL) {
-					s = _("Memory allocation error");
-					*err_msg = (char *) malloc (
-							schedwi_strlen (s) + 1);
-					if (*err_msg != NULL) {
-						strcpy (*err_msg, s);
-					}
-				}
-				return 1;
-			}
-			schedwi_memcpy (len_array, lengths,
-					sizeof (unsigned long int) * cols);
-			if (lwc_addEndLL (list_len, len_array) != 0) {
-				mysql_free_result(result);
-				end_mysql ();
-				lwc_delLL (list, (void (*)(const void *))
-								sql_free_row);
-				lwc_delLL (list_len, (void (*)(const void *))
-								free);
-				free (len_array);
-				if (err_msg != NULL) {
-					s = _("Memory allocation error");
-					*err_msg = (char *) malloc (
-							schedwi_strlen (s) + 1);
-					if (*err_msg != NULL) {
-						strcpy (*err_msg, s);
-					}
-				}
-				return 1;
-			}
-		}
+                case DBI_TYPE_INTEGER:
+                    if (attrs & DBI_INTEGER_UNSIGNED) {
+                        row_array[i].type = RES_UINT;
+                        if (attrs & DBI_INTEGER_SIZE1) {
+                            row_array[i].value_uint =
+                                dbi_result_get_uchar_idx (result, i + 1);
+                        }
+                        else
+                        if (attrs & DBI_INTEGER_SIZE2) {
+                            row_array[i].value_uint =
+                                dbi_result_get_ushort_idx (result, i + 1);
+                        }
+                        else
+                        if (attrs & (DBI_INTEGER_SIZE3|DBI_INTEGER_SIZE4)) {
+                            row_array[i].value_uint =
+                                dbi_result_get_uint_idx (result, i + 1);
+                        }
+                        else {
+                            row_array[i].value_uint =
+                                dbi_result_get_ulonglong_idx (result, i + 1);
+                        }
+                    }
+                    else {
+                        row_array[i].type = RES_INT;
+                        if (attrs & DBI_INTEGER_SIZE1) {
+                            row_array[i].value_int =
+                                dbi_result_get_char_idx (result, i + 1);
+                        }
+                        else
+                        if (attrs & DBI_INTEGER_SIZE2) {
+                            row_array[i].value_int =
+                                dbi_result_get_short_idx (result, i + 1);
+                        }
+                        else
+                        if (attrs & (DBI_INTEGER_SIZE3|DBI_INTEGER_SIZE4)) {
+                            row_array[i].value_int =
+                                dbi_result_get_int_idx (result, i + 1);
+                        }
+                        else {
+                            row_array[i].value_int =
+                                dbi_result_get_longlong_idx (result, i + 1);
+                        }
+                    }
+                    break;
 
-		/* Copy the returned row */
-		for (i = 0; i < cols; i++) {
-			row_array[i] = (char *) malloc (lengths[i] + 1);
-			if (row_array[i] == NULL) {
-				mysql_free_result(result);
-				end_mysql ();
-				lwc_delLL (list, (void (*)(const void *))
-								sql_free_row);
-				lwc_delLL (list_len, (void (*)(const void *))
-								free);
-				sql_free_row (row_array);
-				if (err_msg != NULL) {
-					s = _("Memory allocation error");
-					*err_msg = (char *) malloc (
-							schedwi_strlen (s) + 1);
-					if (*err_msg != NULL) {
-						strcpy (*err_msg, s);
-					}
-				}
-				return 1;
-			}
-			schedwi_memcpy (row_array[i], row[i], lengths[i]);
-			row_array[i][lengths[i]] = '\0';
-		}
+                case DBI_TYPE_DECIMAL:
+                    if (attrs & DBI_DECIMAL_UNSIGNED) {
+                        row_array[i].type = RES_UINT;
+                        if (attrs & DBI_DECIMAL_SIZE4) {
+                            row_array[i].value_uint =
+                                dbi_result_get_uint_idx (result, i + 1);
+                        }
+                        else {
+                            row_array[i].value_uint =
+                                dbi_result_get_ulonglong_idx (result, i + 1);
+                        }
+                    }
+                    else {
+                        row_array[i].type = RES_INT;
+                        if (attrs & DBI_DECIMAL_SIZE4) {
+                            row_array[i].value_int =
+                                dbi_result_get_int_idx (result, i + 1);
+                        }
+                        else {
+                            row_array[i].value_int =
+                                dbi_result_get_longlong_idx (result, i + 1);
+                        }
+                    }
+                    break;
 
-		/* Add the new row to the linked list */
-		if (lwc_addEndLL (list, row_array) != 0) {
-			mysql_free_result(result);
-			end_mysql ();
-			lwc_delLL (list, (void (*)(const void *))sql_free_row);
-			lwc_delLL (list_len, (void (*)(const void *))free);
-			sql_free_row (row_array);
-			if (err_msg != NULL) {
-				s = _("Memory allocation error");
-				*err_msg = (char *) malloc (schedwi_strlen (s)
-								+ 1);
-				if (*err_msg != NULL) {
-					strcpy (*err_msg, s);
-				}
-			}
-			return 1;
-		}
-	}
+                case DBI_TYPE_STRING:
+                    row_array[i].type = RES_STRING;
+                    row_array[i].len  =
+                              dbi_result_get_field_length_idx (result, i + 1);
+                    row_array[i].value_string =
+                              dbi_result_get_string_copy_idx (result, i + 1);
 
-	mysql_free_result(result);
-	end_mysql ();
-	if (num_columns != NULL) {
-		*num_columns = cols;
-	}
-	*rows = list;
-	if (fields_len != NULL) {
-		*fields_len = list_len;
-	}
-	return 0;
+                    if (   (   row_array[i].value_string == NULL
+                            && row_array[i].len > 0)
+                         || strcmp (row_array[i].value_string, "ERROR") == 0)
+                    {
+                        if (row_array[i].value_string != NULL) {
+                            free (row_array[i].value_string);
+                            compose_error_message (_("Database error"),
+                                                   err_msg);
+                        }
+                        else {
+                            if (err_msg != NULL) {
+                                s = _("Memory allocation error");
+                                *err_msg =(char *) malloc(schedwi_strlen(s)+1);
+                                if (*err_msg != NULL) {
+                                    strcpy (*err_msg, s);
+                                }
+                            }
+                        }
+                        dbi_result_free (result);
+                        pthread_mutex_unlock (&lock);
+                        end_sql ();
+                        lwc_delLL (list, (void (*)(const void *))sql_free_row);
+                        row_array[i].type = RES_END;
+                        sql_free_row (row_array);
+                        return -1;
+                    }
+                    break;
+
+                case DBI_TYPE_BINARY:
+                    row_array[i].type = RES_BIN;
+                    row_array[i].len  =
+                              dbi_result_get_field_length_idx (result, i + 1);
+                    row_array[i].value_string = (char *)
+                              dbi_result_get_binary_copy_idx (result, i + 1);
+                    if (   (   row_array[i].value_string == NULL
+                            && row_array[i].len > 0)
+                         || strcmp (row_array[i].value_string, "ERROR") == 0)
+                    {
+                        if (row_array[i].value_string != NULL) {
+                            free (row_array[i].value_string);
+                            compose_error_message (_("Database error"),
+                                                   err_msg);
+                        }
+                        else {
+                            if (err_msg != NULL) {
+                                s = _("Memory allocation error");
+                                *err_msg =(char *) malloc(schedwi_strlen(s)+1);
+                                if (*err_msg != NULL) {
+                                    strcpy (*err_msg, s);
+                                }
+                            }
+                        }
+                        dbi_result_free (result);
+                        pthread_mutex_unlock (&lock);
+                        end_sql ();
+                        lwc_delLL (list, (void (*)(const void *))sql_free_row);
+                        row_array[i].type = RES_END;
+                        sql_free_row (row_array);
+                        return -1;
+                    }
+                    break;
+
+                case DBI_TYPE_DATETIME:
+                    row_array[i].type = RES_TIME;
+                    row_array[i].value_time =
+                              dbi_result_get_datetime_idx (result, i + 1);
+                    break;
+            }
+        }
+
+        /* Add the new row to the linked list */
+        if (lwc_addEndLL (list, row_array) != 0) {
+            dbi_result_free (result);
+            pthread_mutex_unlock (&lock);
+            end_sql ();
+            lwc_delLL (list, (void (*)(const void *))sql_free_row);
+            sql_free_row (row_array);
+            if (err_msg != NULL) {
+                s = _("Memory allocation error");
+                *err_msg = (char *) malloc (schedwi_strlen (s) + 1);
+                if (*err_msg != NULL) {
+                    strcpy (*err_msg, s);
+                }
+            }
+            return -1;
+        }
+    }
+
+    dbi_result_free (result);
+    pthread_mutex_unlock (&lock);
+    end_sql ();
+    if (num_columns != NULL) {
+        *num_columns = cols;
+    }
+    *rows = list;
+    return 0;
 }
 
 
 /*
- * Function to call at the begining of a new thread
+ * Return a SQL standard date/time which represents now + sec (sec can
+ * be negative to compute a date/time in the past)
+ *
+ * Return:
+ *   NULL in case of memory allocation error OR
+ *   the date/string to be freed by the caller by free()
  */
-void
-sql_thread_init ()
+char *
+sql_date_interval (int sec)
 {
-	mysql_thread_init ();
-}
+	char *d;
+	time_t t;
+	struct tm stm;
 
-
-/*
- * Function to call before exiting from a thread
- */
-void
-sql_thread_end ()
-{
-	mysql_thread_end ();
+	d = (char *) malloc (25);
+	if (d == NULL) {
+		return NULL;
+	}
+	time (&t);
+	t += sec;
+	localtime_r (&t, &stm);
+	strftime (d, 24, "%Y-%m-%d %H:%M:%S", &stm);
+	return d;
 }
 
 /*-----------------============== End Of File ==============-----------------*/
