@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
+   Copyright (C) 2007-2010 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -53,44 +53,44 @@
  *    - The value of the provided `field_output' in table `table_source'
  *
  * Return:
+ *     2 --> Erroneous IDs
  *     0 --> No error.
- *     1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
  *           with user_data_error_func as its first parameter and the error
  *           message as the second parameter)
- *     2 --> Erroneous IDs
- * other --> SQL error (if error_func() is not NULL, it is called with
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
  *           user_data_error_func as its first parameter and the error message
  *           as the second parameter)
  */
-unsigned int
+int
 sql_check_id (
 	const char *table_source, const char *field_source,
 	const char *table_destination, const char *field_destination,
 	const char *field_output,
-	int (*callback)(void *, const char *, const char *),
+	int (*callback)(void *,  row_item_t *, row_item_t *),
 	void *user_data,
-	void (*error_func)(void *, const char *, unsigned int),
+	void (*error_func)(void *, const char *, int),
 	void *user_data_error_func)
 {
 	char *err_msg = NULL;
 	lwc_LL *rows;
-	char **row;
-	unsigned int ret;
+	row_item_t *row;
+	int ret;
 
 #if HAVE_ASSERT_H
 	assert (callback != NULL);
 #endif
 
 	/* SQL request */
-	ret = sql_select (NULL, NULL, &err_msg, NULL, &rows, NULL,
+	ret = sql_select (NULL, NULL, &err_msg, NULL, &rows,
 			"SELECT %s,%s FROM %s WHERE %s NOT IN \
 				(SELECT %s FROM %s)",
-			SQL_STRING_NON_ESCAPE, field_source, 
-			SQL_STRING_NON_ESCAPE, field_output, 
-			SQL_STRING_NON_ESCAPE, table_source, 
-			SQL_STRING_NON_ESCAPE, field_source, 
-			SQL_STRING_NON_ESCAPE, field_destination, 
-			SQL_STRING_NON_ESCAPE, table_destination, 
+			SQL_STRING_NON_ESCAPE, field_source,
+			SQL_STRING_NON_ESCAPE, field_output,
+			SQL_STRING_NON_ESCAPE, table_source,
+			SQL_STRING_NON_ESCAPE, field_source,
+			SQL_STRING_NON_ESCAPE, field_destination,
+			SQL_STRING_NON_ESCAPE, table_destination,
 			SQL_END);
 	if (ret != 0) {
 		if (error_func != NULL) {
@@ -103,12 +103,12 @@ sql_check_id (
 	}
 
 	ret = 0;
-	while ((row = (char **) lwc_delStartLL (rows)) != NULL) {
+	while ((row = (row_item_t *) lwc_delStartLL (rows)) != NULL) {
 		/*
 		 * row[0] --> field_source
 		 * row[1] --> field_output
 		 */
-		callback (user_data, row[0], row[1]);
+		callback (user_data, &(row[0]), &(row[1]));
 		sql_free_row (row);
 		ret = 2;
 	}
@@ -128,33 +128,34 @@ sql_check_id (
  *    - The job/jobset description
  *
  * Return:
+ *     2 --> Orphans were found
  *     0 --> No error.
- *     1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
  *           with user_data_error_func as its first parameter and the error
  *           message as the second parameter)
- *     2 --> Orphans were found
- * other --> SQL error (if error_func() is not NULL, it is called with
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
  *           user_data_error_func as its first parameter and the error message
  *           as the second parameter)
  */
-unsigned int
+int
 sql_job_main_check_parent (
-	int (*callback)(void *, const char *, const char *, char, const char *),
+	int (*callback)(void *, unsigned long long int,
+			const char *, char, const char *),
 	void *user_data,
-	void (*error_func)(void *, const char *, unsigned int),
+	void (*error_func)(void *, const char *, int),
 	void *user_data_error_func)
 {
 	char *err_msg = NULL;
 	lwc_LL *rows;
-	char **row;
-	unsigned int ret;
+	row_item_t *row;
+	int ret;
 
 #if HAVE_ASSERT_H
 	assert (callback != NULL);
 #endif
 
 	/* SQL request */
-	ret = sql_select (NULL, NULL, &err_msg, NULL, &rows, NULL,
+	ret = sql_select (NULL, NULL, &err_msg, NULL, &rows,
 			"SELECT id,name,type,description \
 			FROM job_main \
 			WHERE parent <> 0 \
@@ -171,16 +172,18 @@ sql_job_main_check_parent (
 	}
 
 	ret = 0;
-	while ((row = (char **) lwc_delStartLL (rows)) != NULL) {
+	while ((row = (row_item_t *) lwc_delStartLL (rows)) != NULL) {
 		/*
 		 * row[0] --> Job ID
 		 * row[1] --> Name
 		 * row[2] --> Type (0 -> jobset, 1 --> job)
 		 * row[3] --> Description
 		 */
-		callback (	user_data, row[0], row[1],
-				(row[2][0] == '0') ? 0 : 1,
-				row[3]);
+		callback (user_data,
+			(unsigned long long int) sql_row_item2ll (&(row[0])),
+			row[1].value_string,
+			(char)sql_row_item2ll (&(row[2])),
+			row[3].value_string);
 		sql_free_row (row);
 		ret = 2;
 	}
@@ -200,33 +203,34 @@ sql_job_main_check_parent (
  *    - The job/jobset description
  *
  * Return:
+ *     2 --> Missing calendars were found
  *     0 --> No error.
- *     1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
  *           with user_data_error_func as its first parameter and the error
  *           message as the second parameter)
- *     2 --> Missing calendars were found
- * other --> SQL error (if error_func() is not NULL, it is called with
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
  *           user_data_error_func as its first parameter and the error message
  *           as the second parameter)
  */
-unsigned int
+int
 sql_job_main_check_calendar (
-	int (*callback)(void *, const char *, const char *, char, const char *),
+	int (*callback)(void *, unsigned long long int,
+			const char *, char, const char *),
 	void *user_data,
-	void (*error_func)(void *, const char *, unsigned int),
+	void (*error_func)(void *, const char *, int),
 	void *user_data_error_func)
 {
 	char *err_msg = NULL;
 	lwc_LL *rows;
-	char **row;
-	unsigned int ret;
+	row_item_t *row;
+	int ret;
 
 #if HAVE_ASSERT_H
 	assert (callback != NULL);
 #endif
 
 	/* SQL request */
-	ret = sql_select (NULL, NULL, &err_msg, NULL, &rows, NULL,
+	ret = sql_select (NULL, NULL, &err_msg, NULL, &rows,
 			"SELECT id,name,type,description \
 			FROM job_main \
 			WHERE cal_id <> 0 AND cal_id NOT IN \
@@ -243,16 +247,18 @@ sql_job_main_check_calendar (
 	}
 
 	ret = 0;
-	while ((row = (char **) lwc_delStartLL (rows)) != NULL) {
+	while ((row = (row_item_t *) lwc_delStartLL (rows)) != NULL) {
 		/*
 		 * row[0] --> Job ID
 		 * row[1] --> Name
 		 * row[2] --> Type (0 -> jobset, 1 --> job)
 		 * row[3] --> Description
 		 */
-		callback (	user_data, row[0], row[1],
-				(row[2][0] == '0') ? 0 : 1,
-				row[3]);
+		callback (user_data,
+			(unsigned long long int) sql_row_item2ll (&(row[0])),
+			row[1].value_string,
+			(char)sql_row_item2ll (&(row[2])),
+			row[3].value_string);
 		sql_free_row (row);
 		ret = 2;
 	}
@@ -273,34 +279,34 @@ sql_job_main_check_calendar (
  *    - The erroneous start_time
  *
  * Return:
+ *     2 --> Erroneous start time were found
  *     0 --> No error.
- *     1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
  *           with user_data_error_func as its first parameter and the error
  *           message as the second parameter)
- *     2 --> Erroneous start time were found
- * other --> SQL error (if error_func() is not NULL, it is called with
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
  *           user_data_error_func as its first parameter and the error message
  *           as the second parameter)
  */
-unsigned int
+int
 sql_job_main_check_time (
-	int (*callback)(void *, const char *, const char *, char,
-			const char *, const char *),
+	int (*callback)(void *, unsigned long long int, const char *, char,
+			const char *, int),
 	void *user_data,
-	void (*error_func)(void *, const char *, unsigned int),
+	void (*error_func)(void *, const char *, int),
 	void *user_data_error_func)
 {
 	char *err_msg = NULL;
 	lwc_LL *rows;
-	char **row;
-	unsigned int ret;
+	row_item_t *row;
+	int ret;
 
 #if HAVE_ASSERT_H
 	assert (callback != NULL);
 #endif
 
 	/* SQL request */
-	ret = sql_select (NULL, NULL, &err_msg, NULL, &rows, NULL,
+	ret = sql_select (NULL, NULL, &err_msg, NULL, &rows,
 			"SELECT id,name,type,description,start_time \
 			FROM job_main \
 			WHERE start_time < -1 OR start_time > 2359",
@@ -316,7 +322,7 @@ sql_job_main_check_time (
 	}
 
 	ret = 0;
-	while ((row = (char **) lwc_delStartLL (rows)) != NULL) {
+	while ((row = (row_item_t *) lwc_delStartLL (rows)) != NULL) {
 		/*
 		 * row[0] --> Job ID
 		 * row[1] --> Name
@@ -324,9 +330,12 @@ sql_job_main_check_time (
 		 * row[3] --> Description
 		 * row[4] --> Start time
 		 */
-		callback (	user_data, row[0], row[1],
-				(row[2][0] == '0') ? 0 : 1,
-				row[3], row[4]);
+		callback (user_data,
+			(unsigned long long int) sql_row_item2ll (&(row[0])),
+			row[1].value_string,
+			(char)sql_row_item2ll (&(row[2])),
+			row[3].value_string,
+			(int)sql_row_item2ll (&(row[4])));
 		sql_free_row (row);
 		ret = 2;
 	}
@@ -346,34 +355,34 @@ sql_job_main_check_time (
  *    - The host description
  *
  * Return:
+ *     2 --> Missing SSL certificates were found
  *     0 --> No error.
- *     1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
  *           with user_data_error_func as its first parameter and the error
  *           message as the second parameter)
- *     2 --> Missing SSL certificates were found
- * other --> SQL error (if error_func() is not NULL, it is called with
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
  *           user_data_error_func as its first parameter and the error message
  *           as the second parameter)
  */
-unsigned int
+int
 sql_hosts_check_cert (
-	int (*callback)(void *, const char *, const char *,
+	int (*callback)(void *, unsigned long long int, const char *,
 			const char *, const char *),
 	void *user_data,
-	void (*error_func)(void *, const char *, unsigned int),
+	void (*error_func)(void *, const char *, int),
 	void *user_data_error_func)
 {
 	char *err_msg = NULL;
 	lwc_LL *rows;
-	char **row;
-	unsigned int ret;
+	row_item_t *row;
+	int ret;
 
 #if HAVE_ASSERT_H
 	assert (callback != NULL);
 #endif
 
 	/* SQL request */
-	ret = sql_select (NULL, NULL, &err_msg, NULL, &rows, NULL,
+	ret = sql_select (NULL, NULL, &err_msg, NULL, &rows,
 			"SELECT id,hostname,portnum,description \
 			FROM hosts \
 			WHERE sslenable = 1 \
@@ -390,14 +399,18 @@ sql_hosts_check_cert (
 	}
 
 	ret = 0;
-	while ((row = (char **) lwc_delStartLL (rows)) != NULL) {
+	while ((row = (row_item_t *) lwc_delStartLL (rows)) != NULL) {
 		/*
 		 * row[0] --> Host ID
 		 * row[1] --> Host name
 		 * row[2] --> TCP port number
 		 * row[3] --> Description
 		 */
-		callback (user_data, row[0], row[1], row[2], row[3]);
+		callback (user_data,
+			(unsigned long long int) sql_row_item2ll (&(row[0])),
+			row[1].value_string,
+			row[2].value_string,
+			row[3].value_string);
 		sql_free_row (row);
 		ret = 2;
 	}
@@ -417,27 +430,27 @@ sql_hosts_check_cert (
  *    - The calendar formula
  *
  * Return:
+ *     2 --> Erroneous formulas were found
  *     0 --> No error.
- *     1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
  *           with user_data_error_func as its first parameter and the error
  *           message as the second parameter)
- *     2 --> Erroneous formulas were found
- * other --> SQL error (if error_func() is not NULL, it is called with
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
  *           user_data_error_func as its first parameter and the error message
  *           as the second parameter)
  */
-unsigned int
+int
 sql_calendars_check_formula (
-	int (*callback)(void *, const char *, const char *,
+	int (*callback)(void *, unsigned long long int, const char *,
 			const char *, const char *),
 	void *user_data,
-	void (*error_func)(void *, const char *, unsigned int),
+	void (*error_func)(void *, const char *, int),
 	void *user_data_error_func)
 {
 	char *err_msg = NULL;
 	lwc_LL *rows;
-	char **row;
-	unsigned int ret;
+	row_item_t *row;
+	int ret;
 	cal_t cal;
 
 #if HAVE_ASSERT_H
@@ -445,7 +458,7 @@ sql_calendars_check_formula (
 #endif
 
 	/* SQL request */
-	ret = sql_select (NULL, NULL, &err_msg, NULL, &rows, NULL,
+	ret = sql_select (NULL, NULL, &err_msg, NULL, &rows,
 			"SELECT id,name,description,formula \
 			FROM calendars WHERE entry_type = 0",
 			SQL_END);
@@ -460,15 +473,21 @@ sql_calendars_check_formula (
 	}
 
 	ret = 0;
-	while ((row = (char **) lwc_delStartLL (rows)) != NULL) {
+	while ((row = (row_item_t *) lwc_delStartLL (rows)) != NULL) {
 		/*
 		 * row[0] --> Calendar ID
 		 * row[1] --> Name
 		 * row[2] --> Description
 		 * row[3] --> Formula
 		 */
-		if (str2cal (&cal, row[3], 2007, NULL) != CAL_NOERROR) {
-			callback (user_data, row[0], row[1], row[2], row[3]);
+		if (str2cal (	&cal, row[3].value_string,
+				2007, NULL) != CAL_NOERROR)
+		{
+			callback (user_data,
+			(unsigned long long int) sql_row_item2ll (&(row[0])),
+				row[1].value_string,
+				row[2].value_string,
+				row[3].value_string);
 			ret = 2;
 		}
 		sql_free_row (row);
@@ -489,33 +508,34 @@ sql_calendars_check_formula (
  *    - The calendar description
  *
  * Return:
+ *     2 --> Orphans were found
  *     0 --> No error.
- *     1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
  *           with user_data_error_func as its first parameter and the error
  *           message as the second parameter)
- *     2 --> Orphans were found
- * other --> SQL error (if error_func() is not NULL, it is called with
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
  *           user_data_error_func as its first parameter and the error message
  *           as the second parameter)
  */
-unsigned int
+int
 sql_calendars_check_parent (
-	int (*callback)(void *, const char *, const char *, char, const char *),
+	int (*callback)(void *, unsigned long long int, const char *, char,
+			const char *),
 	void *user_data,
-	void (*error_func)(void *, const char *, unsigned int),
+	void (*error_func)(void *, const char *, int),
 	void *user_data_error_func)
 {
 	char *err_msg = NULL;
 	lwc_LL *rows;
-	char **row;
-	unsigned int ret;
+	row_item_t *row;
+	int ret;
 
 #if HAVE_ASSERT_H
 	assert (callback != NULL);
 #endif
 
 	/* SQL request */
-	ret = sql_select (NULL, NULL, &err_msg, NULL, &rows, NULL,
+	ret = sql_select (NULL, NULL, &err_msg, NULL, &rows,
 			"SELECT id,name,entry_type,description \
 			FROM calendars \
 			WHERE parent <> 0 \
@@ -532,16 +552,18 @@ sql_calendars_check_parent (
 	}
 
 	ret = 0;
-	while ((row = (char **) lwc_delStartLL (rows)) != NULL) {
+	while ((row = (row_item_t *) lwc_delStartLL (rows)) != NULL) {
 		/*
 		 * row[0] --> Calendar ID
 		 * row[1] --> Name
 		 * row[2] --> Type (0 -> calendar, 1 --> folder)
 		 * row[3] --> Description
 		 */
-		callback (	user_data, row[0], row[1],
-				(row[2][0] == '0') ? 0 : 1,
-				row[3]);
+		callback (user_data,
+			(unsigned long long int) sql_row_item2ll (&(row[0])),
+				row[1].value_string,
+				(char)sql_row_item2ll (&(row[2])),
+				row[3].value_string);
 		sql_free_row (row);
 		ret = 2;
 	}
@@ -560,26 +582,27 @@ sql_calendars_check_parent (
  *    - The environment variable value
  *
  * Return:
+ *     2 --> Errors were found.
  *     0 --> No error.
- *     1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
  *           with user_data_error_func as its first parameter and the error
  *           message as the second parameter)
- *     2 --> Errors were found.
- * other --> SQL error (if error_func() is not NULL, it is called with
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
  *           user_data_error_func as its first parameter and the error message
  *           as the second parameter)
  */
-unsigned int
+int
 sql_environment_var_check_value (
-	int (*callback)(void *, const char *, const char *, const char *),
+	int (*callback)(void *, unsigned long long int, const char *,
+			const char *),
 	void *user_data,
-	void (*error_func)(void *, const char *, unsigned int),
+	void (*error_func)(void *, const char *, int),
 	void *user_data_error_func)
 {
 	char *err_msg = NULL;
 	lwc_LL *rows;
-	char **row;
-	unsigned int ret;
+	row_item_t *row;
+	int ret;
 	char *out;
 	size_t out_len;
 
@@ -588,7 +611,7 @@ sql_environment_var_check_value (
 #endif
 
 	/* SQL request */
-	ret = sql_select (NULL, NULL, &err_msg, NULL, &rows, NULL,
+	ret = sql_select (NULL, NULL, &err_msg, NULL, &rows,
 			"SELECT env_id,env_key,env_value \
 			FROM environment_var",
 			SQL_END);
@@ -605,17 +628,20 @@ sql_environment_var_check_value (
 	out = NULL;
 	out_len = 0;
 	ret = 0;
-	while ((row = (char **) lwc_delStartLL (rows)) != NULL) {
+	while ((row = (row_item_t *) lwc_delStartLL (rows)) != NULL) {
 		/*
 		 * row[0] --> Environment variable ID
 		 * row[1] --> Name
 		 * row[2] --> Value
 		 */
-		if (parse_string (	row[2],
-					(size_t)schedwi_strlen (row[2]),
+		if (parse_string (	row[2].value_string,
+					row[2].len,
 					NULL, &out, &out_len) != 0)
 		{
-			callback (user_data, row[0], row[1], row[2]);
+			callback (user_data,
+			(unsigned long long int) sql_row_item2ll (&(row[0])),
+				row[1].value_string,
+				row[2].value_string);
 			ret = 2;
 		}
 		sql_free_row (row);
@@ -633,32 +659,32 @@ sql_environment_var_check_value (
  *
  * Return:
  *     0 --> No error
- *     1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
  *           with user_data_error_func as its first parameter and the error
  *           message as the second parameter)
- * other --> SQL error (if error_func() is not NULL, it is called with
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
  *           user_data_error_func as its first parameter and the error message
  *           as the second parameter)
  */
-unsigned int
+int
 sql_check_delete1 (	const char *table_name,
 			const char *field_name,
-			const char *id,
-			void (*error_func)(void *, const char *, unsigned int),
+			unsigned long long int id,
+			void (*error_func)(void *, const char *, int),
 			void *user_data_error_func)
 {
 	char *err_msg = NULL;
-	unsigned int ret;
+	int ret;
 
 #if HAVE_ASSERT_H
-	assert (id != NULL && table_name != NULL && field_name != NULL);
+	assert (table_name != NULL && field_name != NULL);
 #endif
 
-	ret = sql_non_select (	NULL, NULL, &err_msg, NULL, NULL,
-				"DELETE FROM %s WHERE %s=\"%s\"",
+	ret = sql_non_select (	NULL, NULL, &err_msg, NULL, NULL, NULL,
+				"DELETE FROM %s WHERE %s=%ld",
 				SQL_STRING_NON_ESCAPE, table_name,
 				SQL_STRING_NON_ESCAPE, field_name,
-				SQL_STRING, id,
+				SQL_INT, (long int)id,
 				SQL_END);
 	if (ret != 0) {
 		if (error_func != NULL) {
@@ -678,36 +704,36 @@ sql_check_delete1 (	const char *table_name,
  *
  * Return:
  *     0 --> No error
- *     1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
  *           with user_data_error_func as its first parameter and the error
  *           message as the second parameter)
- * other --> SQL error (if error_func() is not NULL, it is called with
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
  *           user_data_error_func as its first parameter and the error message
  *           as the second parameter)
  */
-unsigned int
+int
 sql_check_delete2 (	const char *table_name,
 			const char *field_name1,
 			const char *field_name2,
-			const char *id1,
+			unsigned long long int id1,
 			const char *id2,
-			void (*error_func)(void *, const char *, unsigned int),
+			void (*error_func)(void *, const char *, int),
 			void *user_data_error_func)
 {
 	char *err_msg = NULL;
-	unsigned int ret;
+	int ret;
 
 #if HAVE_ASSERT_H
 	assert (	   table_name != NULL
 			&& field_name1 != NULL && field_name2 != NULL
-			&& id1 != NULL && id2 != NULL);
+			&& id2 != NULL);
 #endif
 
-	ret = sql_non_select (	NULL, NULL, &err_msg, NULL, NULL,
-				"DELETE FROM %s WHERE %s=\"%s\" AND %s=\"%s\"",
+	ret = sql_non_select (	NULL, NULL, &err_msg, NULL, NULL, NULL,
+				"DELETE FROM %s WHERE %s=%ld AND %s=%s",
 				SQL_STRING_NON_ESCAPE, table_name,
 				SQL_STRING_NON_ESCAPE, field_name1,
-				SQL_STRING, id1,
+				SQL_INT, (long int)id1,
 				SQL_STRING_NON_ESCAPE, field_name2,
 				SQL_STRING, id2,
 				SQL_END);
@@ -729,28 +755,24 @@ sql_check_delete2 (	const char *table_name,
  *
  * Return:
  *     0 --> No error
- *     1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
  *           with user_data_error_func as its first parameter and the error
  *           message as the second parameter)
- * other --> SQL error (if error_func() is not NULL, it is called with
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
  *           user_data_error_func as its first parameter and the error message
  *           as the second parameter)
  */
-unsigned int
-sql_check_disable_ssl (	const char *id,
-			void (*error_func)(void *, const char *, unsigned int),
+int
+sql_check_disable_ssl (	unsigned long long int id,
+			void (*error_func)(void *, const char *, int),
 			void *user_data_error_func)
 {
 	char *err_msg = NULL;
-	unsigned int ret;
+	int ret;
 
-#if HAVE_ASSERT_H
-	assert (id != NULL);
-#endif
-
-	ret = sql_non_select (	NULL, NULL, &err_msg, NULL, NULL,
-				"UPDATE hosts SET sslenable=0 WHERE id=\"%s\"",
-				SQL_STRING, id,
+	ret = sql_non_select (	NULL, NULL, &err_msg, NULL, NULL, NULL,
+				"UPDATE hosts SET sslenable=0 WHERE id=%ld",
+				SQL_INT, (long int)id,
 				SQL_END);
 	if (ret != 0) {
 		if (error_func != NULL) {
@@ -771,28 +793,24 @@ sql_check_disable_ssl (	const char *id,
  *
  * Return:
  *     0 --> No error
- *     1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
  *           with user_data_error_func as its first parameter and the error
  *           message as the second parameter)
- * other --> SQL error (if error_func() is not NULL, it is called with
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
  *           user_data_error_func as its first parameter and the error message
  *           as the second parameter)
  */
-unsigned int
-sql_check_fix_cal_id (	const char *id,
-			void (*error_func)(void *, const char *, unsigned int),
+int
+sql_check_fix_cal_id (	unsigned long long int id,
+			void (*error_func)(void *, const char *, int),
 			void *user_data_error_func)
 {
 	char *err_msg = NULL;
-	unsigned int ret;
+	int ret;
 
-#if HAVE_ASSERT_H
-	assert (id != NULL);
-#endif
-
-	ret = sql_non_select (	NULL, NULL, &err_msg, NULL, NULL,
-				"UPDATE job_main SET cal_id=0 WHERE id=\"%s\"",
-				SQL_STRING, id,
+	ret = sql_non_select (	NULL, NULL, &err_msg, NULL, NULL, NULL,
+				"UPDATE job_main SET cal_id=0 WHERE id=%ld",
+				SQL_INT, (long int)id,
 				SQL_END);
 	if (ret != 0) {
 		if (error_func != NULL) {
@@ -813,29 +831,25 @@ sql_check_fix_cal_id (	const char *id,
  *
  * Return:
  *     0 --> No error
- *     1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
  *           with user_data_error_func as its first parameter and the error
  *           message as the second parameter)
- * other --> SQL error (if error_func() is not NULL, it is called with
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
  *           user_data_error_func as its first parameter and the error message
  *           as the second parameter)
  */
-unsigned int
-sql_check_fix_start_time (const char *id,
-			void (*error_func)(void *, const char *, unsigned int),
+int
+sql_check_fix_start_time (unsigned long long int id,
+			void (*error_func)(void *, const char *, int),
 			void *user_data_error_func)
 {
 	char *err_msg = NULL;
-	unsigned int ret;
+	int ret;
 
-#if HAVE_ASSERT_H
-	assert (id != NULL);
-#endif
-
-	ret = sql_non_select (	NULL, NULL, &err_msg, NULL, NULL,
+	ret = sql_non_select (	NULL, NULL, &err_msg, NULL, NULL, NULL,
 				"UPDATE job_main SET start_time=-1 \
-				WHERE id=\"%s\"",
-				SQL_STRING, id,
+				WHERE id=%ld",
+				SQL_INT, (long int)id,
 				SQL_END);
 	if (ret != 0) {
 		if (error_func != NULL) {
@@ -856,31 +870,31 @@ sql_check_fix_start_time (const char *id,
  *
  * Return:
  *     0 --> No error
- *     1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
  *           with user_data_error_func as its first parameter and the error
  *           message as the second parameter)
- * other --> SQL error (if error_func() is not NULL, it is called with
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
  *           user_data_error_func as its first parameter and the error message
  *           as the second parameter)
  */
-unsigned int
+int
 sql_check_create_lost_found_jobset (const char *name, const char *description,
 			int x, int y,
-			unsigned long int *id,
-			void (*error_func)(void *, const char *, unsigned int),
+			unsigned long long int *id,
+			void (*error_func)(void *, const char *, int),
 			void *user_data_error_func)
 {
 	char *err_msg = NULL;
-	unsigned int ret;
+	int ret;
 
 #if HAVE_ASSERT_H
 	assert (name != NULL && description != NULL);
 #endif
 
-	ret = sql_non_select (	NULL, NULL, &err_msg, id, NULL,
+	ret = sql_non_select (	NULL, NULL, &err_msg, id, NULL, "job_main",
 		"INSERT INTO job_main \
 		 (parent,name,type,enabled,description,x,y,cal_id,start_time) \
-		VALUES (1,\"%s\",0,0,\"%s\",%d,%d,0,-1)", 
+		VALUES (1,%s,0,0,%s,%d,%d,0,-1)",
 				SQL_STRING, name,
 				SQL_STRING, description,
 				SQL_INT, (long int)x,
@@ -904,31 +918,27 @@ sql_check_create_lost_found_jobset (const char *name, const char *description,
  *
  * Return:
  *     0 --> No error
- *     1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
  *           with user_data_error_func as its first parameter and the error
  *           message as the second parameter)
- * other --> SQL error (if error_func() is not NULL, it is called with
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
  *           user_data_error_func as its first parameter and the error message
  *           as the second parameter)
  */
-unsigned int
-sql_check_attach_orphan_job (const char *id,
-			unsigned long int parent_id,
-			void (*error_func)(void *, const char *, unsigned int),
+int
+sql_check_attach_orphan_job (unsigned long long int id,
+			unsigned long long int parent_id,
+			void (*error_func)(void *, const char *, int),
 			void *user_data_error_func)
 {
 	char *err_msg = NULL;
-	unsigned int ret;
+	int ret;
 
-#if HAVE_ASSERT_H
-	assert (id != NULL);
-#endif
-
-	ret = sql_non_select (	NULL, NULL, &err_msg, NULL, NULL,
-				"UPDATE job_main SET parent=%d \
-				WHERE id=\"%s\"",
-				SQL_INT, parent_id,
-				SQL_STRING, id,
+	ret = sql_non_select (	NULL, NULL, &err_msg, NULL, NULL, NULL,
+				"UPDATE job_main SET parent=%ld \
+				WHERE id=%ld",
+				SQL_INT, (long int)parent_id,
+				SQL_INT, (long int)id,
 				SQL_END);
 	if (ret != 0) {
 		if (error_func != NULL) {
@@ -949,30 +959,30 @@ sql_check_attach_orphan_job (const char *id,
  *
  * Return:
  *     0 --> No error
- *     1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
  *           with user_data_error_func as its first parameter and the error
  *           message as the second parameter)
- * other --> SQL error (if error_func() is not NULL, it is called with
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
  *           user_data_error_func as its first parameter and the error message
  *           as the second parameter)
  */
-unsigned int
+int
 sql_check_create_lost_found_cal_folder (const char *name,
 			const char *description,
-			unsigned long int *id,
-			void (*error_func)(void *, const char *, unsigned int),
+			unsigned long long int *id,
+			void (*error_func)(void *, const char *, int),
 			void *user_data_error_func)
 {
 	char *err_msg = NULL;
-	unsigned int ret;
+	int ret;
 
 #if HAVE_ASSERT_H
 	assert (name != NULL && description != NULL);
 #endif
 
-	ret = sql_non_select (	NULL, NULL, &err_msg, id, NULL,
+	ret = sql_non_select (	NULL, NULL, &err_msg, id, NULL, "calendars",
 		"INSERT INTO calendars (parent,name,entry_type,description) \
-		VALUES (1,\"%s\",1,\"%s\")", 
+		VALUES (1,%s,1,%s)",
 				SQL_STRING, name,
 				SQL_STRING, description,
 				SQL_END);
@@ -994,31 +1004,28 @@ sql_check_create_lost_found_cal_folder (const char *name,
  *
  * Return:
  *     0 --> No error
- *     1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
  *           with user_data_error_func as its first parameter and the error
  *           message as the second parameter)
- * other --> SQL error (if error_func() is not NULL, it is called with
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
  *           user_data_error_func as its first parameter and the error message
  *           as the second parameter)
  */
-unsigned int
-sql_check_attach_orphan_cal (const char *id,
-			unsigned long int parent_id,
-			void (*error_func)(void *, const char *, unsigned int),
+int
+sql_check_attach_orphan_cal (unsigned long long int id,
+			unsigned long long int parent_id,
+			void (*error_func)(void *, const char *, int),
 			void *user_data_error_func)
 {
 	char *err_msg = NULL;
-	unsigned int ret;
+	int ret;
 
-#if HAVE_ASSERT_H
-	assert (id != NULL);
-#endif
 
-	ret = sql_non_select (	NULL, NULL, &err_msg, NULL, NULL,
-				"UPDATE calendars SET parent=%d \
-				WHERE id=\"%s\"",
-				SQL_INT, parent_id,
-				SQL_STRING, id,
+	ret = sql_non_select (	NULL, NULL, &err_msg, NULL, NULL, NULL,
+				"UPDATE calendars SET parent=%ld \
+				WHERE id=%ld",
+				SQL_INT, (long int)parent_id,
+				SQL_INT, (long int)id,
 				SQL_END);
 	if (ret != 0) {
 		if (error_func != NULL) {
@@ -1038,30 +1045,31 @@ sql_check_attach_orphan_cal (const char *id,
  *
  * Return:
  *     0 --> No error
- *     1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
  *           with user_data_error_func as its first parameter and the error
  *           message as the second parameter)
- * other --> SQL error (if error_func() is not NULL, it is called with
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
  *           user_data_error_func as its first parameter and the error message
  *           as the second parameter)
  */
-unsigned int
-sql_check_comment_out_formula (const char *id, const char *new_formula,
-			void (*error_func)(void *, const char *, unsigned int),
+int
+sql_check_comment_out_formula (unsigned long long int id,
+			const char *new_formula,
+			void (*error_func)(void *, const char *, int),
 			void *user_data_error_func)
 {
 	char *err_msg = NULL;
-	unsigned int ret;
+	int ret;
 
 #if HAVE_ASSERT_H
-	assert (id != NULL && new_formula != NULL);
+	assert (new_formula != NULL);
 #endif
 
-	ret = sql_non_select (	NULL, NULL, &err_msg, NULL, NULL,
-				"UPDATE calendars SET formula=\"%s\" \
-				WHERE id=\"%s\"",
+	ret = sql_non_select (	NULL, NULL, &err_msg, NULL, NULL, NULL,
+				"UPDATE calendars SET formula=%s \
+				WHERE id=%ld",
 				SQL_STRING, new_formula,
-				SQL_STRING, id,
+				SQL_INT, (long int)id,
 				SQL_END);
 	if (ret != 0) {
 		if (error_func != NULL) {
