@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
+   Copyright (C) 2007-2010 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -47,7 +47,7 @@
 
 
 #define SQL_GET_ENVIRONMENT_REQ "SELECT job_environment_s.job_id,job_environment_s.position,environment_var_s.position,environment_var_s.env_key,environment_var_s.env_value FROM environment_var_s,job_environment_s WHERE job_environment_s.workload_date=%d AND environment_var_s.workload_date=%d AND job_environment_s.job_id IN (%s) AND job_environment_s.env_id=environment_var_s.env_id ORDER BY job_environment_s.position,environment_var_s.position"
-#define SQL_GET_ENVIRONMENT_HOST_REQ "SELECT host_environment_s.position,environment_var_s.position,environment_var_s.env_key,environment_var_s.env_value FROM environment_var_s,host_environment_s WHERE host_environment_s.workload_date=%d AND environment_var_s.workload_date=%d AND host_environment_s.host_id = \"%s\" AND host_environment_s.env_id=environment_var_s.env_id ORDER BY host_environment_s.position,environment_var_s.position"
+#define SQL_GET_ENVIRONMENT_HOST_REQ "SELECT host_environment_s.position,environment_var_s.position,environment_var_s.env_key,environment_var_s.env_value FROM environment_var_s,host_environment_s WHERE host_environment_s.workload_date=%d AND environment_var_s.workload_date=%d AND host_environment_s.host_id=%ld AND host_environment_s.env_id=environment_var_s.env_id ORDER BY host_environment_s.position,environment_var_s.position"
 
 
 /*
@@ -61,28 +61,29 @@
  * sql_get_environment() exits with a return code of 2.
  *
  * Return:
+ *     2 --> The callback function returned a value other than 0
  *     0 --> No error.
- *     1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
  *           with user_data_error_func as its first parameter and the error
  *           message as the second parameter)
- *     2 --> The callback function returned a value other than 0
- * other --> SQL error (if error_func() is not NULL, it is called with
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
  *           user_data_error_func as its first parameter and the error message
  *           as the second parameter)
  */
-unsigned int
+int
 sql_get_environment (int workload_date, lwc_LL *lst, unsigned int next_pos,
 	int (*callback)(void *, unsigned int, const char *, const char *),
 	void *user_data,
-	void (*error_func)(void *, const char *, unsigned int),
+	void (*error_func)(void *, const char *, int),
 	void *user_data_error_func)
 {
 	char *err_msg = NULL;
-	char *hierarchy_str, *id_str;
-	unsigned int ret, i, j, hierarchy_len;
+	char *hierarchy_str;
+	unsigned int i, j, hierarchy_len;
+	int ret;
 	lwc_LL *rows, **lists, *result_list;
-	char **row;
-	int callback_ret;
+	row_item_t *row;
+	unsigned long long int jobid, *id;
 
 #if HAVE_ASSERT_H
 	assert (lwc_getNumNode (lst) > 0 && callback != NULL);
@@ -97,7 +98,7 @@ sql_get_environment (int workload_date, lwc_LL *lst, unsigned int next_pos,
 			error_func (	user_data_error_func,
 					_("Memory allocation error"), 0);
 		}
-		return 1;
+		return -1;
 	}
 
 	for (i = 0; i < hierarchy_len; i++) {
@@ -112,7 +113,7 @@ sql_get_environment (int workload_date, lwc_LL *lst, unsigned int next_pos,
 						_("Memory allocation error"),
 						0);
 			}
-			return 1;
+			return -1;
 		}
 	}
 
@@ -126,11 +127,11 @@ sql_get_environment (int workload_date, lwc_LL *lst, unsigned int next_pos,
 			error_func (	user_data_error_func,
 					_("Memory allocation error"), 0);
 		}
-		return 1;
+		return -1;
 	}
 
 	/* SQL request */
-	ret = sql_select (	NULL, NULL, &err_msg, NULL, &rows, NULL,
+	ret = sql_select (	NULL, NULL, &err_msg, NULL, &rows,
 				SQL_GET_ENVIRONMENT_REQ,
 				SQL_INT, (long int)workload_date,
 				SQL_INT, (long int)workload_date,
@@ -151,7 +152,7 @@ sql_get_environment (int workload_date, lwc_LL *lst, unsigned int next_pos,
 		return ret;
 	}
 
-	while ((row = (char **) lwc_delStartLL (rows)) != NULL) {
+	while ((row = (row_item_t *) lwc_delStartLL (rows)) != NULL) {
 		/*
 		 * row[0] --> Job ID
 		 * row[1] --> Position 1
@@ -159,15 +160,16 @@ sql_get_environment (int workload_date, lwc_LL *lst, unsigned int next_pos,
 		 * row[3] --> Environment variable name
 		 * row[4] --> Environment variable value
 		 */
+		jobid = (unsigned long long int)sql_row_item2ll (&(row[0]));
 		lwc_rewindLL (lst);
 		for (	i = 0;
-			   (id_str = (char *)lwc_nextLL (lst)) != NULL
-			&& strcmp (id_str, row[0]) != 0;
+			   (id = (unsigned long long int *) lwc_nextLL (lst)) != NULL
+			&& *id != jobid;
 			i++);
-		if (id_str != NULL) {
+		if (id != NULL) {
 			if (lwc_addEndLL (lists[i], row) != 0) {
 				for (i = 0; i < hierarchy_len; i++) {
-					lwc_delLL (lists[i], 
+					lwc_delLL (lists[i],
 					(void (*)(const void *))sql_free_row);
 				}
 				free (lists);
@@ -179,7 +181,7 @@ sql_get_environment (int workload_date, lwc_LL *lst, unsigned int next_pos,
 						_("Memory allocation error"),
 						0);
 				}
-				return 1;
+				return -1;
 			}
 		}
 		else {
@@ -192,7 +194,7 @@ sql_get_environment (int workload_date, lwc_LL *lst, unsigned int next_pos,
 	result_list = lwc_newLL ();
 	if (result_list == NULL) {
 		for (i = 0; i < hierarchy_len; i++) {
-			lwc_delLL (lists[i], 
+			lwc_delLL (lists[i],
 					(void (*)(const void *))sql_free_row);
 		}
 		free (lists);
@@ -200,13 +202,14 @@ sql_get_environment (int workload_date, lwc_LL *lst, unsigned int next_pos,
 			error_func (	user_data_error_func,
 					_("Memory allocation error"), 0);
 		}
-		return 1;
+		return -1;
 	}
 
 	i = hierarchy_len;
 	do {
 		i--;
-		while ((row = (char **) lwc_delStartLL (lists[i])) != NULL) {
+		while ((row = (row_item_t *)lwc_delStartLL (lists[i])) != NULL)
+		{
 			/*
 			 * row[0] --> Job ID
 			 * row[1] --> Position 1
@@ -216,7 +219,7 @@ sql_get_environment (int workload_date, lwc_LL *lst, unsigned int next_pos,
 			 */
 
 			/* No environment for the current job if pos1 == -1 */
-			if (row[1][0] == '-') {
+			if (sql_row_item2ll (&(row[1])) < 0) {
 				lwc_emptyLL (result_list,
 					(void (*)(const void *))sql_free_row);
 				lwc_emptyLL (lists[i],
@@ -227,7 +230,7 @@ sql_get_environment (int workload_date, lwc_LL *lst, unsigned int next_pos,
 
 			if (lwc_addEndLL (result_list, row) != 0) {
 				for (i = 0; i < hierarchy_len; i++) {
-					lwc_delLL (lists[i], 
+					lwc_delLL (lists[i],
 					(void (*)(const void *))sql_free_row);
 				}
 				free (lists);
@@ -239,7 +242,7 @@ sql_get_environment (int workload_date, lwc_LL *lst, unsigned int next_pos,
 						_("Memory allocation error"),
 						0);
 				}
-				return 1;
+				return -1;
 			}
 		}
 	} while (i > 0);
@@ -249,7 +252,7 @@ sql_get_environment (int workload_date, lwc_LL *lst, unsigned int next_pos,
 	}
 	free (lists);
 
-	while ((row = (char **) lwc_delStartLL (result_list)) != NULL) {
+	while ((row = (row_item_t *) lwc_delStartLL (result_list)) != NULL) {
 		/*
 		 * row[0] --> Job ID
 		 * row[1] --> Position 1
@@ -257,10 +260,10 @@ sql_get_environment (int workload_date, lwc_LL *lst, unsigned int next_pos,
 		 * row[3] --> Environment variable name
 		 * row[4] --> Environment variable value
 		 */
-		callback_ret = callback (	user_data, next_pos++,
-						row[3], row[4]);     	
+		ret = callback (user_data, next_pos++,
+				row[3].value_string, row[4].value_string);
 		sql_free_row (row);
-		if (callback_ret != 0) {
+		if (ret != 0) {
 			lwc_delLL (	result_list,
 					(void (*)(const void *))sql_free_row);
 			return 2;
@@ -285,39 +288,39 @@ sql_get_environment (int workload_date, lwc_LL *lst, unsigned int next_pos,
  * function.
  *
  * Return:
+ *     2 --> The callback function returned a value other than 0
  *     0 --> No error.
- *     1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
  *           with user_data_error_func as its first parameter and the error
  *           message as the second parameter)
- *     2 --> The callback function returned a value other than 0
- * other --> SQL error (if error_func() is not NULL, it is called with
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
  *           user_data_error_func as its first parameter and the error message
  *           as the second parameter)
  */
-unsigned int
-sql_get_environment_host (int workload_date, const char *host_id, 
+int
+sql_get_environment_host (int workload_date, unsigned long long int host_id,
 	int (*callback)(void *, unsigned int, const char *, const char *),
 	void *user_data,
 	unsigned int *next_pos,
-	void (*error_func)(void *, const char *, unsigned int),
+	void (*error_func)(void *, const char *, int),
 	void *user_data_error_func)
 {
 	char *err_msg = NULL;
 	lwc_LL *rows;
-	char **row;
-	int callback_ret;
-	unsigned int i, ret;
+	row_item_t *row;
+	unsigned int i;
+	int ret;
 
 #if HAVE_ASSERT_H
-	assert (host_id != NULL && callback != NULL);
+	assert (callback != NULL);
 #endif
 
 	/* SQL request */
-	ret = sql_select (	NULL, NULL, &err_msg, NULL, &rows, NULL,
+	ret = sql_select (	NULL, NULL, &err_msg, NULL, &rows,
 				SQL_GET_ENVIRONMENT_HOST_REQ,
 				SQL_INT, (long int)workload_date,
 				SQL_INT, (long int)workload_date,
-				SQL_STRING, host_id,
+				SQL_INT, (long int)host_id,
 				SQL_END);
 	if (ret != 0) {
 		if (error_func != NULL) {
@@ -330,16 +333,17 @@ sql_get_environment_host (int workload_date, const char *host_id,
 	}
 
 	i = 1;
-	while ((row = (char **) lwc_delStartLL (rows)) != NULL) {
+	while ((row = (row_item_t *) lwc_delStartLL (rows)) != NULL) {
 		/*
 		 * row[0] --> Position 1
 		 * row[1] --> Position 2
 		 * row[2] --> Environment variable name
 		 * row[3] --> Environment variable value
 		 */
-		callback_ret = callback (user_data, i++, row[2], row[3]);     	
+		ret = callback (user_data, i++,
+				row[2].value_string, row[3].value_string);
 		sql_free_row (row);
-		if (callback_ret != 0) {
+		if (ret != 0) {
 			lwc_delLL (rows, (void (*)(const void *))sql_free_row);
 			return 2;
 		}
