@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
+   Copyright (C) 2007-2010 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -30,15 +30,15 @@
 #include <assert.h>
 #endif
 
+#include <utils.h>
 #include <lib_functions.h>
-#include <sql_common.h>
 #include <sql_hierarchy.h>
 
-#define SQL_GET_JOB_NAME "SELECT name,parent FROM job_main WHERE id=\"%s\"" 
-#define SQL_GET_PARENT "SELECT parent FROM job_main_s WHERE workload_date=%d AND id=\"%s\""
-#define SQL_GET_JOB_PARAMS "SELECT %s FROM %s WHERE workload_date=%d AND job_id=\"%s\""
-#define SQL_GET_JOB_PARAMS_TIME "SELECT %s FROM %s WHERE workload_date=%d AND id=\"%s\""
-#define SQL_GET_JOB_PARAMS_SORTED "SELECT %s FROM %s WHERE workload_date=%d AND job_id=\"%s\" ORDER BY %s"
+#define SQL_GET_JOB_NAME "SELECT name,parent FROM job_main WHERE id=%ld"
+#define SQL_GET_PARENT "SELECT parent FROM job_main_s WHERE workload_date=%d AND id=%ld"
+#define SQL_GET_JOB_PARAMS "SELECT %s FROM %s WHERE workload_date=%d AND job_id=%ld"
+#define SQL_GET_JOB_PARAMS_TIME "SELECT %s FROM %s WHERE workload_date=%d AND id=%ld"
+#define SQL_GET_JOB_PARAMS_SORTED "SELECT %s FROM %s WHERE workload_date=%d AND job_id=%ld ORDER BY %s"
 #define SQL_GET_JOB_ID "SELECT id,parent,name,type FROM job_main WHERE name IN (%s)"
 
 
@@ -118,46 +118,40 @@ prepend_path (char **str, unsigned int *str_len, const char *s)
  *     0 --> No error.  path is set and must be freed by the caller (free()).
  *           The working buffer buf may have been set by this function and
  *           must also be freed by the caller.
- *     1 --> Memory allocation error (if err_msg is not NULL it contains an
+ *    -1 --> Memory allocation error (if err_msg is not NULL it contains an
  *           error message which must be freed by the caller.  *err_msg may be
  *           NULL if there is not enough memory to store the error message).
  *           path and buf must be freed by the caller.
- *     2 --> Database connection error (if err_msg is not NULL it contains an
- *           error message which must be freed by the caller.  *err_msg may be
- *           NULL if there is not enough memory to store the error message).
- *           path and buf must be freed by the caller.
- * other --> Database error (if err_msg is not NULL it contains an error
+ *    -2 --> Database error (if err_msg is not NULL it contains an error
  *           message which must be freed by the caller.  *err_msg may be NULL
  *           if there is not enough memory to store the error message).
- *           The returned value is the one returned by mysql_errno().
- *           path and buf must be freed by the caller.
  */
-static unsigned int
-get_job_path_recursive (const char *job_id,
+static int
+get_job_path_recursive (unsigned long long int job_id,
 			char **buf, unsigned int *buf_len,
 			char **path, unsigned int *path_len,
 			char **err_msg)
 {
-	unsigned int ret;
+	int ret;
 	lwc_LL *rows;
-	char **row;
+	row_item_t *row;
 	char *s;
 
 #if HAVE_ASSERT_H
-	assert (job_id != NULL && path != NULL && path_len != NULL);
+	assert (path != NULL && path_len != NULL);
 #endif
 
 	/* Get the job name and its parent */
-	ret = sql_select (	buf, buf_len, err_msg, NULL, &rows, NULL,
+	ret = sql_select (	buf, buf_len, err_msg, NULL, &rows,
 				SQL_GET_JOB_NAME,
-				SQL_STRING, job_id,
+				SQL_INT, (long int)job_id,
 				SQL_END);
 	if (ret != 0) {
 		return ret;
 	}
 
 	/* Retrieve the result (only one row) */
-	row = (char **) lwc_delStartLL (rows);
+	row = (row_item_t *) lwc_delStartLL (rows);
 	if (row == NULL) {
 		/*
 		 * Nothing found - The root of the hieararchy
@@ -169,7 +163,7 @@ get_job_path_recursive (const char *job_id,
 	lwc_delLL (rows, (void (*)(const void *)) sql_free_row);
 
 	/* Prepend the job name to the path */
-	if (prepend_path (path, path_len, row[0]) != 0) {
+	if (prepend_path (path, path_len, row[0].value_string) != 0) {
 		sql_free_row (row);
 		if (err_msg != NULL) {
 			s = _("Memory allocation error");
@@ -178,12 +172,13 @@ get_job_path_recursive (const char *job_id,
 				strcpy (*err_msg, s);
 			}
 		}
-		return 1;
+		return -1;
 	}
 
 	/* Recursively retrieve the parent jobset */
-	ret = get_job_path_recursive (	row[1], buf, buf_len, path, path_len,
-					err_msg);
+	ret = get_job_path_recursive (
+			(unsigned long long int) sql_row_item2ll (&(row[1])),
+			buf, buf_len, path, path_len, err_msg);
 	sql_free_row (row);
 
 	return ret;
@@ -196,33 +191,29 @@ get_job_path_recursive (const char *job_id,
  * Return:
  *     0 --> No error.  path is set and must be freed by the caller (free()).
  *           If path is NULL then the provided job_id is not in the database
- *     1 --> Memory allocation error (if err_msg is not NULL it contains an
+ *    -1 --> Memory allocation error (if err_msg is not NULL it contains an
  *           error message which must be freed by the caller.  *err_msg may be
  *           NULL if there is not enough memory to store the error message).
- *     2 --> Database connection error (if err_msg is not NULL it contains an
- *           error message which must be freed by the caller.  *err_msg may be
- *           NULL if there is not enough memory to store the error message).
- * other --> Database error (if err_msg is not NULL it contains an error
+ *    -2 --> Database error (if err_msg is not NULL it contains an error
  *           message which must be freed by the caller.  *err_msg may be NULL
  *           if there is not enough memory to store the error message).
- *           The returned value is the one returned by mysql_errno().
  */
-unsigned int
-get_job_path (const char *job_id, char **path, char **err_msg)
+int
+get_job_path (unsigned long long int job_id, char **path, char **err_msg)
 {
-	MYSQL *sql;
-	unsigned int ret;
+	dbi_conn sql;
+	int ret;
 	char *buf, *out;
 	unsigned int buf_len, out_len;
 
 #if HAVE_ASSERT_H
-	assert (job_id != NULL && path != NULL);
+	assert (path != NULL);
 #endif
 
 	/* Connect to the database */
-	sql = begin_mysql (err_msg);
+	sql = begin_sql (err_msg);
 	if (sql == NULL) {
-		return 2;
+		return -2;
 	}
 
 	/* Build the path */
@@ -241,7 +232,7 @@ get_job_path (const char *job_id, char **path, char **err_msg)
 	}
 
 	/* Disconnect from the database */
-	end_mysql ();
+	end_sql ();
 
 	if (buf != NULL) {
 		free (buf);
@@ -252,9 +243,9 @@ get_job_path (const char *job_id, char **path, char **err_msg)
 
 
 /*
- * Get the hierarchy list of a job.  This list must be passed as a parameter
- * to the get_job_parameters() and get_job_parameter() functions and must be
- * freed by the caller.
+ * Get the hierarchy list of a job.  This list can then be passed as a
+ * parameter to the get_job_parameters() and get_job_parameter() functions and
+ * must be freed by the caller.
  *
  * Return:
  *     The list to be freed by the caller using
@@ -264,18 +255,17 @@ get_job_path (const char *job_id, char **path, char **err_msg)
  *     NULL if there is not enough memory to store the error message)
  */
 lwc_LL *
-get_hierarchy_list (int workload_date, const char *job_id, char **err_msg)
+get_hierarchy_list (	int workload_date, unsigned long long int job_id,
+			char **err_msg)
 {
-	MYSQL *sql;
-	unsigned int ret;
+	dbi_conn sql;
+	int ret;
 	lwc_LL *lst, *rows;
-	char **row;
-	char *id, *buf, *s;
+	row_item_t *row;
+	char *buf, *s;
 	unsigned int buf_len;
+	unsigned long long int *id;
 
-#if HAVE_ASSERT_H
-	assert (job_id != NULL);
-#endif
 
 	/* Create the result list */
 	lst = lwc_newLL ();
@@ -291,7 +281,8 @@ get_hierarchy_list (int workload_date, const char *job_id, char **err_msg)
 	}
 
 	/* Initialize the first id to look in the database */
-	id = (char *) malloc (schedwi_strlen (job_id) + 1);
+	id = (unsigned long long int *)
+				malloc (sizeof (unsigned long long int));
 	if (id == NULL || lwc_addEndLL (lst, id) != 0) {
 		if (id != NULL) {
 			free (id);
@@ -306,10 +297,10 @@ get_hierarchy_list (int workload_date, const char *job_id, char **err_msg)
 		}
 		return NULL;
 	}
-	strcpy (id, job_id);
+	*id = job_id;
 
 	/* Connect to the database */
-	sql = begin_mysql (err_msg);
+	sql = begin_sql (err_msg);
 	if (sql == NULL) {
 		lwc_delLL (lst, (void (*)(const void *))free);
 		return NULL;
@@ -319,13 +310,13 @@ get_hierarchy_list (int workload_date, const char *job_id, char **err_msg)
 	buf_len = 0;
 	while (1) {
 		/* Get the parent of the job */
-		ret = sql_select (&buf, &buf_len, err_msg, NULL, &rows, NULL,
+		ret = sql_select (&buf, &buf_len, err_msg, NULL, &rows,
 				SQL_GET_PARENT,
 				SQL_INT, (long int)workload_date,
-				SQL_STRING, id,
+				SQL_INT, (long int)*id,
 				SQL_END);
 		if (ret != 0) {
-			end_mysql ();
+			end_sql ();
 			lwc_delLL (lst, (void (*)(const void *))free);
 			if (buf != NULL) {
 				free (buf);
@@ -334,7 +325,7 @@ get_hierarchy_list (int workload_date, const char *job_id, char **err_msg)
 		}
 
 		/* Retrieve the result (only one row) */
-		row = (char **) lwc_delStartLL (rows);
+		row = (row_item_t *) lwc_delStartLL (rows);
 		if (row == NULL) {
 			/*
 			 * Nothing found - The root of the hierarchy
@@ -346,9 +337,10 @@ get_hierarchy_list (int workload_date, const char *job_id, char **err_msg)
 		lwc_delLL (rows, (void (*)(const void *)) sql_free_row);
 
 		/* Copy and add the parent id to the list */
-		id = (char *) malloc (schedwi_strlen (row[0]) + 1);
+		id = (unsigned long long int *)
+				malloc (sizeof (unsigned long long int));
 		if (id == NULL || lwc_addEndLL (lst, id) != 0) {
-			end_mysql ();
+			end_sql ();
 			if (id != NULL) {
 				free (id);
 			}
@@ -367,12 +359,12 @@ get_hierarchy_list (int workload_date, const char *job_id, char **err_msg)
 			}
 			return NULL;
 		}
-		strcpy (id, row[0]);
+		*id = (unsigned long long int) sql_row_item2ll (&(row[0]));
 		sql_free_row (row);
 	}
 
 	/* Close the database and free the working buffer */
-	end_mysql ();
+	end_sql ();
 	if (buf != NULL) {
 		free (buf);
 	}
@@ -392,8 +384,9 @@ get_hierarchy_list (int workload_date, const char *job_id, char **err_msg)
 int
 hierarchy_list_to_str (lwc_LL *lst, char **result)
 {
-	char *s, *id;
-	unsigned int i;
+	char *s;
+	unsigned int i, n;
+	unsigned long long int *id;
 
 	if (lwc_getNumNode (lst) <= 0 || result == NULL) {
 		return 0;
@@ -406,10 +399,10 @@ hierarchy_list_to_str (lwc_LL *lst, char **result)
 
 	i = 0;
 	lwc_rewindLL (lst);
-	while ((id = (char *)lwc_nextLL (lst)) != NULL) {
-		strcpy (s + i, id);
-		i += schedwi_strlen (s + i);
-		s[i++] = ',';
+	while ((id = (unsigned long long int *)lwc_nextLL (lst)) != NULL) {
+		n = copy_ulltostr (*id, s + i);
+		s[i + n] = ',';
+		i += n + 1;
 	}
 	if (i > 0) {
 		s[i - 1] = '\0';
@@ -448,7 +441,7 @@ duplicate_hierarchy_list (lwc_LL *lst, char **err_msg)
 		}
 		return NULL;
 	}
-	
+
 	lwc_rewindLL (lst);
 	while ((id = (char *)lwc_nextLL (lst)) != NULL) {
 		if (lwc_addEndLL (copy_lst, id) != 0) {
@@ -475,38 +468,28 @@ duplicate_hierarchy_list (lwc_LL *lst, char **err_msg)
  *
  * Return:
  *     0 --> No error.  values contains the retrieved parameters and must be
- *           freed by the caller by sql_free_row().  If values_len is not
- *           NULL it contains the length of the values stored in values
- *           (without the trailing '\0') and must be freed by the caller by
- *           free().
+ *           freed by the caller by sql_free_row().
  *           If the parameters has not been found in the hierarchy (ie. no
- *           ancestor defines these parameters), values and values_len are
- *           set to NULL.
- *     1 --> Memory allocation error (if err_msg is not NULL it contains an
+ *           ancestor defines these parameters), values is set to NULL.
+ *    -1 --> Memory allocation error (if err_msg is not NULL it contains an
  *           error message which must be freed by the caller.  *err_msg may be
  *           NULL if there is not enough memory to store the error message).
- *     2 --> Database connection error (if err_msg is not NULL it contains an
- *           error message which must be freed by the caller.  *err_msg may be
- *           NULL if there is not enough memory to store the error message).
- * other --> Database error (if err_msg is not NULL it contains an error
+ *    -2 --> Database error (if err_msg is not NULL it contains an error
  *           message which must be freed by the caller.  *err_msg may be NULL
  *           if there is not enough memory to store the error message).
- *           The returned value is the one returned by mysql_errno().
  */
-unsigned int
-get_job_parameters (	int workload_date,
-			lwc_LL *lst,
+int
+get_job_parameters (	int workload_date, lwc_LL *lst,
 			const char *tablename, const char *colnames,
-			char ***values, unsigned long int **values_len,
-			char **err_msg)
+			row_item_t **values, char **err_msg)
 {
-	MYSQL *sql;
-	unsigned int ret;
-	lwc_LL *rows, *rows_len;
-	char **row;
-	char *id, *buf;
+	dbi_conn sql;
+	int ret;
+	lwc_LL *rows;
+	row_item_t *row;
+	char *buf;
 	unsigned int buf_len;
-	unsigned long int *row_len;
+	unsigned long long int *id;
 
 #if HAVE_ASSERT_H
 	assert (   lst != NULL && tablename != NULL && colnames != NULL
@@ -514,26 +497,25 @@ get_job_parameters (	int workload_date,
 #endif
 
 	/* Connect to the database */
-	sql = begin_mysql (err_msg);
+	sql = begin_sql (err_msg);
 	if (sql == NULL) {
-		return 2;
+		return -2;
 	}
 
 	buf = NULL;
 	buf_len = 0;
 	lwc_rewindLL (lst);
-	while ((id = (char *)lwc_nextLL (lst)) != NULL) {
+	while ((id = (unsigned long long int *)lwc_nextLL (lst)) != NULL) {
 		/* Try to get the required parameter */
-		ret = sql_select (&buf, &buf_len, err_msg, NULL,
-				&rows, &rows_len,
+		ret = sql_select (&buf, &buf_len, err_msg, NULL, &rows,
 				SQL_GET_JOB_PARAMS,
 				SQL_STRING_NON_ESCAPE, colnames,
 				SQL_STRING_NON_ESCAPE, tablename,
 				SQL_INT, (long int)workload_date,
-				SQL_STRING, id,
+				SQL_INT, (long int)*id,
 				SQL_END);
 		if (ret != 0) {
-			end_mysql ();
+			end_sql ();
 			if (buf != NULL) {
 				free (buf);
 			}
@@ -541,32 +523,22 @@ get_job_parameters (	int workload_date,
 		}
 
 		/* Retrieve the result (only one row) */
-		row = (char **) lwc_delStartLL (rows);
+		row = (row_item_t *) lwc_delStartLL (rows);
 		if (row == NULL) {
 			/*
 			 * Nothing found - The same request will then be
 			 * issued for the parent
 			 */
 			lwc_delLL (rows, (void (*)(const void *))sql_free_row);
-			lwc_delLL (rows_len, (void (*)(const void *))free);
 			continue;
 		}
 		lwc_delLL (rows, (void (*)(const void *)) sql_free_row);
 
-		/* Retrieve the fields len */
-		row_len = (unsigned long int *) lwc_delStartLL (rows_len);
-		lwc_delLL (rows_len, (void (*)(const void *))free);
-
 		/* Copy the returned values */
 		*values = row;
 
-		/* Copy the values length */
-		if (values_len != NULL) {
-			*values_len = row_len;
-		}
-		
 		/* Close the database and free the working buffer */
-		end_mysql ();
+		end_sql ();
 		if (buf != NULL) {
 			free (buf);
 		}
@@ -578,15 +550,12 @@ get_job_parameters (	int workload_date,
 	 */
 
 	/* Close the database and free the working buffer */
-	end_mysql ();
+	end_sql ();
 	if (buf != NULL) {
 		free (buf);
 	}
-
 	*values = NULL;
-	if (values_len != NULL) {
-		*values_len = NULL;
-	}
+
 	return 0;
 }
 
@@ -597,34 +566,26 @@ get_job_parameters (	int workload_date,
  * this function.
  *
  * Return:
- *     0 --> No error.  value contains the retrieved parameter and must be
- *           freed by the caller (by free()).  If value_len is not NULL it
- *           contains the length of the value stored in value (without the
- *           added trailing '\0').
+ *     0 --> No error.  value contains the retrieved parameter.
  *           If the parameter has not been found in the hierarchy (ie. no
- *           ancestor defines this parameter), value is set to NULL and
- *           value_len (if not NULL) is set to 0.
- *     1 --> Memory allocation error (if err_msg is not NULL it contains an
+ *           ancestor defines this parameter), value is set to the type
+ *           RES_END (value->type == RES_END)
+ *           If the parameter is set and is a string, the caller must free the
+ *           value_string member.
+ *    -1 --> Memory allocation error (if err_msg is not NULL it contains an
  *           error message which must be freed by the caller.  *err_msg may be
  *           NULL if there is not enough memory to store the error message).
- *     2 --> Database connection error (if err_msg is not NULL it contains an
- *           error message which must be freed by the caller.  *err_msg may be
- *           NULL if there is not enough memory to store the error message).
- * other --> Database error (if err_msg is not NULL it contains an error
+ *    -2 --> Database error (if err_msg is not NULL it contains an error
  *           message which must be freed by the caller.  *err_msg may be NULL
  *           if there is not enough memory to store the error message).
- *           The returned value is the one returned by mysql_errno().
  */
-unsigned int
-get_job_parameter (	int workload_date,
-			lwc_LL *lst,
+int
+get_job_parameter (	int workload_date, lwc_LL *lst,
 			const char *tablename, const char *colname,
-			char **value, unsigned long int *value_len,
-			char **err_msg)
+			row_item_t *value, char **err_msg)
 {
-	unsigned int ret;
-	char **row;
-	unsigned long int *row_len;
+	int ret;
+	row_item_t *row;
 	char *s;
 
 #if HAVE_ASSERT_H
@@ -633,48 +594,44 @@ get_job_parameter (	int workload_date,
 #endif
 
 
-	ret = get_job_parameters (	workload_date, lst,
-					tablename, colname,
-					&row, &row_len,
-					err_msg);
+	ret = get_job_parameters (	workload_date, lst, tablename, colname,
+					&row, err_msg);
 	if (ret != 0) {
 		return ret;
 	}
 
 	/* Parameter not found */
-	if (row == NULL || row_len == NULL) {
-		/* Return NULL in *value */
-		*value = NULL;
-		if (value_len != NULL) {
-			*value_len = 0;
-		}
+	if (row == NULL) {
+		value->type = RES_END;
 		return 0;
 	}
 
 	/* Copy the returned value */
-	s = (char *) malloc (row_len[0] + 1);
-	if (s == NULL) {
-		sql_free_row (row);
-		free (row_len);
-		if (err_msg != NULL) {
-			s = _("Memory allocation error");
-			*err_msg = (char *) malloc (schedwi_strlen (s) + 1);
-			if (*err_msg != NULL) {
-				strcpy (*err_msg, s);
+	if (row[0].type == RES_STRING || row[0].type == RES_BIN) {
+		s = (char *) malloc (row[0].len + 1);
+		if (s == NULL) {
+			sql_free_row (row);
+			if (err_msg != NULL) {
+				s = _("Memory allocation error");
+				*err_msg = (char *) malloc (schedwi_strlen (s)
+								+ 1);
+				if (*err_msg != NULL) {
+					strcpy (*err_msg, s);
+				}
 			}
+			return -1;
 		}
-		return 1;
+		schedwi_memcpy (s, row[0].value_string, row[0].len);
+		s[row[0].len]       = '\0';
+		value->type         = row[0].type;
+		value->len          = row[0].len;
+		value->value_string = s;
+	}
+	else {
+		schedwi_memcpy (value, &(row[0]), sizeof (row_item_t));
 	}
-	schedwi_memcpy (s, row[0], row_len[0]);
-	s[row_len[0]] = '\0';
 
-	/* Copy the value length */
-	if (value_len != NULL) {
-		*value_len = row_len[0];
-	}
-	*value = s;
 	sql_free_row (row);
-	free (row_len);
 
 	return 0;
 }
@@ -683,45 +640,34 @@ get_job_parameter (	int workload_date,
 /*
  * Retrieve parameters for a job hierarchy.  The provided hierarchy list
  * (in lst) must have been built by get_hierarchy_list() prior to calling
- * this function.  The provided rows and rows_len parameters are filled with
- * the retrieved parameters.  rows_len may be NULL if the row lengths are not
- * required.  rows must be freed by the caller by
+ * this function.  The provided rows parameter is filled with
+ * the retrieved parameters.  rows must be freed by the caller by
  *     lwc_delLL (rows, (void (*)(const void *))sql_free_row)
- * and rows_len must be freed by
- *     lwc_delLL (rows_len, (void (*)(const void *))free)
- * If there is no row retrieved from the database, rows (and rows_len if not
- * NULL) is set to NULL.
+ * If there is no row retrieved from the database, rows is set to NULL.
  *
  * Return:
- *     0 --> No error.  rows and rows_len (if not NULL) are set and must be
- *           freed by the caller by
+ *     0 --> No error.  rows is set and must be freed by the caller by
  *               lwc_delLL (rows, (void (*)(const void *))sql_free_row)
- *               lwc_delLL (rows_len, (void (*)(const void *))free)
- *     1 --> Memory allocation error (if err_msg is not NULL it contains an
+ *    -1 --> Memory allocation error (if err_msg is not NULL it contains an
  *           error message which must be freed by the caller.  *err_msg may be
  *           NULL if there is not enough memory to store the error message).
- *     2 --> Database connection error (if err_msg is not NULL it contains an
- *           error message which must be freed by the caller.  *err_msg may be
- *           NULL if there is not enough memory to store the error message).
- * other --> Database error (if err_msg is not NULL it contains an error
+ *    -2 --> Database error (if err_msg is not NULL it contains an error
  *           message which must be freed by the caller.  *err_msg may be NULL
  *           if there is not enough memory to store the error message).
- *           The returned value is the one returned by mysql_errno().
  */
-unsigned int
+int
 get_job_parameters_multi_row (
-		int workload_date,
-		lwc_LL *lst,
+		int workload_date, lwc_LL *lst,
 		const char *tablename, const char *colnames,
 		const char *sort_colname,
-		lwc_LL **rows, lwc_LL **rows_len,
-		char **err_msg)
+		lwc_LL **rows, char **err_msg)
 {
-	MYSQL *sql;
-	unsigned int ret;
-	lwc_LL *r, *r_len;
-	char *id, *buf;
+	dbi_conn sql;
+	int ret;
+	lwc_LL *r;
+	char *buf;
 	unsigned int buf_len;
+	unsigned long long int *id;
 
 #if HAVE_ASSERT_H
 	assert (   lst != NULL && tablename != NULL && colnames != NULL
@@ -729,40 +675,38 @@ get_job_parameters_multi_row (
 #endif
 
 	/* Connect to the database */
-	sql = begin_mysql (err_msg);
+	sql = begin_sql (err_msg);
 	if (sql == NULL) {
-		return 2;
+		return -2;
 	}
 
 	buf = NULL;
 	buf_len = 0;
 	lwc_rewindLL (lst);
-	while ((id = (char *)lwc_nextLL (lst)) != NULL) {
+	while ((id = (unsigned long long int *)lwc_nextLL (lst)) != NULL) {
 		/* Try to get the required parameter */
 		if (sort_colname == NULL) {
-			ret = sql_select (&buf, &buf_len, err_msg, NULL,
-				&r, &r_len,
+			ret = sql_select (&buf, &buf_len, err_msg, NULL, &r,
 				SQL_GET_JOB_PARAMS,
 				SQL_STRING_NON_ESCAPE, colnames,
 				SQL_STRING_NON_ESCAPE, tablename,
 				SQL_INT, (long int)workload_date,
-				SQL_STRING, id,
+				SQL_INT, (long int) *id,
 				SQL_END);
 		}
 		else {
-			ret = sql_select (&buf, &buf_len, err_msg, NULL,
-				&r, &r_len,
+			ret = sql_select (&buf, &buf_len, err_msg, NULL, &r,
 				SQL_GET_JOB_PARAMS_SORTED,
 				SQL_STRING_NON_ESCAPE, colnames,
 				SQL_STRING_NON_ESCAPE, tablename,
 				SQL_INT, (long int)workload_date,
-				SQL_STRING, id,
+				SQL_INT, (long int) *id,
 				SQL_STRING_NON_ESCAPE, sort_colname,
 				SQL_END);
 		}
 
 		if (ret != 0) {
-			end_mysql ();
+			end_sql ();
 			if (buf != NULL) {
 				free (buf);
 			}
@@ -775,21 +719,13 @@ get_job_parameters_multi_row (
 		 */
 		if (lwc_getNumNode (r) <= 0) {
 			lwc_delLL (r, (void (*)(const void *))sql_free_row);
-			lwc_delLL (r_len, (void (*)(const void *))free);
 			continue;
 		}
 
 		*rows = r;
-		if (rows_len != NULL) {
-			*rows_len = r_len;
-		}
-		else {
-			lwc_delLL (r_len, (void (*)(const void *))free);
-		}
-
 
 		/* Close the database and free the working buffer */
-		end_mysql ();
+		end_sql ();
 		if (buf != NULL) {
 			free (buf);
 		}
@@ -800,12 +736,9 @@ get_job_parameters_multi_row (
 	 * Parameters not found in the database
 	 */
 	*rows = NULL;
-	if (rows_len != NULL) {
-		*rows_len = NULL;
-	}
 
 	/* Close the database and free the working buffer */
-	end_mysql ();
+	end_sql ();
 	if (buf != NULL) {
 		free (buf);
 	}
@@ -821,37 +754,29 @@ get_job_parameters_multi_row (
  * row.  Its first parameter is the provided user_data object, the second
  * is the row array and the last is the array of each row size.
  * If callback returns a value other than 0, get_job_parameters_multi()
- * returns a value of 3.
+ * returns a value of 2.
  *
  * Return:
+ *     2 --> The callback function didn't return 0
  *     0 --> No error.
- *     1 --> Memory allocation error (if err_msg is not NULL it contains an
+ *    -1 --> Memory allocation error (if err_msg is not NULL it contains an
  *           error message which must be freed by the caller.  *err_msg may be
  *           NULL if there is not enough memory to store the error message).
- *     2 --> Database connection error (if err_msg is not NULL it contains an
- *           error message which must be freed by the caller.  *err_msg may be
- *           NULL if there is not enough memory to store the error message).
- *     3 --> The callback function didn't return 0
- * other --> Database error (if err_msg is not NULL it contains an error
+ *    -2 --> Database error (if err_msg is not NULL it contains an error
  *           message which must be freed by the caller.  *err_msg may be NULL
  *           if there is not enough memory to store the error message).
- *           The returned value is the one returned by mysql_errno().
  */
-unsigned int
+int
 get_job_parameters_multi (
-		int workload_date,
-		lwc_LL *lst,
+		int workload_date, lwc_LL *lst,
 		const char *tablename, const char *colnames,
 		const char *sort_colname,
-		int (*callback)(void *, char **, unsigned long int *),
-		void *user_data,
+		int (*callback)(void *, row_item_t *), void *user_data,
 		char **err_msg)
 {
-	unsigned int ret;
-	int callback_ret;
-	lwc_LL *rows, *rows_len;
-	char **row;
-	unsigned long int *row_len;
+	int ret;
+	lwc_LL *rows;
+	row_item_t *row;
 
 #if HAVE_ASSERT_H
 	assert (   lst != NULL && tablename != NULL && colnames != NULL
@@ -861,32 +786,22 @@ get_job_parameters_multi (
 	/* Retrieve the rows */
 	ret = get_job_parameters_multi_row (	workload_date,
 						lst, tablename, colnames,
-						sort_colname,
-						&rows, &rows_len,
-						err_msg);
+						sort_colname, &rows, err_msg);
 	if (ret != 0) {
 		return ret;
 	}
 
 	/* Call the callback function for each row */
-	while ((row = (char **) lwc_delStartLL (rows)) != NULL) {
-		row_len = (unsigned long int *) lwc_delStartLL (rows_len);
-			
-		callback_ret = callback (user_data, row, row_len);
+	while ((row = (row_item_t *) lwc_delStartLL (rows)) != NULL) {
 
+		ret = callback (user_data, row);
 		sql_free_row (row);
-		if (row_len != NULL) {
-			free (row_len);
-		}
-
-		if (callback_ret != 0) {
+		if (ret != 0) {
 			lwc_delLL (rows, (void (*)(const void *))sql_free_row);
-			lwc_delLL (rows_len, (void (*)(const void *))free);
-			return 3;
+			return 2;
 		}
 	}
 	lwc_delLL (rows, (void (*)(const void *))sql_free_row);
-	lwc_delLL (rows_len, (void (*)(const void *))free);
 
 	return 0;
 }
@@ -898,14 +813,11 @@ get_job_parameters_multi (
  * this function.
  *
  * Return:
- *     0 --> No error.  values contains the retrieved parameters and must be
- *           freed by the caller by sql_free_row().  If values_len is not
- *           NULL it contains the length of the values stored in values
- *           (without the trailing '\0') and must be freed by the caller by
- *           free().
+ *     0 --> No error.  values contains the retrieved row and must be
+ *           freed by the caller by sql_free_row().
  *           If the parameters has not been found in the hierarchy (ie. all
  *           ancestors have a value of -1 in the first provided column),
- *           values and values_len are set to NULL.
+ *           values is set to NULL.
  *     1 --> Memory allocation error (if err_msg is not NULL it contains an
  *           error message which must be freed by the caller.  *err_msg may be
  *           NULL if there is not enough memory to store the error message).
@@ -915,24 +827,21 @@ get_job_parameters_multi (
  * other --> Database error (if err_msg is not NULL it contains an error
  *           message which must be freed by the caller.  *err_msg may be NULL
  *           if there is not enough memory to store the error message).
- *           The returned value is the one returned by mysql_errno().
  */
-unsigned int
+int
 get_job_parameters_by_id (
-			int workload_date,
-			lwc_LL *lst,
+			int workload_date, lwc_LL *lst,
 			const char *tablename, const char *colnames,
-			const char *not_defined,
-			char ***values, unsigned long int **values_len,
-			char **err_msg)
+			const row_item_t *not_defined,
+			row_item_t **values, char **err_msg)
 {
-	MYSQL *sql;
-	unsigned int ret;
-	lwc_LL *rows, *rows_len;
-	char **row;
-	char *id, *buf;
+	dbi_conn sql;
+	int ret;
+	lwc_LL *rows;
+	row_item_t *row;
+	char *buf;
 	unsigned int buf_len;
-	unsigned long int *row_len;
+	unsigned long long int *id;
 
 #if HAVE_ASSERT_H
 	assert (   lst != NULL && not_defined != NULL
@@ -940,26 +849,25 @@ get_job_parameters_by_id (
 #endif
 
 	/* Connect to the database */
-	sql = begin_mysql (err_msg);
+	sql = begin_sql (err_msg);
 	if (sql == NULL) {
-		return 2;
+		return -2;
 	}
 
 	buf = NULL;
 	buf_len = 0;
 	lwc_rewindLL (lst);
-	while ((id = (char *)lwc_nextLL (lst)) != NULL) {
+	while ((id = (unsigned long long int *)lwc_nextLL (lst)) != NULL) {
 		/* Try to get the required parameter */
-		ret = sql_select (&buf, &buf_len, err_msg, NULL,
-				&rows, &rows_len,
+		ret = sql_select (&buf, &buf_len, err_msg, NULL, &rows,
 				SQL_GET_JOB_PARAMS_TIME,
 				SQL_STRING_NON_ESCAPE, colnames,
 				SQL_STRING_NON_ESCAPE, tablename,
 				SQL_INT, (long int)workload_date,
-				SQL_STRING, id,
+				SQL_INT, (long int) *id,
 				SQL_END);
 		if (ret != 0) {
-			end_mysql ();
+			end_sql ();
 			if (buf != NULL) {
 				free (buf);
 			}
@@ -967,33 +875,25 @@ get_job_parameters_by_id (
 		}
 
 		/* Retrieve the result (only one row) */
-		row = (char **) lwc_delStartLL (rows);
-		if (row == NULL || strcmp (not_defined, row[0]) == 0) {
+		row = (row_item_t *) lwc_delStartLL (rows);
+		if (	   row == NULL
+			|| sql_row_item_cmp (not_defined, &(row[0])) == 0)
+		{
 			/*
 			 * Nothing found - The same request will then be
 			 * issued for the parent
 			 */
 			sql_free_row (row);
 			lwc_delLL (rows, (void (*)(const void *))sql_free_row);
-			lwc_delLL (rows_len, (void (*)(const void *))free);
 			continue;
 		}
 		lwc_delLL (rows, (void (*)(const void *)) sql_free_row);
 
-		/* Retrieve the fields len */
-		row_len = (unsigned long int *) lwc_delStartLL (rows_len);
-		lwc_delLL (rows_len, (void (*)(const void *))free);
-
-		/* Copy the returned values */
+		/* Copy the returned row */
 		*values = row;
 
-		/* Copy the values length */
-		if (values_len != NULL) {
-			*values_len = row_len;
-		}
-		
 		/* Close the database and free the working buffer */
-		end_mysql ();
+		end_sql ();
 		if (buf != NULL) {
 			free (buf);
 		}
@@ -1005,15 +905,12 @@ get_job_parameters_by_id (
 	 */
 
 	/* Close the database and free the working buffer */
-	end_mysql ();
+	end_sql ();
 	if (buf != NULL) {
 		free (buf);
 	}
-
 	*values = NULL;
-	if (values_len != NULL) {
-		*values_len = NULL;
-	}
+
 	return 0;
 }
 
@@ -1022,7 +919,7 @@ get_job_parameters_by_id (
  * Split the provided string using the provided separator
  *
  * Return:
- *   A string array (last element is NULL) to be freed by te caller using
+ *   A string array (last element is NULL) to be freed by the caller using
  *            split_array_destroy().  If not NULL, array_len contains the
  *            array len (last NULL element not included)
  *   OR
@@ -1123,26 +1020,30 @@ split_array_destroy (char **a)
  *     0 --> No error.  If not NULL, j_id, j_parent_id and j_type contain the
  *           job ID, its parent jobset ID and its type.  They must be freed by
  *           the caller using free()
- *     1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
  *           with user_data_error_func as its first parameter and the error
  *           message as the second parameter)
- * other --> SQL error (if error_func() is not NULL, it is called with
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
  *           user_data_error_func as its first parameter and the error message
  *           as the second parameter)
  */
-unsigned int
+int
 get_job_id (	const char *job_name,
-		char **j_id, char **j_parent_id, char **j_type,
-		void (*error_func)(void *, const char *, unsigned int),
+		unsigned long long int *j_id,
+		unsigned long long int *j_parent_id,
+		int *j_type,
+		void (*error_func)(void *, const char *, int),
 		void *user_data_error_func)
 {
-	MYSQL *sql;
+	dbi_conn sql;
 	char *err_msg = NULL;
 	lwc_LL *rows;
-	char **row;
+	row_item_t *row;
+	int ret;
 	char **split, **a;
-	char *s, *s_id, *s_parent_id, *s_type;
-	unsigned int i, j, array_len, str_len, ret;
+	char *s;
+	unsigned int i, j, array_len, str_len;
+	long long int parent_id;
 
 #if HAVE_ASSERT_H
 	assert (job_name != NULL);
@@ -1156,66 +1057,23 @@ get_job_id (	const char *job_name,
 					_("Memory allocation error"),
 					0);
 		}
-		return 1;
+		return -1;
 	}
 
 	/* Empty path.  The details of the root jobset are returned */
 	if (array_len == 0) {
 		split_array_destroy (split);
-		s_id = (char *) malloc (2);
-		if (s_id == NULL) {
-			if (error_func != NULL) {
-				error_func (	user_data_error_func,
-						_("Memory allocation error"),
-						0);
-			}
-			return 1;
-		}
-		s_id[0] = '1';
-		s_id[1] = '\0';
-		s_parent_id = (char *) malloc (2);
-		if (s_parent_id == NULL) {
-			free (s_id);
-			if (error_func != NULL) {
-				error_func (	user_data_error_func,
-						_("Memory allocation error"),
-						0);
-			}
-			return 1;
-		}
-		s_parent_id[0] = '0';
-		s_parent_id[1] = '\0';
-		s_type = (char *) malloc (2);
-		if (s_type == NULL) {
-			free (s_parent_id);
-			free (s_id);
-			if (error_func != NULL) {
-				error_func (	user_data_error_func,
-						_("Memory allocation error"),
-						0);
-			}
-			return 1;
-		}
-		s_type[0] = '0';
-		s_type[1] = '\0';
 
 		if (j_id != NULL) {
-			*j_id = s_id;
-		}
-		else {
-			free (s_id);
+			*j_id = 1;
 		}
+
 		if (j_parent_id != NULL) {
-			*j_parent_id = s_parent_id;
-		}
-		else {
-			free (s_parent_id);
+			*j_parent_id = 0;
 		}
+
 		if (j_type != NULL) {
-			*j_type = s_type;
-		}
-		else {
-			free (s_type);
+			*j_type = 0;
 		}
 		return 0;
 	}
@@ -1232,11 +1090,11 @@ get_job_id (	const char *job_name,
 					_("Memory allocation error"),
 					0);
 		}
-		return 1;
+		return -1;
 	}
 
 	/* Connect to the database */
-	sql = begin_mysql (&err_msg);
+	sql = begin_sql (&err_msg);
 	if (sql == NULL) {
 		split_array_destroy (split);
 		free (a);
@@ -1246,14 +1104,14 @@ get_job_id (	const char *job_name,
 		if (err_msg != NULL) {
 			free (err_msg);
 		}
-		return 2;
+		return -2;
 	}
 
 	/* Escape the jobset names and populate the array */
 	for (str_len = i = 0; i < array_len; i++) {
 		a[i] = sql_escape (sql, split[i]);
 		if (a[i] == NULL) {
-			end_mysql ();
+			end_sql ();
 			split_array_destroy (split);
 			for (j = 0; j < i; j++) {
 				free (a[j]);
@@ -1264,9 +1122,9 @@ get_job_id (	const char *job_name,
 						_("Memory allocation error"),
 						0);
 			}
-			return 1;
+			return -1;
 		}
-		str_len += schedwi_strlen (a[i]) + 2;
+		str_len += schedwi_strlen (a[i]);
 	}
 
 	/*
@@ -1275,7 +1133,7 @@ get_job_id (	const char *job_name,
 	 */
 	s = (char *) malloc (str_len + array_len + 1);
 	if (s == NULL) {
-		end_mysql ();
+		end_sql ();
 		split_array_destroy (split);
 		for (j = 0; j < array_len; j++) {
 			free (a[j]);
@@ -1286,15 +1144,13 @@ get_job_id (	const char *job_name,
 					_("Memory allocation error"),
 					0);
 		}
-		return 1;
+		return -1;
 	}
 
 	/* Contatenate the escaped jobset names in s */
 	for (j = i = 0; i < array_len; i++) {
-		s[j++] = '"';
 		strcpy (s + j, a[i]);
 		j += schedwi_strlen (a[i]);
-		s[j++] = '"';
 		s[j++] = ',';
 		free (a[i]);
 	}
@@ -1302,11 +1158,11 @@ get_job_id (	const char *job_name,
 	s[j - 1] = '\0';	/* Remove the last `,' */
 
 	/* Retrieve all the jobsets details */
-	ret = sql_select (	NULL, NULL, &err_msg, NULL, &rows, NULL,
+	ret = sql_select (	NULL, NULL, &err_msg, NULL, &rows,
 				SQL_GET_JOB_ID,
 				SQL_STRING_NON_ESCAPE, s,
 				SQL_END);
-	end_mysql ();
+	end_sql ();
 	free (s);
 	if (ret != 0) {
 		split_array_destroy (split);
@@ -1321,14 +1177,14 @@ get_job_id (	const char *job_name,
 
 	/* Check that the retrieved jobsets are part of the provided path */
 	row = NULL;
-	s = "0";
+	parent_id  = 0;
 	for (i = 0; i < array_len; i++) {
 		lwc_rewindLL (rows);
-		while ((row = (char **)lwc_nextLL (rows)) != NULL) {
-			if (	   strcmp (row[2], split[i]) == 0
-				&& strcmp (row[1], s) == 0)
+		while ((row = (row_item_t *)lwc_nextLL (rows)) != NULL) {
+			if (	   parent_id == sql_row_item2ll (&(row[1]))
+				&& strcmp (row[2].value_string, split[i]) == 0)
 			{
-				s = row[0];
+				parent_id = sql_row_item2ll (&(row[0]));
 				break;
 			}
 		}
@@ -1339,74 +1195,29 @@ get_job_id (	const char *job_name,
 			if (error_func != NULL) {
 				error_func (	user_data_error_func,
 						_("Unknown jobset"),
-						ret);
+						0);
 			}
-			return 2;
+			return -2;
 		}
 	}
 	split_array_destroy (split);
 
 	/* Copy the details of the `leaf' job/jobset */
-	s_id = (char *)malloc (schedwi_strlen (row[0]) + 1);
-	if (s_id == NULL) {
-		lwc_delLL (rows, (void (*)(const void *)) sql_free_row);
-		if (error_func != NULL) {
-			error_func (	user_data_error_func,
-					_("Memory allocation error"),
-					0);
-		}
-		return 1;
+	if (j_id != NULL) {
+		*j_id = (unsigned long long int) sql_row_item2ll (&(row[0]));
 	}
-	strcpy (s_id, row[0]);
 
-	s_parent_id = (char *)malloc (schedwi_strlen (row[1]) + 1);
-	if (s_parent_id == NULL) {
-		lwc_delLL (rows, (void (*)(const void *)) sql_free_row);
-		free (s_id);
-		if (error_func != NULL) {
-			error_func (	user_data_error_func,
-					_("Memory allocation error"),
-					0);
-		}
-		return 1;
+	if (j_parent_id != NULL) {
+		*j_parent_id =
+			(unsigned long long int) sql_row_item2ll (&(row[1]));
 	}
-	strcpy (s_parent_id, row[1]);
 
-	s_type = (char *)malloc (schedwi_strlen (row[3]) + 1);
-	if (s_type == NULL) {
-		lwc_delLL (rows, (void (*)(const void *)) sql_free_row);
-		free (s_id);
-		free (s_parent_id);
-		if (error_func != NULL) {
-			error_func (	user_data_error_func,
-					_("Memory allocation error"),
-					0);
-		}
-		return 1;
+	if (j_type != NULL) {
+		*j_type = (int) sql_row_item2ll (&(row[3]));
 	}
-	strcpy (s_type, row[3]);
 
 	lwc_delLL (rows, (void (*)(const void *)) sql_free_row);
 
-	if (j_id != NULL) {
-		*j_id = s_id;
-	}
-	else {
-		free (s_id);
-	}
-	if (j_parent_id != NULL) {
-		*j_parent_id = s_parent_id;
-	}
-	else {
-		free (s_parent_id);
-	}
-	if (j_type != NULL) {
-		*j_type = s_type;
-	}
-	else {
-		free (s_type);
-	}
-
 	return 0;
 }
 
