@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
+   Copyright (C) 2007-2010 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -24,58 +24,51 @@
 
 #include <schedwi.h>
 #include <lwc_linkedlist.h>
+#include <sql_common.h>
 
 /* Allow the use in C++ code.  */
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-extern unsigned int get_job_path OF((	const char *job_id, char **path,
-					char **err_msg));
+extern int get_job_path OF((	unsigned long long int job_id,
+				char **path, char **err_msg));
 extern lwc_LL *get_hierarchy_list OF((	int workload_date,
-					const char *job_id, char **err_msg));
+					unsigned long long int job_id,
+					char **err_msg));
 extern lwc_LL *duplicate_hierarchy_list OF((lwc_LL *lst, char **err_msg));
 extern int hierarchy_list_to_str OF((lwc_LL *lst, char **result));
-extern unsigned int get_job_parameters OF((
-			int workload_date,
-			lwc_LL *lst,
+extern int get_job_parameters OF((
+			int workload_date, lwc_LL *lst,
 			const char *tablename, const char *colnames,
-			char ***values, unsigned long int **values_len,
-			char **err_msg));
-extern unsigned int get_job_parameter OF((
-			int workload_date,
-			lwc_LL *lst,
+			row_item_t **values, char **err_msg));
+extern int get_job_parameter OF((
+			int workload_date, lwc_LL *lst,
 			const char *tablename, const char *colname,
-			char **value, unsigned long int *value_len,
-			char **err_msg));
-extern unsigned int get_job_parameters_multi_row OF((
-			int workload_date,
-			lwc_LL *lst,
+			row_item_t *value, char **err_msg));
+extern int get_job_parameters_multi_row OF((
+			int workload_date, lwc_LL *lst,
 			const char *tablename, const char *colnames,
 			const char *sort_colname,
-			lwc_LL **rows, lwc_LL **rows_len,
-			char **err_msg));
-extern unsigned int get_job_parameters_multi OF((
-			int workload_date,
-			lwc_LL *lst,
+			lwc_LL **rows, char **err_msg));
+extern int get_job_parameters_multi OF((
+			int workload_date, lwc_LL *lst,
 			const char *tablename, const char *colnames,
 			const char *sort_colname,
-			int (*callback)(void *, char **,
-					unsigned long int *),
-			void *user_data,
+			int (*callback)(void *, row_item_t *), void *user_data,
 			char **err_msg));
-extern unsigned int get_job_parameters_by_id OF((
-			int workload_date,
-			lwc_LL *lst,
+extern int get_job_parameters_by_id OF((
+			int workload_date, lwc_LL *lst,
 			const char *tablename, const char *colnames,
-			const char *not_defined,
-			char ***values, unsigned long int **values_len,
-			char **err_msg));
+			const row_item_t *not_defined,
+			row_item_t **values, char **err_msg));
 
-extern unsigned int get_job_id OF((
+extern int get_job_id OF((
 		const char *job_name,
-		char **j_id, char **j_parent_id, char **j_type,
-		void (*error_func)(void *, const char *, unsigned int),
+		unsigned long long int *j_id,
+		unsigned long long int *j_parent_id,
+		int *j_type,
+		void (*error_func)(void *, const char *, int),
 		void *user_data_error_func));
 
 #ifdef __cplusplus
