@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
+   Copyright (C) 2007-2010 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -39,18 +39,17 @@
 
 #include <utils.h>
 #include <lib_functions.h>
-#include <sql_common.h>
 #include <sql_hosts.h>
 
 #define SQL_HOSTS_LIST "SELECT id,hostname,portnum FROM hosts ORDER BY hostname"
-#define SQL_HOST_DELETE "DELETE FROM hosts WHERE id=\"%s\"" 
-#define SQL_HOST_DETAIL "SELECT id,hostname,portnum,sslenable,sslcert,description FROM hosts WHERE id=\"%s\"" 
-#define SQL_HOST_DETAIL_FROM_NAMES "SELECT id,hostname,portnum,sslenable,sslcert,description FROM hosts WHERE hostname IN (%s)" 
-#define SQL_HOST_NEW "INSERT INTO hosts (hostname,portnum,sslenable,sslcert,description) VALUES (\"%s\",\"%s\",%d,\"%s\",\"%s\")"
-#define SQL_HOST_UPDATE "UPDATE hosts SET hostname=\"%s\",portnum=\"%s\",sslenable=%d,sslcert=\"%s\",description=\"%s\" WHERE id=\"%s\""
-#define SQL_HOST_CERT "SELECT sslenable,sslcert FROM hosts WHERE hostname IN (%s) ORDER BY sslenable DESC" 
-#define SQL_HOST_REPLACE_UPDATE_NODESCR "UPDATE hosts SET sslenable=%d,sslcert=\"%s\" WHERE hostname=\"%s\" AND portnum=\"%s\""
-#define SQL_HOST_REPLACE_UPDATE_DESCR "UPDATE hosts SET sslenable=%d,sslcert=\"%s\",description=\"%s\" WHERE hostname=\"%s\" AND portnum=\"%s\""
+#define SQL_HOST_DELETE "DELETE FROM hosts WHERE id=%s"
+#define SQL_HOST_DETAIL "SELECT id,hostname,portnum,sslenable,sslcert,description FROM hosts WHERE id=%ld"
+#define SQL_HOST_DETAIL_FROM_NAMES "SELECT id,hostname,portnum,sslenable,sslcert,description FROM hosts WHERE hostname IN (%s) AND portnum=%s"
+#define SQL_HOST_NEW "INSERT INTO hosts (hostname,portnum,sslenable,sslcert,description) VALUES (%s,%s,%d,%s,%s)"
+#define SQL_HOST_UPDATE "UPDATE hosts SET hostname=%s,portnum=%s,sslenable=%d,sslcert=%s,description=%s WHERE id=%s"
+#define SQL_HOST_CERT "SELECT sslenable,sslcert FROM hosts WHERE hostname IN (%s) ORDER BY sslenable DESC"
+#define SQL_HOST_REPLACE_UPDATE_NODESCR "UPDATE hosts SET sslenable=%d,sslcert=%s WHERE hostname=%s AND portnum=%s"
+#define SQL_HOST_REPLACE_UPDATE_DESCR "UPDATE hosts SET sslenable=%d,sslcert=%s,description=%s WHERE hostname=%s AND portnum=%s"
 
 
 /*
@@ -72,21 +71,22 @@
  *         as the second parameter)
  */
 int
-sql_hosts_list (int (*add_row)(void *,const char *, const char *, const char *),
-		void *user_data_add_row,
-		void (*error_func)(void *, const char *, unsigned int),
-		void *user_data_error_func)
+sql_hosts_list (
+    int (*add_row)(void *,unsigned long long int, const char *, const char *),
+    void *user_data_add_row,
+    void (*error_func)(void *, const char *, int),
+    void *user_data_error_func)
 {
 	char *err_msg = NULL;
 	lwc_LL *rows;
-	unsigned int ret;
-	char **row;
+	int ret;
+	row_item_t *row;
 
 #if HAVE_ASSERT_H
 	assert (add_row != NULL);
 #endif
 
-	ret = sql_select (	NULL, NULL, &err_msg, NULL, &rows, NULL,
+	ret = sql_select (	NULL, NULL, &err_msg, NULL, &rows,
 				SQL_HOSTS_LIST, SQL_END);
 	if (ret != 0) {
 		if (error_func != NULL) {
@@ -98,11 +98,11 @@ sql_hosts_list (int (*add_row)(void *,const char *, const char *, const char *),
 		return -2;
 	}
 
-	while ((row = (char **) lwc_delStartLL (rows)) != NULL) {
-		if (add_row (	user_data_add_row,
-				row[ID],
-				row[HOSTNAME],
-				row[PORTNUM]) != 0)
+	while ((row = (row_item_t *) lwc_delStartLL (rows)) != NULL) {
+		if (add_row (user_data_add_row,
+			(unsigned long long int) sql_row_item2ll (&(row[ID])),
+			row[HOSTNAME].value_string,
+			row[PORTNUM].value_string) != 0)
 		{
 			sql_free_row (row);
 			lwc_delLL (rows, (void (*)(const void *))sql_free_row);
@@ -120,26 +120,26 @@ sql_hosts_list (int (*add_row)(void *,const char *, const char *, const char *),
  *
  * Return:
  *     0 --> No error
- *     1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
  *           with user_data_error_func as its first parameter and the error
  *           message as the second parameter)
- * other --> SQL error (if error_func() is not NULL, it is called with
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
  *           user_data_error_func as its first parameter and the error message
  *           as the second parameter)
  */
-unsigned int
+int
 sql_host_delete (	const char *host_id,
-			void (*error_func)(void *, const char *, unsigned int),
+			void (*error_func)(void *, const char *, int),
 			void *user_data_error_func)
 {
 	char *err_msg = NULL;
-	unsigned int ret;
+	int ret;
 
 #if HAVE_ASSERT_H
 	assert (host_id != NULL);
 #endif
 
-	ret = sql_non_select (	NULL, NULL, &err_msg, NULL, NULL,
+	ret = sql_non_select (	NULL, NULL, &err_msg, NULL, NULL, NULL,
 				SQL_HOST_DELETE, SQL_STRING, host_id, SQL_END);
 	if (ret != 0) {
 		if (error_func != NULL) {
@@ -159,35 +159,35 @@ sql_host_delete (	const char *host_id,
  *
  * Return:
  *     0 --> No error
- *     1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
  *           with user_data_error_func as its first parameter and the error
  *           message as the second parameter)
- * other --> SQL error (if error_func() is not NULL, it is called with
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
  *           user_data_error_func as its first parameter and the error message
  *           as the second parameter)
  */
-unsigned int
+int
 sql_host_get_main (	const char *host_id,
-			void (*set_host_id)(const char *, void *),
+			void (*set_host_id)(unsigned long long int, void *),
 			void (*set_host_name)(const char *, void *),
 			void (*set_host_port)(const char *, void *),
-			void (*set_host_SSL)(const char *, void *),
+			void (*set_host_SSL)(char, void *),
 			void (*set_host_cert)(const char *, void *),
 			void (*set_host_descr)(const char *, void *),
 			void *user_data_set_host,
-			void (*error_func)(void *, const char *, unsigned int),
+			void (*error_func)(void *, const char *, int),
 			void *user_data_error_func)
 {
 	char *err_msg = NULL;
-	unsigned int ret;
+	int ret;
 	lwc_LL *rows;
-	char **row;
+	row_item_t *row;
 
 #if HAVE_ASSERT_H
 	assert (host_id != NULL);
 #endif
 
-	ret = sql_select (	NULL, NULL, &err_msg, NULL, &rows, NULL,
+	ret = sql_select (	NULL, NULL, &err_msg, NULL, &rows,
 				SQL_HOST_DETAIL, SQL_STRING, host_id, SQL_END);
 
 	if (ret != 0) {
@@ -200,7 +200,7 @@ sql_host_get_main (	const char *host_id,
 		return ret;
 	}
 
-	row = (char **) lwc_delStartLL (rows);
+	row = (row_item_t *) lwc_delStartLL (rows);
 	if (row == NULL) {
 		lwc_delLL (rows, (void (*)(const void *)) sql_free_row);
 		if (error_func != NULL) {
@@ -208,16 +208,18 @@ sql_host_get_main (	const char *host_id,
 					_("Unknown host id in the database"),
 					ret);
 		}
-		return 2;
+		return -2;
 	}
 	lwc_delLL (rows, (void (*)(const void *)) sql_free_row);
 
-	set_host_id (row[ID], user_data_set_host);
-	set_host_name (row[HOSTNAME], user_data_set_host);
-	set_host_port (row[PORTNUM], user_data_set_host);
-	set_host_SSL (row[SSLENABLE], user_data_set_host);
-	set_host_cert (row[SSLCERT], user_data_set_host);
-	set_host_descr (row[DESCR], user_data_set_host);
+	set_host_id (	(unsigned long long int)sql_row_item2ll (&(row[ID])),
+			user_data_set_host);
+	set_host_name (row[HOSTNAME].value_string, user_data_set_host);
+	set_host_port (row[PORTNUM].value_string, user_data_set_host);
+	set_host_SSL (	(char)sql_row_item2ll (&(row[SSLENABLE])),
+			user_data_set_host);
+	set_host_cert (row[SSLCERT].value_string, user_data_set_host);
+	set_host_descr (row[DESCR].value_string, user_data_set_host);
 
 	sql_free_row (row);
 
@@ -229,46 +231,41 @@ sql_host_get_main (	const char *host_id,
  * Get the host details from the database
  *
  * Return:
- *     0 --> No error.  row_out and row_out_len are set and must be freed by
- *           the caller by
+ *     0 --> No error.  row_out is set and must be freed by the caller by
  *                   sql_free_row (row_out);
- *                   free (row_out_len);
- *           row_out is an array of string as follow:
- *                   row_out[0] --> host ID (the same as the host_id parameter) 
+ *           row_out is an array of row_item_t objects as follow:
+ *                   row_out[0] --> host ID (the same as the host_id parameter)
  *                   row_out[1] --> Host name
  *                   row_out[2] --> TCP port
- *                   row_out[3] --> SSL client
- *                   row_out[4] --> SSL client certificate
+ *                   row_out[3] --> SSL?
+ *                   row_out[4] --> SSL agent certificate
  *                   row_out[5] --> Host description
- *           row_out_len is an array of unsigned long int.  The elements
- *           are the lenght of the corresponding strings in row_out.  If
- *           row_out_len is NULL, no lenght information is returned.
- *     1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
  *           with user_data_error_func as its first parameter and the error
  *           message as the second parameter)
- * other --> SQL error (if error_func() is not NULL, it is called with
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
  *           user_data_error_func as its first parameter and the error message
  *           as the second parameter)
  */
-unsigned int
-sql_host_get_main_row (	const char *host_id,
-			char ***row_out,
-			unsigned long int **row_out_len,
-			void (*error_func)(void *, const char *, unsigned int),
+int
+sql_host_get_main_row (	unsigned long long int host_id,
+			row_item_t **row_out,
+			void (*error_func)(void *, const char *, int),
 			void *user_data_error_func)
 {
 	char *err_msg = NULL;
-	unsigned int ret;
-	lwc_LL *rows, *rows_len;
-	char **row;
-	unsigned long int *row_len;
+	int ret;
+	lwc_LL *rows;
+	row_item_t *row;
 
 #if HAVE_ASSERT_H
-	assert (host_id != NULL && row_out != NULL);
+	assert (row_out != NULL);
 #endif
 
-	ret = sql_select (	NULL, NULL, &err_msg, NULL, &rows, &rows_len,
-				SQL_HOST_DETAIL, SQL_STRING, host_id, SQL_END);
+	ret = sql_select (	NULL, NULL, &err_msg, NULL, &rows,
+				SQL_HOST_DETAIL,
+				SQL_INT, (long int)host_id,
+				SQL_END);
 
 	if (ret != 0) {
 		if (error_func != NULL) {
@@ -280,26 +277,18 @@ sql_host_get_main_row (	const char *host_id,
 		return ret;
 	}
 
-	row = (char **) lwc_delStartLL (rows);
+	row = (row_item_t *) lwc_delStartLL (rows);
 	lwc_delLL (rows, (void (*)(const void *)) sql_free_row);
-	row_len = (unsigned long int *) lwc_delStartLL (rows_len);
-	lwc_delLL (rows_len, (void (*)(const void *)) free);
-	if (row == NULL || row_len == NULL) {
+	if (row == NULL) {
 		if (error_func != NULL) {
 			error_func (	user_data_error_func,
 					_("Unknown host id in the database"),
 					ret);
 		}
-		return 2;
+		return -2;
 	}
 
 	*row_out = row;
-	if (row_out_len != NULL) {
-		*row_out_len = row_len;
-	}
-	else {
-		free (row_len);
-	}
 
 	return 0;
 }
@@ -310,43 +299,39 @@ sql_host_get_main_row (	const char *host_id,
  * host_names array contains all the known names of the searched host.
  *
  * Return:
- *     0 --> No error.  row_out and row_out_len are set and must be freed by
- *           the caller by
+ *     0 --> No error.  row_out is set and must be freed by the caller by
  *                   sql_free_row (row_out);
- *                   free (row_out_len);
- *           row_out is an array of string as follow:
- *                   row_out[0] --> host ID (the same as the host_id parameter) 
+ *           row_out is an array of row_item_t objects as follow:
+ *                   row_out[0] --> host ID (the same as the host_id parameter)
  *                   row_out[1] --> Host name
  *                   row_out[2] --> TCP port
- *                   row_out[3] --> SSL client
- *                   row_out[4] --> SSL client certificate
+ *                   row_out[3] --> SSL?
+ *                   row_out[4] --> SSL agent certificate
  *                   row_out[5] --> Host description
- *           row_out_len is an array of unsigned long int.  The elements
- *           are the lenght of the corresponding strings in row_out.  If
- *           row_out_len is NULL, no lenght information is returned.
- *     1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
  *           with user_data_error_func as its first parameter and the error
  *           message as the second parameter)
- * other --> SQL error (if error_func() is not NULL, it is called with
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
  *           user_data_error_func as its first parameter and the error message
  *           as the second parameter)
  */
-unsigned int
-sql_host_get_main_row_names (char **host_names,
-			char ***row_out,
-			unsigned long int **row_out_len,
-			void (*error_func)(void *, const char *, unsigned int),
+int
+sql_host_get_main_row_names (char **host_names, const char *portnum,
+			row_item_t **row_out,
+			void (*error_func)(void *, const char *, int),
 			void *user_data_error_func)
 {
 	char *err_msg = NULL;
-	unsigned int ret, i, len, j;
-	lwc_LL *rows, *rows_len;
-	char **row, *s, *tmp;
-	unsigned long int *row_len;
-	MYSQL *sql_handler;
+	unsigned int i, len, j;
+	lwc_LL *rows;
+	row_item_t *row;
+	char *s, *tmp;
+	dbi_conn sql_handler;
+	int ret;
+
 
 #if HAVE_ASSERT_H
-	assert (host_names != NULL && row_out != NULL);
+	assert (host_names != NULL && portnum != NULL && row_out != NULL);
 #endif
 
 	/*
@@ -363,36 +348,34 @@ sql_host_get_main_row_names (char **host_names,
 					_("Memory allocation error"),
 					0);
 		}
-		return 1;
+		return -1;
 	}
 
 	/* Get the SQL handler */
-	sql_handler = begin_mysql (&err_msg);
+	sql_handler = begin_sql (&err_msg);
 	if (sql_handler == NULL) {
 		free (s);
 		if (error_func != NULL) {
 			error_func (user_data_error_func, err_msg, 0);
 		}
-		return 2;
+		return -2;
 	}
-	
+
 	/* Build the string */
 	for (i = j = 0; host_names[i] != NULL; i++) {
 		tmp = sql_escape (sql_handler, host_names[i]);
 		if (tmp == NULL) {
-			end_mysql ();
+			end_sql ();
 			free (s);
 			if (error_func != NULL) {
 				error_func (	user_data_error_func,
 						_("Memory allocation error"),
 						0);
 			}
-			return 1;
+			return -1;
 		}
-		s[j++] = '"';
 		strcpy (s + j, tmp);
 		j += schedwi_strlen (tmp);
-		s[j++] = '"';
 		s[j++] = ',';
 		free (tmp);
 	}
@@ -404,12 +387,13 @@ sql_host_get_main_row_names (char **host_names,
 	}
 
 	err_msg = NULL;
-	ret = sql_select (	NULL, NULL, &err_msg, NULL, &rows, &rows_len,
+	ret = sql_select (	NULL, NULL, &err_msg, NULL, &rows,
 				SQL_HOST_DETAIL_FROM_NAMES,
 				SQL_STRING_NON_ESCAPE, s,
+				SQL_STRING, portnum,
 				SQL_END);
 	free (s);
-	end_mysql ();
+	end_sql ();
 
 	if (ret != 0) {
 		if (error_func != NULL) {
@@ -421,26 +405,18 @@ sql_host_get_main_row_names (char **host_names,
 		return ret;
 	}
 
-	row = (char **) lwc_delStartLL (rows);
+	row = (row_item_t *) lwc_delStartLL (rows);
 	lwc_delLL (rows, (void (*)(const void *)) sql_free_row);
-	row_len = (unsigned long int *) lwc_delStartLL (rows_len);
-	lwc_delLL (rows_len, (void (*)(const void *)) free);
-	if (row == NULL || row_len == NULL) {
+	if (row == NULL) {
 		if (error_func != NULL) {
 			error_func (	user_data_error_func,
 					_("Host not defined in the database"),
 					ret);
 		}
-		return 2;
+		return -2;
 	}
 
 	*row_out = row;
-	if (row_out_len != NULL) {
-		*row_out_len = row_len;
-	}
-	else {
-		free (row_len);
-	}
 
 	return 0;
 }
@@ -451,29 +427,30 @@ sql_host_get_main_row_names (char **host_names,
  *
  * Return:
  *     0 --> No error
- *     1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
  *           with user_data_error_func as its first parameter and the error
  *           message as the second parameter)
- * other --> SQL error (if error_func() is not NULL, it is called with
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
  *           user_data_error_func as its first parameter and the error message
  *           as the second parameter)
  */
-unsigned int
+int
 sql_host_new (	const char *hostname, const char *portnum,
 		char sslenable, const char *sslcert,
 		const char *description,
-		unsigned long int *new_host_id,
-		void (*error_func)(void *, const char *, unsigned int),
+		unsigned long long int *new_host_id,
+		void (*error_func)(void *, const char *, int),
 		void *user_data_error_func)
 {
 	char *err_msg = NULL;
-	unsigned int ret;
+	int ret;
 
 #if HAVE_ASSERT_H
 	assert (hostname != NULL && portnum != NULL && description != NULL);
 #endif
 
 	ret = sql_non_select (	NULL, NULL, &err_msg, new_host_id, NULL,
+				"hosts",
 				SQL_HOST_NEW,
 				SQL_STRING, hostname,
 				SQL_STRING, portnum,
@@ -499,30 +476,30 @@ sql_host_new (	const char *hostname, const char *portnum,
  *
  * Return:
  *     0 --> No error
- *     1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
  *           with user_data_error_func as its first parameter and the error
  *           message as the second parameter)
- * other --> SQL error (if error_func() is not NULL, it is called with
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
  *           user_data_error_func as its first parameter and the error message
  *           as the second parameter)
  */
-unsigned int
+int
 sql_host_update (	const char *host_id,
 			const char *hostname, const char *portnum,
 			char sslenable, const char *sslcert,
 			const char *description,
-			void (*error_func)(void *, const char *, unsigned int),
+			void (*error_func)(void *, const char *, int),
 			void *user_data_error_func)
 {
 	char *err_msg = NULL;
-	unsigned int ret;
+	int ret;
 
 #if HAVE_ASSERT_H
 	assert (host_id != NULL && hostname != NULL && portnum != NULL
 		&& description != NULL);
 #endif
 
-	ret = sql_non_select (	NULL, NULL, &err_msg, NULL, NULL,
+	ret = sql_non_select (	NULL, NULL, &err_msg, NULL, NULL, NULL,
 				SQL_HOST_UPDATE,
 				SQL_STRING, hostname,
 				SQL_STRING, portnum,
@@ -549,27 +526,27 @@ sql_host_update (	const char *host_id,
  * in the provided array)
  *
  * Return:
+ *     2 --> Host not found in the database
  *     0 --> No error.  sslenable, certificate and certificate_len are set (if
  *           not NULL).  certificate must be freed by the caller (by free())
- *     1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
  *           with user_data_error_func as its first parameter and the error
  *           message as the second parameter)
- *     2 --> Host not found in the database
- * other --> SQL error (if error_func() is not NULL, it is called with
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
  *           user_data_error_func as its first parameter and the error message
  *           as the second parameter)
  */
-unsigned int
+int
 sql_host_certificate (	char **names, char *sslenable,
 			char **certificate, unsigned long int *certificate_len,
-			void (*error_func)(void *, const char *, unsigned int),
+			void (*error_func)(void *, const char *, int),
 			void *user_data_error_func)
 {
 	char *err_msg = NULL;
-	lwc_LL *rows, *rows_len;
+	lwc_LL *rows;
+	row_item_t *row;
 	unsigned int ret, i, j, len_names;
-	char *s, **row;
-	unsigned long int *row_len;
+	char *s, *t;
 
 
 #if HAVE_ASSERT_H
@@ -580,13 +557,13 @@ sql_host_certificate (	char **names, char *sslenable,
 	for (len_names = i = 0; names[i] != NULL; i++) {
 		len_names += schedwi_strlen (names[i]);
 	}
-	len_names += 3 * i; /* For the two `"' and the `,' */
+	len_names += 3 * i; /* For the two `'' and the `,' */
 
 	/* names is empty */
 	if (i == 0) {
 		error_func (	user_data_error_func,
 				_("Host not found in the database"), 0);
-		return 2;
+		return -2;
 	}
 
 	s = (char *) malloc (len_names);
@@ -595,28 +572,39 @@ sql_host_certificate (	char **names, char *sslenable,
 			error_func (	user_data_error_func,
 					_("Memory allocation error"), 0);
 		}
-		return 1;
+		return -1;
 	}
 
 	/*
 	 * Copy all the names in a new string.  Each name are inserted
-	 * between `"'.  Names are separated by `,'
+	 * between `''.  Names are separated by `,'
 	 */
 	for (i = j = 0; names[i] != NULL; i++) {
-		s[j++] = '"';
-		strcpy (s + j, names[i]);
-		j += schedwi_strlen (names[i]);
-		s[j++] = '"';
-		s[j++] = ',';
+		/*
+		 * Basic check against SQL injections. Host names containing
+		 * the character `'' are  ignored.
+		 */
+		if (strchr (names[i], '\'') == NULL) {
+			s[j++] = '\'';
+			strcpy (s + j, names[i]);
+			j += schedwi_strlen (names[i]);
+			s[j++] = '\'';
+			s[j++] = ',';
+		}
 	}
-	s[j - 1] = '\0';
+	if (j != 0) {
+		s[j - 1] = '\0';
+	}
+	else {
+		s[0] = '\0';
+	}
+
 
 	/* SQL request */
-	ret = sql_select (	NULL, NULL, &err_msg, NULL, &rows, &rows_len,
-                                SQL_HOST_CERT,
-                                SQL_STRING_NON_ESCAPE, s,
-                                SQL_END);
-	free (s);
+	ret = sql_select (	NULL, NULL, &err_msg, NULL, &rows,
+				SQL_HOST_CERT,
+				SQL_STRING_NON_ESCAPE, s,
+				SQL_END);
 	if (ret != 0) {
 		if (error_func != NULL) {
 			error_func (user_data_error_func, err_msg, ret);
@@ -624,60 +612,63 @@ sql_host_certificate (	char **names, char *sslenable,
 		if (err_msg != NULL) {
 			free (err_msg);
 		}
+		free (s);
 		return ret;
 	}
 
 	/* Only keep the first row and delete the others */
-	row = (char **) lwc_delStartLL (rows);
-	row_len = (unsigned long int *) lwc_delStartLL (rows_len);
+	row = (row_item_t *) lwc_delStartLL (rows);
 	lwc_delLL (rows, (void (*)(const void *)) sql_free_row);
-	lwc_delLL (rows_len, (void (*)(const void *)) free);
-	if (row == NULL || row_len == NULL) {
-		if (row != NULL) {
-			sql_free_row (row);
-		}
-		if (row_len != NULL) {
-			free (row_len);
-		}
+	if (row == NULL) {
 		if (error_func != NULL) {
-			error_func (	user_data_error_func,
-					_("Host not found in the database"),
-					0);
+            t = (char *)malloc (
+                    schedwi_strlen (_("Host not found in the database: looking for "))
+                    + j + 2);
+            if (t == NULL) {
+			    error_func (	user_data_error_func,
+				    _("Host not found in the database"), 0);
+            }
+            else {
+                strcpy (t, _("Host not found in the database: looking for "));
+                strcat (t, s);
+			    error_func (user_data_error_func, t, 0);
+                free (t);
+            }
 		}
-		return 2;
+		free (s);
+		return -2;
 	}
+	free (s);
 
 	/*
-	 * row[0] --> SSL enable
+	 * row[0] --> SSL enabled
 	 * row[1] --> SSL certificate
 	 */
 	if (certificate != NULL) {
 		/* Copy the certificate in a new string */
-		s = (char *) malloc (row_len[1] + 1);
+		s = (char *) malloc (row[1].len + 1);
 		if (s == NULL) {
 			sql_free_row (row);
-			free (row_len);
 			if (error_func != NULL) {
 				error_func (user_data_error_func,
 					_("Memory allocation error"), 0);
 			}
-			return 1;
+			return -1;
 		}
-		strncpy (s, row[1], row_len[1]);
-		s[row_len[1]] = '\0';
+		schedwi_memcpy (s, row[1].value_string, row[1].len);
+		s[row[1].len] = '\0';
 		*certificate = s;
 	}
 
 	if (certificate_len != NULL) {
-		*certificate_len = row_len[1];
+		*certificate_len = row[1].len;
 	}
 
 	if (sslenable != NULL) {
-		*sslenable = (row[0][0] == '0') ? 0: 1;
+		*sslenable = (char) sql_row_item2ll (&(row[0]));
 	}
 
 	sql_free_row (row);
-	free (row_len);
 
 	return 0;
 }
@@ -688,22 +679,22 @@ sql_host_certificate (	char **names, char *sslenable,
  *
  * Return:
  *     0 --> No error
- *     1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
  *           with user_data_error_func as its first parameter and the error
  *           message as the second parameter)
- * other --> SQL error (if error_func() is not NULL, it is called with
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
  *           user_data_error_func as its first parameter and the error message
  *           as the second parameter)
  */
-unsigned int
+int
 sql_host_replace (const char *hostname, const char *portnum,
 		char sslenable, const char *sslcert,
 		const char *description,
-		void (*error_func)(void *, const char *, unsigned int),
+		void (*error_func)(void *, const char *, int),
 		void *user_data_error_func)
 {
 	char *err_msg = NULL;
-	unsigned int ret;
+	int ret;
 	unsigned long long int nb = 0;
 
 #if HAVE_ASSERT_H
@@ -712,7 +703,7 @@ sql_host_replace (const char *hostname, const char *portnum,
 
 	/* Try to update the existing host in the database */
 	if (description == NULL) {
-		ret = sql_non_select (	NULL, NULL, &err_msg, NULL, &nb,
+		ret = sql_non_select (	NULL, NULL, &err_msg, NULL, &nb, NULL,
 					SQL_HOST_REPLACE_UPDATE_NODESCR,
 					SQL_BOOL, sslenable,
 					SQL_STRING, sslcert,
@@ -721,7 +712,7 @@ sql_host_replace (const char *hostname, const char *portnum,
 					SQL_END);
 	}
 	else {
-		ret = sql_non_select (	NULL, NULL, &err_msg, NULL, &nb,
+		ret = sql_non_select (	NULL, NULL, &err_msg, NULL, &nb, NULL,
 					SQL_HOST_REPLACE_UPDATE_DESCR,
 					SQL_BOOL, sslenable,
 					SQL_STRING, sslcert,
@@ -747,7 +738,7 @@ sql_host_replace (const char *hostname, const char *portnum,
 
 	/* This host is not yet in the database */
 	err_msg = NULL;
-	ret = sql_non_select (	NULL, NULL, &err_msg, NULL, NULL,
+	ret = sql_non_select (	NULL, NULL, &err_msg, NULL, NULL, NULL,
 				SQL_HOST_NEW,
 				SQL_STRING, hostname,
 				SQL_STRING, portnum,
@@ -756,7 +747,7 @@ sql_host_replace (const char *hostname, const char *portnum,
 				SQL_STRING, (description == NULL)
 						? "": description,
 				SQL_END);
-	if (ret != 0 && ret != ER_DUP_ENTRY) {
+	if (ret != 0) {
 		if (error_func != NULL) {
 			error_func (user_data_error_func, err_msg, ret);
 		}
