@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
+   Copyright (C) 2007-2010 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -42,18 +42,21 @@
 #include <sql_status.h>
 
 #define SQLSTATUS_WORKLOAD_LIST "SELECT DISTINCT workload_date FROM job_status ORDER BY workload_date"
-#define SQLSTATUS_GET "SELECT status,start_time,retry_num,error_msg FROM job_status WHERE workload_date=%d AND job_id=\"%s\""
-#define SQLSTATUS_SET "REPLACE INTO job_status (workload_date,job_id,status,start_time,retry_num,error_msg) VALUES (%d,\"%s\",%d,%u,%d,\"%s\")"
+#define SQLSTATUS_GET "SELECT status,start_time,retry_num,error_msg FROM job_status WHERE workload_date=%d AND job_id=%ld"
+#define SQLSTATUS_UPDATE "UPDATE job_status SET status=%d,start_time=%u,retry_num=%d,error_msg=%s WHERE workload_date=%d AND job_id=%s"
+#define SQLSTATUS_SET "INSERT INTO job_status (workload_date,job_id,status,start_time,retry_num,error_msg) VALUES (%d,%s,%d,%u,%d,%s)"
+#define SQLSTATUS_UPDATE_START_TIME "UPDATE job_status SET start_time=%u WHERE workload_date=%d AND job_id=%ld"
 #define SQLSTATUS_DEL "DELETE FROM job_status WHERE workload_date=%d"
 #define SQLSTATUS_DEL_TABLES "DELETE FROM %s_s WHERE workload_date=%d"
-#define SQLSTATUS_COPY "REPLACE INTO %s_s SELECT *,%d FROM %s"
+#define SQLSTATUS_COPY "INSERT INTO %s_s SELECT *,%d FROM %s"
 #define SQLSTATUS_GET_OLD "SELECT workload_date FROM %s_s WHERE workload_date<%d"
 
 /*
  * List of tables to copy.  These tables will be copied in tables with the
  * same name but suffixed by `_s'
  */
-static char * status_tables[] = {	"job_main",
+static char * status_tables[] = {	"calendars",
+					"job_main",
 					"job_start_limit",
 					"job_max_duration",
 					"job_retries",
@@ -70,7 +73,6 @@ static char * status_tables[] = {	"job_main",
 					"job_environment",
 					"host_environment",
 					"job_host",
-					"calendars",
 					"constraint_file",
 					"jobset_background",
 					"job_icon_default",
@@ -95,26 +97,26 @@ static char * status_tables[] = {	"job_main",
  *     0 --> No error.  rows contains the retrieved workload date and must be
  *           freed by the caller by
  *              lwc_delLL (rows, (void (*)(const void *)) sql_free_row);
- *     1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
  *           with user_data_error_func as its first parameter and the error
  *           message as the second parameter)
- * other --> SQL error (if error_func() is not NULL, it is called with
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
  *           user_data_error_func as its first parameter and the error message
  *           as the second parameter)
  */
-unsigned int
+int
 sql_status_workload_list (lwc_LL **rows,
-			void (*error_func)(void *, const char *, unsigned int),
+			void (*error_func)(void *, const char *, int),
 			void *user_data_error_func)
 {
 	char *err_msg = NULL;
-	unsigned int ret;
+	int ret;
 
 #if HAVE_ASSERT_H
 	assert (rows != NULL);
 #endif
 
-	ret = sql_select (	NULL, NULL, &err_msg, NULL, rows, NULL,
+	ret = sql_select (	NULL, NULL, &err_msg, NULL, rows,
 				SQLSTATUS_WORKLOAD_LIST,
 				SQL_END);
 
@@ -137,36 +139,37 @@ sql_status_workload_list (lwc_LL **rows,
  * Return:
  *     0 --> No error.  status contains the retrieved status and start_time
  *           the associated start time.
- *     1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
  *           with user_data_error_func as its first parameter and the error
  *           message as the second parameter)
- * other --> SQL error (if error_func() is not NULL, it is called with
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
  *           user_data_error_func as its first parameter and the error message
  *           as the second parameter)
  */
-unsigned int
+int
 sql_status_get (int workload_date,
-		const char *job_id,
+		unsigned long long int job_id,
 		int *status,
 		long int *start_time,
 		int *retry_num,
 		char **message,
-		void (*error_func)(void *, const char *, unsigned int),
+		void (*error_func)(void *, const char *, int),
 		void *user_data_error_func)
 {
 	char *err_msg = NULL;
 	lwc_LL *rows;
-	unsigned int ret;
-	char **row, *s;
+	int ret;
+	char *s;
+	row_item_t *row;
 
 #if HAVE_ASSERT_H
-	assert (job_id != NULL && status != NULL);
+	assert (status != NULL);
 #endif
 
-	ret = sql_select (	NULL, NULL, &err_msg, NULL, &rows, NULL,
+	ret = sql_select (	NULL, NULL, &err_msg, NULL, &rows,
 				SQLSTATUS_GET,
 				SQL_INT, (long int)workload_date,
-				SQL_STRING, job_id,
+				SQL_INT, (long int)job_id,
 				SQL_END);
 
 	if (ret != 0) {
@@ -179,15 +182,18 @@ sql_status_get (int workload_date,
 		return ret;
 	}
 
-	row = (char **) lwc_delStartLL (rows);
+	row = (row_item_t *) lwc_delStartLL (rows);
 
 	/* Message */
 	if (message != NULL) {
-		if (row == NULL || row[3] == NULL || row[3][0] == '\0') {
+		if (	   row == NULL
+			|| row[3].value_string == NULL
+			|| row[3].value_string[0] == '\0')
+		{
 			*message = NULL;
 		}
 		else {
-			s = (char *) malloc (schedwi_strlen (row[3]) + 1);
+			s = (char *) malloc (row[3].len + 1);
 			if (s == NULL) {
 				sql_free_row (row);
 				lwc_delLL (rows,
@@ -197,23 +203,27 @@ sql_status_get (int workload_date,
 						_("Memory allocation error"),
 						0);
 				}
-				return 1;
+				return -1;
 			}
-			strcpy (s, row[3]);
+			strcpy (s, row[3].value_string);
 			*message = s;
 		}
 	}
 
 	/* Status */
-	*status = (row == NULL) ? 0 : atoi (row[0]);
+	*status = (row == NULL) ? 0 : (int)sql_row_item2ll (&(row[0]));
 
-	/* Status time */
+	/* Start time */
 	if (start_time != NULL) {
-		*start_time = (row == NULL) ? 0 : strtol (row[1], NULL, 0);
+		*start_time = (row == NULL) ? 0
+				: (long int)sql_row_item2ll (&(row[1]));
 	}
 
 	/* Current retry number */
-	*retry_num = (row == NULL) ? 0 : atoi (row[2]);
+	if (retry_num != NULL) {
+		*retry_num = (row == NULL) ? 0
+				: (int)sql_row_item2ll (&(row[2]));
+	}
 
 	sql_free_row (row);
 	lwc_delLL (rows, (void (*)(const void *))sql_free_row);
@@ -226,32 +236,59 @@ sql_status_get (int workload_date,
  *
  * Return:
  *     0 --> No error
- *     1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
  *           with user_data_error_func as its first parameter and the error
  *           message as the second parameter)
- * other --> SQL error (if error_func() is not NULL, it is called with
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
  *           user_data_error_func as its first parameter and the error message
  *           as the second parameter)
  */
-unsigned int
+int
 sql_status_set (int workload_date,
 		const char *job_id,
 		int status,
 		unsigned int start_time,
 		int retry_num,
 		const char *error_message,
-		void (*error_func)(void *, const char *, unsigned int),
+		void (*error_func)(void *, const char *, int),
 		void *user_data_error_func)
 {
 	char *err_msg = NULL;
-	unsigned int ret;
+	int ret;
+	unsigned long long int nb = 0;
 
 #if HAVE_ASSERT_H
 	assert (job_id != NULL);
 #endif
 
+	ret = sql_non_select (  NULL, NULL, &err_msg, NULL, &nb, NULL,
+				SQLSTATUS_UPDATE,
+				SQL_INT, (long int)status,
+				SQL_INT, (long int)start_time,
+				SQL_INT, (long int)retry_num,
+				SQL_STRING, (error_message != NULL)
+							? error_message: "",
+				SQL_INT, (long int)workload_date,
+				SQL_STRING, job_id,
+				SQL_END);
+	if (ret != 0) {
+		if (error_func != NULL) {
+			error_func (user_data_error_func, err_msg, ret);
+		}
+		if (err_msg != NULL) {
+			free (err_msg);
+		}
+		return ret;
+	}
 
-	ret = sql_non_select (  NULL, NULL, &err_msg, NULL, NULL,
+	/* Update successful.  There was already a row for this job */
+	if (nb != 0) {
+		return 0;
+	}
+
+	/* There is not yet a status row for this job.  Add one */
+	err_msg = NULL;
+	ret = sql_non_select (  NULL, NULL, &err_msg, NULL, NULL, NULL,
 				SQLSTATUS_SET,
 				SQL_INT, (long int)workload_date,
 				SQL_STRING, job_id,
@@ -270,6 +307,49 @@ sql_status_set (int workload_date,
 		}
 		return ret;
 	}
+
+	return 0;
+}
+
+
+/*
+ * Change the start time of a running job/jobset
+ *
+ * Return:
+ *     0 --> No error
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *           with user_data_error_func as its first parameter and the error
+ *           message as the second parameter)
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
+ *           user_data_error_func as its first parameter and the error message
+ *           as the second parameter)
+ */
+int
+sql_status_adjust_start_time (int workload_date,
+			unsigned long long int job_id,
+			unsigned int start_time,
+			void (*error_func)(void *, const char *, int),
+			void *user_data_error_func)
+{
+	char *err_msg = NULL;
+	int ret;
+
+	ret = sql_non_select (  NULL, NULL, &err_msg, NULL, NULL, NULL,
+				SQLSTATUS_UPDATE_START_TIME,
+				SQL_INT, (long int)start_time,
+				SQL_INT, (long int)workload_date,
+				SQL_INT, (long int)job_id,
+				SQL_END);
+	if (ret != 0) {
+		if (error_func != NULL) {
+			error_func (user_data_error_func, err_msg, ret);
+		}
+		if (err_msg != NULL) {
+			free (err_msg);
+		}
+		return ret;
+	}
+
 	return 0;
 }
 
@@ -279,20 +359,21 @@ sql_status_set (int workload_date,
  *
  * Return:
  *     0 --> No error
- *     1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
  *           with user_data_error_func as its first parameter and the error
  *           message as the second parameter)
- * other --> SQL error (if error_func() is not NULL, it is called with
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
  *           user_data_error_func as its first parameter and the error message
  *           as the second parameter)
  */
-unsigned int
+int
 sql_status_delete (int workload_date,
-		void (*error_func)(void *, const char *, unsigned int),
+		void (*error_func)(void *, const char *, int),
 		void *user_data_error_func)
 {
 	char *err_msg = NULL;
-	unsigned int ret, err, i;
+	unsigned int err, i;
+	int ret;
 	char *buff;
 	unsigned int buff_len;
 
@@ -300,7 +381,7 @@ sql_status_delete (int workload_date,
 	buff_len = 0;
 
 	/* First, delete the entry in the job_status table */
-	ret = sql_non_select (  &buff, &buff_len, &err_msg, NULL, NULL,
+	ret = sql_non_select (  &buff, &buff_len, &err_msg, NULL, NULL, NULL,
 				SQLSTATUS_DEL,
 				SQL_INT, (long int)workload_date,
 				SQL_END);
@@ -318,9 +399,12 @@ sql_status_delete (int workload_date,
 	}
 
 	/* And then delete the entries for all the other tables */
+	for (i = 0; status_tables[i] != NULL; i++);
 	err = 0;
-	for (i = 0; status_tables[i] != NULL; i++) {
+	do {
+		i--;
 		ret = sql_non_select (  &buff, &buff_len, &err_msg, NULL, NULL,
+					NULL,
 					SQLSTATUS_DEL_TABLES,
 					SQL_STRING_NON_ESCAPE,
 							status_tables[i],
@@ -336,7 +420,7 @@ sql_status_delete (int workload_date,
 				free (err_msg);
 			}
 		}
-	}
+	} while (i > 0);
 
 	if (buff != NULL) {
 		free (buff);
@@ -358,26 +442,27 @@ sql_status_delete (int workload_date,
  *    - The workload date
  *
  * Return:
+ *     2 --> Old workloads found
  *     0 --> No error.
- *     1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
  *           with user_data_error_func as its first parameter and the error
  *           message as the second parameter)
- *     2 --> Old workloads found
- * other --> SQL error (if error_func() is not NULL, it is called with
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
  *           user_data_error_func as its first parameter and the error message
  *           as the second parameter)
  */
-unsigned int
+int
 sql_status_get_old (	int workload_date,
-			int (*callback)(void *, const char *, const char *),
+			int (*callback)(void *, const char *, int),
 			void *user_data,
-			void (*error_func)(void *, const char *, unsigned int),
+			void (*error_func)(void *, const char *, int),
 			void *user_data_error_func)
 {
 	char *err_msg = NULL;
 	lwc_LL *rows;
-	char **row;
-	unsigned int ret, i, global_ret;
+	row_item_t *row;
+	unsigned int i;
+	int ret, global_ret;
 	char *buff;
 	unsigned int buff_len;
 
@@ -389,8 +474,7 @@ sql_status_get_old (	int workload_date,
 	buff_len = 0;
 	global_ret = 0;
 	for (i = 0; status_tables[i] != NULL; i++) {
-		ret = sql_select (&buff, &buff_len, &err_msg, NULL,
-				&rows, NULL,
+		ret = sql_select (&buff, &buff_len, &err_msg, NULL, &rows,
 				SQLSTATUS_GET_OLD,
 				SQL_STRING_NON_ESCAPE, status_tables[i],
 				SQL_INT, (long int)workload_date,
@@ -409,11 +493,12 @@ sql_status_get_old (	int workload_date,
 			return ret;
 		}
 
-		while ((row = (char **) lwc_delStartLL (rows)) != NULL) {
+		while ((row = (row_item_t *) lwc_delStartLL (rows)) != NULL) {
 			/*
 			 * row[0] --> workload_date
 			 */
-			callback (user_data, status_tables[i], row[0]);
+			callback (	user_data, status_tables[i],
+					(int)sql_row_item2ll (&(row[0])));
 			sql_free_row (row);
 			global_ret = 2;
 		}
@@ -433,29 +518,37 @@ sql_status_get_old (	int workload_date,
  *
  * Return:
  *     0 --> No error
- *     1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
  *           with user_data_error_func as its first parameter and the error
  *           message as the second parameter)
- * other --> SQL error (if error_func() is not NULL, it is called with
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
  *           user_data_error_func as its first parameter and the error message
  *           as the second parameter)
  */
-unsigned int
+int
 sql_status_copy_to_workload (int workload_date,
-		void (*error_func)(void *, const char *, unsigned int),
+		void (*error_func)(void *, const char *, int),
 		void *user_data_error_func)
 {
 	char *err_msg = NULL;
-	unsigned int ret, i;
+	unsigned int i;
+	int ret;
 	char *buff;
 	unsigned int buff_len;
 
 	buff = NULL;
 	buff_len = 0;
 
+	/*
+	 * First, delete the workload from all the tables if it already
+	 * exists. It should not, but to be on the safe side...
+	 */
+	sql_status_delete (workload_date, NULL, NULL);
+
 	/* Copy all the tables */
 	for (i = 0; status_tables[i] != NULL; i++) {
 		ret = sql_non_select (  &buff, &buff_len, &err_msg, NULL, NULL,
+					NULL,
 					SQLSTATUS_COPY,
 					SQL_STRING_NON_ESCAPE,
 							status_tables[i],
@@ -474,7 +567,7 @@ sql_status_copy_to_workload (int workload_date,
 			if (buff != NULL) {
 				free (buff);
 			}
-			
+
 			/* Try to remove the successfully copied tables */
 			sql_status_delete (workload_date, NULL, NULL);
 			return ret;
