@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
+   Copyright (C) 2007-2010 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -47,7 +47,6 @@ extern int errno;
 #include <utils.h>
 #include <lwc_log.h>
 #include <net_utils.h>
-#include <net_module_name.h>
 #include <sql_common.h>
 #include <sql_hierarchy.h>
 #include <sql_hosts.h>
@@ -60,14 +59,14 @@ extern int errno;
  */
 static void
 sql_host_get_main_row_error_logger (	void *data, const char *msg,
-					unsigned int err_code)
+					int err_code)
 {
 	if (msg != NULL) {
 		lwc_writeLog (LOG_ERR, msg);
 	}
 	else {
 		lwc_writeLog (LOG_ERR,
-		_("Database error while retrieving the client host details"));
+		_("Database error while retrieving the agent host details"));
 	}
 }
 
@@ -82,7 +81,7 @@ sql_host_get_main_row_error_logger (	void *data, const char *msg,
  *   1 --> Stop the sql function.  No arguments for this job
  */
 static int
-argument_callback (void *obj, char **row, unsigned long int *row_len)
+argument_callback (void *obj, row_item_t *row)
 {
 	startjob_request_ptr request = (startjob_request_ptr) obj;
 
@@ -96,16 +95,16 @@ argument_callback (void *obj, char **row, unsigned long int *row_len)
 
 	/*
 	 * row[0] --> Position
-	 * row[1] --> Value
+	 * row[1] --> Argument
 	 */
 
-	if (row[0][0] == '-') {
+	if (sql_row_item2ll (&(row[0])) == -1) {
 		/* No argument for the command */
 		return 1;
 	}
 	return startjob_request_add_argument (request,
-					row[0], (unsigned int)row_len[0],
-					row[1], (unsigned int)row_len[1]);
+					"1", 1, /* Ignored */
+					row[1].value_string, row[1].len);
 }
 
 
@@ -114,12 +113,12 @@ argument_callback (void *obj, char **row, unsigned long int *row_len)
  *
  * Return:
  *   0 --> No error.  Job successfully started on the remote client.  If not
- *         NULL, out_message contains a reply message sent by the client.  It
- *         may be NULL if the client didn't send back a message.  out_message
+ *         NULL, out_message contains a reply message sent by the agent.  It
+ *         may be NULL if the agent didn't send back a message.  out_message
  *         must be freed by the caller by free()
- *   1 --> The client cannot start the job.  If not NULL, out_message
- *         contains a reply message sent by the client.  It may be NULL if the
- *         client didn't send back a message.  out_message must be freed by
+ *   1 --> The agent cannot start the job.  If not NULL, out_message
+ *         contains a reply message sent by the agent.  It may be NULL if the
+ *         agent didn't send back a message.  out_message must be freed by
  *         the caller by free()
  *   2 --> The job does not have an associated command.  out_message is not
  *         changed
@@ -133,19 +132,16 @@ startjob (	int workload_date, unsigned long long int job_id,
 		char **out_message)
 {
 	lwc_LL *hierarchy_list;
-	char *host_id_str, *value, *module;
-	char *err_msg = NULL;
-	unsigned long int value_len;
-	unsigned int len, ret;
-	int module_len, ret_read;
+	row_item_t value;
+	unsigned long long int host_id;
+	char *s, *err_msg = NULL;
+	int ret;
 	startjob_request_ptr request;
-	const char *req;
-	char **host_details;
-	unsigned long int *host_details_len;
+	row_item_t *host_details;
 	schedwi_BIO *b;
 
 	if (request_job_init (	workload_date, job_id,
-				&host_id_str, &request, &hierarchy_list) != 0)
+				&host_id, &request, &hierarchy_list) != 0)
 	{
 		if (out_message != NULL) {
 			err_msg =
@@ -162,9 +158,8 @@ startjob (	int workload_date, unsigned long long int job_id,
 	/* Get the command */
 	if (get_job_parameter (	workload_date, hierarchy_list,
 				"job_command_s", "command",
-				&value, &value_len, &err_msg) != 0)
+				&value, &err_msg) != 0)
 	{
-		free (host_id_str);
 		lwc_delLL (hierarchy_list, (void (*)(const void *))free);
 		startjob_request_destroy (request);
 		if (err_msg != NULL) {
@@ -193,15 +188,17 @@ startjob (	int workload_date, unsigned long long int job_id,
 		return -1;
 	}
 
-	if (	   value != NULL
-		&& value[0] != '\0'
-		&& (value[0] != '-' || value[1] != '\0'))
+	if (	   value.type != RES_END
+		&& value.value_string != NULL
+		&& value.value_string[0] != '\0'
+		&& (	   value.value_string[0] != '-'
+			|| value.value_string[1] != '\0'))
 	{
 		if (startjob_request_add_command (request,
-						value, value_len) != 0)
+						value.value_string,
+						value.len) != 0)
 		{
-			free (value);
-			free (host_id_str);
+			free (value.value_string);
 			lwc_delLL (	hierarchy_list,
 					(void (*)(const void *))free);
 			startjob_request_destroy (request);
@@ -216,14 +213,13 @@ startjob (	int workload_date, unsigned long long int job_id,
 			}
 			return -1;
 		}
-		free (value);
+		free (value.value_string);
 	}
 	else {
 		/* No command, the job is then considered completed */
-		if (value != NULL) {
-			free (value);
+		if (value.type != RES_END && value.value_string != NULL) {
+			free (value.value_string);
 		}
-		free (host_id_str);
 		lwc_delLL (hierarchy_list, (void (*)(const void *))free);
 		startjob_request_destroy (request);
 		return 2;
@@ -232,9 +228,8 @@ startjob (	int workload_date, unsigned long long int job_id,
 	/* Get the username */
 	if (get_job_parameter (	workload_date, hierarchy_list,
 				"job_username_s", "username",
-				&value, &value_len, &err_msg) != 0)
+				&value, &err_msg) != 0)
 	{
-		free (host_id_str);
 		lwc_delLL (hierarchy_list, (void (*)(const void *))free);
 		startjob_request_destroy (request);
 		if (err_msg != NULL) {
@@ -263,12 +258,14 @@ startjob (	int workload_date, unsigned long long int job_id,
 		return -1;
 	}
 
-	if (value != NULL) {
+	if (value.type != RES_END) {
 		if (startjob_request_add_username (request,
-						value, value_len) != 0)
+						value.value_string,
+						value.len) != 0)
 		{
-			free (value);
-			free (host_id_str);
+			if (value.value_string != NULL) {
+				free (value.value_string);
+			}
 			lwc_delLL (	hierarchy_list,
 					(void (*)(const void *))free);
 			startjob_request_destroy (request);
@@ -283,10 +280,11 @@ startjob (	int workload_date, unsigned long long int job_id,
 			}
 			return -1;
 		}
-		free (value);
+		if (value.value_string != NULL) {
+			free (value.value_string);
+		}
 	}
 	else {
-		free (host_id_str);
 		lwc_delLL (hierarchy_list, (void (*)(const void *))free);
 		startjob_request_destroy (request);
 		lwc_writeLog (	LOG_CRIT,
@@ -307,9 +305,8 @@ startjob (	int workload_date, unsigned long long int job_id,
 	/* Get the output file name */
 	if (get_job_parameter (	workload_date, hierarchy_list,
 				"job_file_out_s", "file_out",
-				&value, &value_len, &err_msg) != 0)
+				&value, &err_msg) != 0)
 	{
-		free (host_id_str);
 		lwc_delLL (hierarchy_list, (void (*)(const void *))free);
 		startjob_request_destroy (request);
 		if (err_msg != NULL) {
@@ -338,12 +335,14 @@ startjob (	int workload_date, unsigned long long int job_id,
 		return -1;
 	}
 
-	if (value != NULL) {
+	if (value.type != RES_END) {
 		if (startjob_request_add_fileout (request,
-						value, value_len) != 0)
+						value.value_string,
+						value.len) != 0)
 		{
-			free (value);
-			free (host_id_str);
+			if (value.value_string != NULL) {
+				free (value.value_string);
+			}
 			lwc_delLL (	hierarchy_list,
 					(void (*)(const void *))free);
 			startjob_request_destroy (request);
@@ -358,15 +357,16 @@ startjob (	int workload_date, unsigned long long int job_id,
 			}
 			return -1;
 		}
-		free (value);
+		if (value.value_string != NULL) {
+			free (value.value_string);
+		}
 	}
 
 	/* Get the error file name */
 	if (get_job_parameter (	workload_date, hierarchy_list,
 				"job_file_err_s", "file_err",
-				&value, &value_len, &err_msg) != 0)
+				&value, &err_msg) != 0)
 	{
-		free (host_id_str);
 		lwc_delLL (hierarchy_list, (void (*)(const void *))free);
 		startjob_request_destroy (request);
 		if (err_msg != NULL) {
@@ -395,12 +395,14 @@ startjob (	int workload_date, unsigned long long int job_id,
 		return -1;
 	}
 
-	if (value != NULL) {
+	if (value.type != RES_END) {
 		if (startjob_request_add_filerr (request,
-						value, value_len) != 0)
+						value.value_string,
+						value.len) != 0)
 		{
-			free (value);
-			free (host_id_str);
+			if (value.value_string != NULL) {
+				free (value.value_string);
+			}
 			lwc_delLL (	hierarchy_list,
 					(void (*)(const void *))free);
 			startjob_request_destroy (request);
@@ -415,15 +417,16 @@ startjob (	int workload_date, unsigned long long int job_id,
 			}
 			return -1;
 		}
-		free (value);
+		if (value.value_string != NULL) {
+			free (value.value_string);
+		}
 	}
 
 	/* Get the load environment flag */
 	if (get_job_parameter (	workload_date, hierarchy_list,
 				"job_loadenv_s", "loadenv",
-				&value, &value_len, &err_msg) != 0)
+				&value, &err_msg) != 0)
 	{
-		free (host_id_str);
 		lwc_delLL (hierarchy_list, (void (*)(const void *))free);
 		startjob_request_destroy (request);
 		if (err_msg != NULL) {
@@ -452,12 +455,9 @@ startjob (	int workload_date, unsigned long long int job_id,
 		return -1;
 	}
 
-	if (value != NULL) {
-		if (startjob_request_add_loadenv (request,
-						value, value_len) != 0)
-		{
-			free (value);
-			free (host_id_str);
+	if (value.type != RES_END) {
+		s = (sql_row_item2ll(&value) == 0) ? "0" : "1";
+		if (startjob_request_add_loadenv (request, s, 1) != 0) {
 			lwc_delLL (	hierarchy_list,
 					(void (*)(const void *))free);
 			startjob_request_destroy (request);
@@ -472,15 +472,14 @@ startjob (	int workload_date, unsigned long long int job_id,
 			}
 			return -1;
 		}
-		free (value);
 	}
 
 	/* Get the command arguments */
 	ret = get_job_parameters_multi (workload_date, hierarchy_list,
-				"job_arguments_s", "position,argument", NULL,
+				"job_arguments_s", "position,argument",
+				"position",
 				argument_callback, request, &err_msg);
-	if (ret != 0 && ret != 3) {
-		free (host_id_str);
+	if (ret != 0 && ret != 2) {
 		lwc_delLL (hierarchy_list, (void (*)(const void *))free);
 		startjob_request_destroy (request);
 		if (err_msg != NULL) {
@@ -512,11 +511,9 @@ startjob (	int workload_date, unsigned long long int job_id,
 	lwc_delLL (hierarchy_list, (void (*)(const void *))free);
 
 	/* Get the host details */
-	ret = sql_host_get_main_row (	host_id_str,
-					&host_details, &host_details_len,
+	ret= sql_host_get_main_row (host_id, &host_details,
 					sql_host_get_main_row_error_logger,
 					NULL);
-	free (host_id_str);
 	if (ret != 0) {
 		startjob_request_destroy (request);
 		if (out_message != NULL) {
@@ -531,36 +528,14 @@ startjob (	int workload_date, unsigned long long int job_id,
 		return -1;
 	}
 
-	/* Compose the module name */
-	module = NULL;
-	module_len = build_module_name ("runjob-0.1", &module);
-	if (module_len < 0) {
-		sql_free_row (host_details);
-		free (host_details_len);
-		startjob_request_destroy (request);
-		lwc_writeLog (LOG_CRIT, _("Memory allocation error"));
-		if (out_message != NULL) {
-			err_msg =
-				_("Server error: memory allocation error");
-			*out_message = (char *)malloc (
-						schedwi_strlen (err_msg) + 1);
-			if (*out_message != NULL) {
-				strcpy (*out_message, err_msg);
-			}
-		}
-		return -1;
-	}
-
-	/* Establish the connection to the client */
-	b = net_client (host_details[2],  /* TCP port */
-			host_details[1],  /* Client host name */
-			(host_details[3][0] == '0') ? 0: 1, /* SSL */
-			host_details[4],  /* Certificate */
-			(unsigned int)(host_details_len[4]));
-	free (host_details_len);
+	/* Establish the connection to the agent */
+	b = net_client (host_details[2].value_string,  /* TCP port */
+			host_details[1].value_string,  /* Agent host name */
+			(char)sql_row_item2ll (&(host_details[3])),
+			host_details[4].value_string,  /* Certificate */
+			(unsigned int)(host_details[4].len));
 	sql_free_row (host_details);
 	if (b == NULL) {
-		free (module);
 		startjob_request_destroy (request);
 		if (out_message != NULL) {
 			err_msg =
@@ -574,16 +549,9 @@ _("Server error: cannot establish the connection with the remote agent");
 		return -1;
 	}
 
-	/* Get the request string */
-	len = get_request_string (request, &req);
-
 	/* Send the request */
-	if (	   net_write (b, module, (int)module_len) != 0
-		|| net_write (b, req, (int)len) != 0
-		|| net_write_eof (b) != 0)
-	{
+	if (net_write_request (request, b, "runjob") != 0) {
 		net_close (b);
-		free (module);
 		startjob_request_destroy (request);
 		if (out_message != NULL) {
 			err_msg =
@@ -596,13 +564,12 @@ _("Server error: cannot establish the connection with the remote agent");
 		}
 		return -1;
 	}
-	free (module);
 	startjob_request_destroy (request);
 
 	/* Read the result */
-	ret_read = net_read_result (b, out_message);
+	ret = net_read_result (b, out_message);
 	net_close (b);
-	return ret_read;
+	return ret;
 }
 
 /*-----------------============== End Of File ==============-----------------*/
