@@ -26,6 +26,8 @@
 #define _STARTJOB_REQUEST_H
 
 #include <schedwi.h>
+#include <lwc_linkedlist.h>
+#include <net_utils.h>
 
 /* Allow the use in C++ code.  */
 #ifdef __cplusplus
@@ -33,8 +35,9 @@ extern "C" {
 #endif
 
 struct _startjob_request {
-	char *req;
-	unsigned int idx, size;
+	lwc_LL *parameters;
+	lwc_LL *env;
+	lwc_LL *args;
 };
 typedef struct _startjob_request startjob_request;
 typedef struct _startjob_request *startjob_request_ptr;
@@ -48,23 +51,25 @@ extern int startjob_request_add_argument OF((startjob_request_ptr ptr,
 extern int startjob_request_add_environment OF((startjob_request_ptr ptr,
 					const char *pos,
 					const char *key, const char *value));
-extern int startjob_request_add_parameter OF((startjob_request_ptr ptr,
+extern int startjob_request_add_parameter_string OF((startjob_request_ptr ptr,
 					const char *name,
 					const char *val,
 					unsigned int len_val));
+extern int startjob_request_add_parameter_bool OF((startjob_request_ptr ptr,
+					const char *name, char val));
 extern int startjob_request_add_jobid OF((startjob_request_ptr ptr,
 					const char *workload,
 					const char *job_id_str,
 					unsigned int len_job_id_str));
-extern unsigned int get_request_string OF((startjob_request_ptr ptr,
-					const char **req));
+extern int net_write_request OF((startjob_request_ptr ptr,
+				schedwi_BIO *b, const char *module_name));
 
-#define startjob_request_add_command(X,VAL,LEN) startjob_request_add_parameter(X,"CMD",VAL,LEN)
-#define startjob_request_add_username(X,VAL,LEN) startjob_request_add_parameter(X,"USR",VAL,LEN)
-#define startjob_request_add_fileout(X,VAL,LEN) startjob_request_add_parameter(X,"OUT",VAL,LEN)
-#define startjob_request_add_filerr(X,VAL,LEN) startjob_request_add_parameter(X,"ERR",VAL,LEN)
-#define startjob_request_add_loadenv(X,VAL,LEN) startjob_request_add_parameter(X,"LDE",VAL,LEN)
-#define startjob_request_add_filename(X,VAL,LEN) startjob_request_add_parameter(X,"FIL",VAL,LEN)
+#define startjob_request_add_command(X,VAL,LEN) startjob_request_add_parameter_string(X,"command",VAL,LEN)
+#define startjob_request_add_username(X,VAL,LEN) startjob_request_add_parameter_string(X,"user",VAL,LEN)
+#define startjob_request_add_fileout(X,VAL,LEN) startjob_request_add_parameter_string(X,"stdout",VAL,LEN)
+#define startjob_request_add_filerr(X,VAL,LEN) startjob_request_add_parameter_string(X,"stderr",VAL,LEN)
+#define startjob_request_add_loadenv(X,VAL,LEN) startjob_request_add_parameter_bool(X,"user env",((VAL)[0]=='0')?0:1)
+#define startjob_request_add_filename(X,VAL,LEN) startjob_request_add_parameter_string(X,"file",VAL,LEN)
 
 
 #ifdef __cplusplus
