@@ -37,7 +37,7 @@
 static void
 sigusr2_foo (int sig)
 {
-                return;
+	return;
 }
 
 
@@ -50,17 +50,14 @@ thread_init ()
 	sigset_t mask;
 	struct sigaction sa;
 
-	/* 
+	/*
 	 * Be sure the thread can be cancelled only
 	 * at cancellation points
 	 */
 	pthread_setcancelstate (PTHREAD_CANCEL_ENABLE, NULL);
 	pthread_setcanceltype (PTHREAD_CANCEL_DEFERRED, NULL);
 
-	/* SQL thread initialization */
-	sql_thread_init ();
-
-	/* 
+	/*
 	 * SIGUSR2 handler
 	 * To cancel a thread the following functions are called in this order:
 	 *     pthread_cancel (<thread_id>)
@@ -91,7 +88,7 @@ thread_init ()
 void
 thread_clean (void *foo)
 {
-	sql_thread_end ();
+	return;
 }
 
 /*-----------------============== End Of File ==============-----------------*/
