@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
+   Copyright (C) 2007-2010 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -40,6 +40,8 @@ extern void close_pid_file OF((const char *file_name));
 extern unsigned int copy_ulltostr OF((unsigned long long int v, char *str));
 extern int change_user OF((const char *name));
 extern int change_group OF((const char *name));
+extern char *escape_string OF((const char *str, unsigned int str_len,
+				char add_quotes));
 
 #ifdef __cplusplus
 }
